
#include <linux/config.h>
#include <linux/module.h>
#include <stdarg.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/segment.h>
#include <asm/byteorder.h>
#include <linux/spinlock.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>
#include <linux/blk.h>
#include <linux/tqueue.h>
#include <../drivers/scsi/scsi.h>
#include <../drivers/scsi/hosts.h>
#include "qla4x_fw.h"
#include "qla4xxx.h"
#include "ql4print.h"

extern Scsi_Host_Template driver_template;
			  
//#define QLP0	0x00000001	// Messages to always display (We use printk instead, in Linux)
//#define QLP1	0x00000002  // Warnings and potential errors
//#define QLP2	0x00000004  // Significant events error messages
//#define QLP3	0x00000008  // Function trace messages
//#define QLP4	0x00000010  // IOCTL trace messages
//#define QLP5	0x00000020  // I/O & Request/Response queue trace messages
//#define QLP6	0x00000040  // Watchdog messages (current state)
//#define QLP7	0x00000080  // Initialization
//#define QLP8	0x00000100  // Internal command queue traces
//#define QLP9	0x00000200  // Register accesses
//#define QLP10	0x00000400  // Extra Debug messages (dump buffers)
//#define QLP11	0x00000800  // Mailbox & ISR Details
//#define QLP12	0x00001000	// Enter/Leave routine messages
//#define QLP13 0x00002000  // Display data for Inquiry, TUR, ReqSense, RptLuns

#ifdef DEBUG
uint32_t ql_dbg_level = QLP2|QLP7;	// see defines in ql4print.h

#if EXTENDED_DBG_ON_ERROR 
uint32_t ql_saved_dbg_level = QLP2|QLP7;

/*
 * This routine set an extended debug print level.  Used to display more detailed print messages
 * when an error occurs.
 */
inline void qla4000_set_extended_debug_level(void)
{
		uint32_t ext_dbg_level = QLP1|QLP2|QLP5|QLP7;
		qla4000_set_debug_level(ext_dbg_level);
		QL4PRINT(QLP2, printk("qla4000_set_extended_debug_level: 0x%x\n", ext_dbg_level));
}

/*
 * This routine restores the original debug print level.  Used after error recovery has completed.
 */
inline void qla4000_rst_extended_debug_level(void)
{
		QL4PRINT(QLP2, printk("qla4000_rst_extended_debug_level: 0x%x\n", ql_saved_dbg_level));
		qla4000_set_debug_level(ql_saved_dbg_level); 
}
#endif

inline uint8_t qla4000_get_debug_level(uint32_t *dbg_level)
{
	*dbg_level = ql_dbg_level;
	barrier();
	return (QLA_SUCCESS);
}

inline uint8_t qla4000_set_debug_level(uint32_t dbg_level)
{
	ql_dbg_level = dbg_level;
	barrier();
	return (QLA_SUCCESS);
}


/****************************************************************************/
/*                      Debug Print Routines                          	*/
/****************************************************************************/

void
qla4000_dump_bytes(uint32_t dbg_mask, void *buffer, uint32_t size)
{
	uint32_t i;
	uint8_t  *data = (uint8_t *)buffer;

	if ((ql_dbg_level & dbg_mask) != 0)
	{
		printk("        0  1  2  3  4  5  6  7  - 8  9  A  B  C  D  E  F\n");
		printk("---------------------------------------------------------\n");

		for (i = 0; i < size; i++, data++)
		{
			if (i % 0x10 == 0)
			{
				printk("%04X:  %02X", i, *data);
			}
			else if (i % 0x10 == 0x08)
			{
				printk(" - %02X", *data);
			}
			else if (i % 0x10 == 0xF)
			{
				printk(" %02X\n", *data);
			}
			else
			{
				printk(" %02X", *data);
			}
		}

		if ((i != 0) && (i % 0x10))
		{
			printk("\n");
		}
	}
}

void
qla4000_dump_words(uint32_t dbg_mask, void *buffer, uint32_t size)
{
	uint16_t *data = (uint16_t *)buffer;   
	uint32_t i;

	if ((ql_dbg_level & dbg_mask) != 0)
	{
		for (i = 0; i < size; i+=2, data++)
		{
			if (i % 0x10 == 0)
			{
				printk("%04X:  %04X", i, *data);
			}
			else if (i % 0x10 == 0x08)
			{
				printk(" - %04X", *data);
			}
			else if (i % 0x10 == 0x0E)
			{
				printk(" %04X\n", *data);
			}
			else
			{
				printk(" %04X", *data);
			}
		}
		if ((i != 0) && (i % 0x10 != 0))
		{
			printk("\n");
		}
	}
}

void
qla4000_dump_dwords(uint32_t dbg_mask, void *buffer, uint32_t size)
{
	uint32_t *data = (uint32_t *)buffer;   
	uint32_t i;

	if ((ql_dbg_level & dbg_mask) != 0)
	{
		for (i = 0; i < size; i+=4, data++)
		{
			if (i % 0x10 == 0)
			{
				printk("%04X:  %08X", i, *data);
			}
			else if (i % 0x10 == 0x08)
			{
				printk(" - %08X", *data);
			}
			else if (i % 0x10 == 0x0C)
			{
				printk(" %08X\n", *data);
			}
			else
			{
				printk(" %08X", *data);
			}
		}
		if ((i != 0) && (i % 0x10 != 0))
		{
			printk("\n");
		}
	}
}

void
qla4000_dump_registers(uint32_t dbg_mask, scsi_qla_host_t *ha)
{
	if ((ql_dbg_level & dbg_mask) != 0)
	{
		ISP_REG_T *regs = ha->base_address;
		unsigned long flags = 0;
		uint8_t  i;

		spin_lock_irqsave(&ha->hardware_lock, flags);
		printk("0x%02X RequestQueueIn  = 0x%08lX\n", 
		       OFFSET(regs->requestQueueInPointer), regs->requestQueueInPointer);
		printk("0x%02X ResponseQueueIn = 0x%08lX\n", 
		       OFFSET(regs->responseQueueInPointer), regs->responseQueueInPointer);
		printk("0x%02X FastPostHandle  = 0x%08lX\n", 
		       OFFSET(regs->fastPostHandle), regs->fastPostHandle);
		printk("0x%02X MailBoxIn0      = 0x%08lX\n", 
		       OFFSET(regs->mailboxIn0), regs->mailboxIn0);
		printk("0x%02X IntrStatusIn    = 0x%08lX\n", 
		       OFFSET(regs->interruptStatusIn), regs->interruptStatusIn);
		printk("0x%02X IntrMaskIn      = 0x%08lX\n", 
		       OFFSET(regs->interruptMaskIn), regs->interruptMaskIn);
		printk("0x%02X MailBoxOut0     = 0x%08lX\n", 
		       OFFSET(regs->mailboxOut0), regs->mailboxOut0);
		printk("0x%02X IntrStatusOut   = 0x%08lX\n", 
		       OFFSET(regs->interruptStatusOut), regs->interruptStatusOut);
		printk("0x%02X IntrMaskOut     = 0x%08lX\n", 
		       OFFSET(regs->interruptMaskOut), regs->interruptMaskOut);
		for (i=0; i<MBOX_REG_COUNT/2; i++)
		{
			printk("0x%02X MailBoxIn[%d]    = 0x%08lX\n", 
			       OFFSET(regs->mailboxIn[i]), i+1, regs->mailboxIn[i]);
		}
		for (i=0; i<MBOX_REG_COUNT/2; i++)
		{
			printk("0x%02X MailBoxOut[%d]   = 0x%08lX\n", 
			       OFFSET(regs->mailboxOut[i]), i+1, regs->mailboxOut[i]);
		}
		printk("0x%02X RequestQueueOut = 0x%08lX\n", 
		       OFFSET(regs->requestQueueOutPointer), regs->requestQueueOutPointer);
		printk("0x%02X ResponseQueueOut= 0x%08lX\n", 
		       OFFSET(regs->responseQueueOutPointer), regs->responseQueueOutPointer);
		spin_unlock_irqrestore(&ha->hardware_lock, flags);
	}
}

/**************************************************************************
 *   ql4000_print_scsi_cmd
 *
 **************************************************************************/
void qla4000_print_scsi_cmd(uint32_t dbg_mask, Scsi_Cmnd *cmd)
{
	if ((ql_dbg_level & dbg_mask) != 0)
	{
		int   i;

		printk("SCSI Command = 0x%p, Handle=0x%p\n", cmd, CMD_HANDLE(cmd));
		printk("  b=%d, t=%02xh, l=%02xh, cmd_len = %02xh\n",
			       cmd->channel, cmd->target, cmd->lun, cmd->cmd_len);

		printk("  CDB = ");
		for (i = 0; i < cmd->cmd_len; i++)
			printk("%02x ", cmd->cmnd[i]);

		printk("  seg_cnt = %d\n",cmd->use_sg);
		printk("  request buffer = 0x%p, request buffer len = 0x%x\n",
		       cmd->request_buffer,cmd->request_bufflen);

		if (cmd->use_sg)
		{
			struct scatterlist *sg; 
			sg = (struct scatterlist *) cmd->request_buffer;
			printk("  SG buffer: \n");
			qla4000_dump_bytes(dbg_mask, (caddr_t)sg, 
					   (cmd->use_sg*sizeof(struct scatterlist)));
		}

		printk("  tag = %d, flags = 0x%x, transfersize = 0x%x \n", 
			       cmd->tag, cmd->flags,cmd->transfersize );
		printk("  Pid = %d, SP = 0x%p\n", (int)cmd->pid, CMD_SP(cmd));
		printk("  underflow size = 0x%x, direction=0x%x, req.cmd=0x%x \n", 
			       cmd->underflow, cmd->sc_data_direction,cmd->request.cmd);
		if (driver_template.use_new_eh_code)
		{
			printk("  Current time (jiffies) = 0x%lx, timeout expires = 0x%lx\n", 
			       jiffies, cmd->eh_timeout.expires);
		}
	}
}


/**************************************************************************
 *   ql4000_dump_device
 *
 **************************************************************************/
//void
//ql4000_dump_device(scsi_qla_host_t *ha)
//{
//
//	Scsi_Cmnd       *cp;
//	srb_t           *sp;
//	int i;
//	printk("Outstanding Commands on controller:\n");   
//	for (i=0; i < MAX_OUTSTANDING_COMMANDS; i++)
//	{
//		if ((sp = ha->outstanding_cmds[i]) == NULL)
//			continue;
//		if ((cp = sp->cmd) == NULL)
//			continue;
//		qla4000_print_scsi_cmd(cp);
//	}
//
//}

/*
 *  Get byte from I/O port
 */
inline uint8_t
qla4000_getbyte(uint8_t *port)
{
	uint8_t ret = readb(port);

	QL4PRINT(QLP9, printk("qla4000_getbyte: address = %ph data = %02Xh\n", port, ret));
	return(ret);
}

/*
 *  Get word from I/O port
 */
inline uint16_t
qla4000_getword(uint16_t *port)
{
	uint16_t ret = readw(port);

	QL4PRINT(QLP9, printk("qla4000_getword: address = %ph data = %04Xh\n", port, ret));
	return(ret);
}

/*
 *  Get double word from I/O port
 */
inline uint32_t
qla4000_getdword(uint32_t *port)
{
	uint32_t ret = readl(port);

	QL4PRINT(QLP9, printk("qla4000_getdword: address = %ph data = %08Xh\n", port, ret));
	return(ret);
}

/*
 *  Send byte to I/O port
 */
inline void
qla4000_putbyte(uint8_t *port, uint8_t data)
{
	writeb(data, port);
	QL4PRINT(QLP9, printk("qla4000_putbyte: address = %ph data = %02Xh\n", port, data));
}

/*
 *  Send word to I/O port
 */
inline void
qla4000_putword(uint16_t *port, uint16_t data)
{
	writew(data, port);
	QL4PRINT(QLP9, printk("qla4000_putword: address = %ph data = %04Xh\n", port, data));
}

/*
 *  Send double word to I/O port
 */
inline void
qla4000_putdword(uint32_t *port, uint32_t data)
{
	writel(data, port);
	QL4PRINT(QLP9, printk("qla4000_putdword: address = %ph data = %08Xh\n", port, data));
}

#endif

