/*
 * Driver debug definitions.
 */
#define QLP0    0x00000001  // Messages to always display (We use printk instead, in Linux)
#define QLP1    0x00000002  // Warnings and potential errors
#define QLP2    0x00000004  // Significant events error messages
#define QLP3    0x00000008  // Function trace messages
#define QLP4    0x00000010  // IOCTL trace messages
#define QLP5    0x00000020  // I/O & Request/Response queue trace messages
#define QLP6    0x00000040  // Watchdog messages (current state)
#define QLP7    0x00000080  // Initialization
#define QLP8    0x00000100  // Internal command queue traces
#define QLP9    0x00000200  // Register accesses
#define QLP10   0x00000400  // Extra Debug messages (dump buffers)
#define QLP11   0x00000800  // Mailbox & ISR Details
#define QLP12   0x00001000  // Enter/Leave routine messages
#define QLP13   0x00002000  // Display data for Inquiry, TUR, ReqSense, RptLuns

extern uint32_t ql_dbg_level;		

/*
 *  Debug Print Routine Prototypes.  
 */
#ifdef DEBUG
	#define QL4PRINT(m,x) {if(((m) & ql_dbg_level) != 0) (x);}
	#define ENTER(x) QL4PRINT(QLP12, printk("qla4000: Entering %s()\n", x))
	#define LEAVE(x) QL4PRINT(QLP12, printk("qla4000: Leaving  %s()\n", x))
			
    inline uint8_t qla4000_get_debug_level(uint32_t *dbg_level);
    inline uint8_t qla4000_set_debug_level(uint32_t dbg_level);

	void     qla4000_dump_bytes(uint32_t, void *, uint32_t);
	void     qla4000_dump_words(uint32_t, void *, uint32_t);
	void     qla4000_dump_dwords(uint32_t, void *, uint32_t);
	void     qla4000_dump_registers(uint32_t, scsi_qla_host_t *);
	void	 qla4000_print_scsi_cmd(uint32_t dbg_mask, Scsi_Cmnd *cmd);
	
	uint8_t  qla4000_getbyte(uint8_t *);
	uint16_t qla4000_getword(uint16_t *);
	uint32_t qla4000_getdword(uint32_t *);
	void     qla4000_putbyte(uint8_t *, uint8_t);
	void     qla4000_putword(uint16_t *, uint16_t);
	void     qla4000_putdword(uint32_t *, uint32_t);

	#if EXTENDED_DBG_ON_ERROR
	    inline void qla4000_set_extended_debug_level(void);
	    inline void qla4000_rst_extended_debug_level(void);
    #else
	    #define qla4000_set_extended_debug_level()
	    #define qla4000_rst_extended_debug_level()
	#endif
#else
	#define ENTER(x)
	#define LEAVE(x)
	
	#define qla4000_get_debug_level(d) (QLA_ERROR)
	#define qla4000_set_debug_level(d) (QLA_ERROR)
	
	#define QL4PRINT(m,x)
	#define qla4000_dump_bytes(m, b, s)
	#define qla4000_dump_words(m, b, s)
	#define qla4000_dump_dwords(m, b, s)
	#define qla4000_dump_registers(m, h)
	#define qla4000_print_scsi_cmd(m, c)
	
	#define qla4000_getbyte(p)
	#define qla4000_getword(p)
	#define qla4000_getdword(p)
	#define qla4000_putbyte(p,d)
	#define qla4000_putword(p,d)
	#define qla4000_putdword(p,d)
	
	#define qla4000_set_extended_debug_level()
	#define qla4000_rst_extended_debug_level()
#endif
