/*************************************************************************
*                                                                       *
*                              NOTICE                                   *
*                                                                       *
*                 COPYRIGHT 2002 QLOGIC CORPORATION                     *
*                       ALL RIGHTS RESERVED                             *
*                                                                       *
* This computer program is CONFIDENTIAL  and contains TRADE SECRETS of  *
* QLOGIC CORPORATION.  The  receipt or possession of this program does  *
* not convey any rights to reproduce or disclose  its contents,  or to  *
* manufacture, use, or sell anything that it may describe, in whole or  *
* in part, without the specific written consent of QLOGIC CORPORATION.  *
* Any reproduction of this program without the express written consent  *
* of QLOGIC CORPORATION  is a violation of the copyright laws and  may  *
* subject you to civil liability and criminal prosecution.              *
*                                                                       *
*************************************************************************/
/*
 * qla4mbx.h
 *
 * This file defines mailbox structures and definitions for	the QLA4xxx
 *  iSCSI HBA firmware.
 *
 *-----------------------------------------------------------------------
 *          Modification History
 *  Edit date   Editor  Description
 *-----------------------------------------------------------------------
 *  
 */

#ifndef _QLA4X_FW_H
#define _QLA4X_FW_H

#ifdef __cplusplus
extern "C" {
#endif

#ifndef INT8 
#define INT8 char 
#endif
#ifndef INT16
#define INT16 short
#endif
#ifndef INT32
#define INT32 long
#endif
#ifndef UINT8
#define UINT8 unsigned char
#endif
#ifndef UINT16
#define UINT16 unsigned short
#endif
#ifndef UINT32
#define UINT32 unsigned long
#endif
#ifndef UINT64
#define UINT64 unsigned long long
#endif

#define QLA4000_MAX_CMDS_TO_RISC	1024
#define QLA4010_MAX_CMDS_TO_RISC	1024

#define QLA4XXX_VENDOR_ID   	0x1077
#define QLA4000_DEVICE_ID  		0x4000
#define QLA4010_DEVICE_ID  		0x4010

/*************************************************************************
 *
 * 			ISP I/O Register Set Structure and Definitions
 *
 *************************************************************************/

#define MBOX_REG_COUNT		16

typedef volatile struct _ISP_REG_T
{
   INT32  reserved1[4];            	/* first four registers are reserved */

   UINT32  requestQueueInPointer;   /*in-bound Message Register 0 offset : 0x10         */
					/*writing to this register generates an interrupt   */

   UINT32  inboundMesgReg1;         /*in-bound Message Register 1 offset : 0x14         */
					/*writing to this register generates an interrupt   */

   UINT32  responseQueueInPointer;  /*out-bound Message Register 0   offset : 0x18      */
					/*writing to this register generates an interrupt   */

   UINT32  fastPostHandle;          /*out-bound Message Register 1   offset : 0x1c      */
					/*writing to this register generates an interrupt   */

   UINT32  mailboxIn0;              /*in-bound doorbell register  offset : 0x20         */
					/* Opcode in hi 16 bits, # of mailboxes in lo 16 bit*/
					/*writing to this register generates an interrupt   */
   UINT32  interruptStatusIn;       /*in-bound interrupt status register offset : 0x24  */
   #define ISI_REQ_QUEUE_PENDING	0x00000001  // Bit 0
   #define ISI_MBOX_CMD_PENDING	0x00000004  // Bit 2
   
   UINT32  interruptMaskIn;         /*in-bound interrupt mask register  offset : 0x28   */
   #define IMI_REQ_QUEUE_PENDING	ISI_REQ_QUEUE_PENDING
   #define IMI_MBOX_CMD_PENDING	ISI_MBOX_CMD_PENDING
   #define IMI_FW_RESET_COMPLETE	0x0000007B

   UINT32  mailboxOut0;             /*out-bound doorbell register offset : 0x2c         */

   UINT32  interruptStatusOut;      /*out-bound interrupt status register offset : 0x30 */
   #define ISO_RSP_QUEUE_COMPLETE	0x00000001  // Bit 0
   #define ISO_FASTPOST_COMPLETE	0x00000002  // Bit 1
   #define ISO_MB0X_CMD_COMPLETE	0x00000004  // Bit 2
   #define INTR_PENDING (ISO_RSP_QUEUE_COMPLETE | ISO_FASTPOST_COMPLETE | ISO_MB0X_CMD_COMPLETE)
   #define INVALID_REGISTER		0xFFFFFFFF

   UINT32  interruptMaskOut;        /*out-bound interrupt mask register offset : 0x34   */
   #define IMO_RSP_QUEUE_COMPLETE	ISO_RSP_QUEUE_COMPLETE
   #define IMO_FASTPOST_COMPLETE	ISO_FASTPOST_COMPLETE
   #define IMO_MB0X_CMD_COMPLETE	ISO_MB0X_CMD_COMPLETE

   UINT32  reserved2[18];            /* reserved registers  offset : 0x38         */
   UINT32  mailboxIn[MBOX_REG_COUNT-1]; /*mailboxIn registers   offset : 0x80           */
   UINT32  reserved3;
   UINT32  mailboxOut[MBOX_REG_COUNT-1]; /*mailboxIn registers   offset : 0xc0           */
   UINT32  reserved4;
   UINT32  requestQueueOutPointer;  /* offset : 0x100   use this register for Request Queue OUT pointer */
   UINT32  responseQueueOutPointer;	/* offset : 0x104   use this register for Response Queue OUT pointer */
} ISP_REG_T;

/*************************************************************************
 *
 *				Mailbox Commands Structures and Definitions
 *
 *************************************************************************/

// Mailbox command definitions
#define MBOX_CMD_LOAD_RISC_RAM_EXT              0x0001
#define MBOX_CMD_EXECUTE_FW                     0x0002
#define MBOX_CMD_DUMP_RISC_RAM_EXT              0x0003
#define MBOX_CMD_WRITE_RISC_RAM_EXT             0x0004
#define MBOX_CMD_READ_RISC_RAM_EXT              0x0005
#define MBOX_CMD_REGISTER_TEST                  0x0006
#define MBOX_CMD_VERIFY_CHECKSUM                0x0007
#define MBOX_CMD_ABOUT_FW                       0x0009
#define MBOX_CMD_LOOPBACK_DIAG                  0x000A
#define MBOX_CMD_PING                           0x000B
#define MBOX_CMD_CHECKSUM_FW                    0x000E
#define MBOX_CMD_RESET_FW                       0x0014
#define MBOX_CMD_ABORT_TASK                     0x0015
#define MBOX_CMD_LUN_RESET                      0x0016
#define MBOX_CMD_TARGET_WARM_RESET              0x0017
#define MBOX_CMD_TARGET_COLD_RESET              0x0018
#define MBOX_CMD_ABORT_QUEUE                    0x001C
#define MBOX_CMD_GET_QUEUE_STATUS               0x001D
#define MBOX_CMD_GET_MANAGEMENT_DATA            0x001E
#define MBOX_CMD_GET_FW_STATUS                  0x001F
#define MBOX_CMD_COPY_FLASH                     0x0024
		#define COPY_FLASH_OPTION_PRIM_TO_SEC           0
		#define COPY_FLASH_OPTION_SEC_TO_PRIM           1
#define MBOX_CMD_WRITE_FLASH                    0x0025
		#define WRITE_FLASH_OPTION_HOLD_DATA            0
		#define WRITE_FLASH_OPTION_COMMIT_DATA          2
#define MBOX_CMD_READ_FLASH                     0x0026
#define MBOX_CMD_GET_QUEUE_PARAMS               0x0029
#define MBOX_CMD_SET_QUEUE_PARAMS               0x0039
#define MBOX_CMD_CONN_CLOSE_SESS_LOGOUT         0x0056
		#define LOGOUT_OPTION_CLOSE_SESSION             0x01
		#define LOGOUT_OPTION_RELOGIN                   0x02
#define MBOX_CMD_INITIALIZE_FIRMWARE            0x0060
#define MBOX_CMD_GET_INIT_FW_CTRL_BLOCK         0x0061
#define MBOX_CMD_REQUEST_DATABASE_ENTRY         0x0062
#define MBOX_CMD_SET_DATABASE_ENTRY             0x0063
#define MBOX_CMD_GET_DATABASE_ENTRY             0x0064
		#define DDB_DS_UNASSIGNED                       0x00
		#define DDB_DS_NO_CONNECTION_ACTIVE             0x01
		#define DDB_DS_DISCOVERY                        0x02
		#define DDB_DS_NO_SESSION_ACTIVE                0x03
		#define DDB_DS_SESSION_ACTIVE                   0x04
		#define DDB_DS_LOGGING_OUT                      0x05
		#define DDB_DS_SESSION_FAILED                   0x06
#define MBOX_CMD_CLEAR_ACA                      0x0065
#define MBOX_CMD_CLEAR_TASK_SET                 0x0067
#define MBOX_CMD_ABORT_TASK_SET                 0x0068
#define MBOX_CMD_GET_FW_STATE                   0x0069
		#define FW_STATE_READY                          0x0000
		#define FW_STATE_CONFIG_WAIT                    0x0001
		#define FW_STATE_WAIT_LOGIN                     0x0002
		#define FW_STATE_INITIALIZING                   0x0003
		#define FW_STATE_ERROR                          0x0004
	        
		#define MEDIA_TYPE_COPPER                       0
	        #define MEDIA_TYPE_OPTICAL                      1
#define MBOX_CMD_GET_INIT_FW_CTRL_BLOCK_DEFAULTS 0x006A
#define MBOX_CMD_GET_DATABASE_ENTRY_DEFAULTS    0x006B
#define MBOX_CMD_CONN_OPEN_SESS_LOGIN           0x0074
#define MBOX_CMD_NOP                            0x00FF

// Mailbox status definitions
#define MBOX_STS_FIRMWARE_ALIVE                 0x0000
#define MBOX_STS_CHECKSUM_ERROR                 0x0001
#define MBOX_STS_BUSY                           0x0004
#define MBOX_STS_FATAL_ERROR                    0xF000

#define MBOX_STS_INTERMEDIATE		    0x1000
#define MBOX_STS_COMMAND_COMPLETE               0x4000
#define MBOX_STS_INVALID_COMMAND                0x4001
#define MBOX_STS_HOST_INTERFACE_ERROR           0x4002
#define MBOX_STS_TEST_FAILED                    0x4003
#define MBOX_STS_COMMAND_ERROR                  0x4005
#define MBOX_STS_COMMAND_TIMEOUT                0x4005
#define MBOX_STS_COMMAND_PARAMETER_ERROR        0x4006
#define MBOX_STS_TARGET_MODE_INIT_FAIL          0x4007
#define MBOX_STS_INITIATOR_MODE_INIT_FAIL       0x4008
#define MBOX_STS_TARGET_MODE_INIT_FAIL_2        0x4009
#define MBOX_STS_TARGET_MODE_INIT_FAIL_3        0x400A
#define MBOX_ASTS_SYSTEM_ERROR                  0x8002
#define MBOX_ASTS_REQUEST_TRANSFER_ERROR        0x8003
#define MBOX_ASTS_RESPONSE_TRANSFER_ERROR       0x8004
#define MBOX_ASTS_PROTOCOL_STATISTIC_ALARM      0x8005
#define MBOX_ASTS_SCSI_COMMAND_PDU_REJECTED     0x8006
#define MBOX_ASTS_DATABASE_CHANGED              0x8014
#define MBOX_ASTS_UNSOLICITED_PDU_RECEIVED      0x8015


/* Host Adapter Initialization Control Block (from host) */
typedef struct _INIT_FW_CTRL_BLK {
   UINT8 	Version ;                /* 00 */
   UINT8 	RES0;                    /* 01 */

   UINT16   FwOptions ;           /* 02-03 */
   #define  FWOPT_MARKER_DISABLE             0x0400
   #define  FWOPT_PROTOCOL_STAT_ALARM_ENABLE 0x0200
   #define  FWOPT_DDB_CHANGE_AE_ENABLE       0x0100
   #define  FWOPT_ACCESS_CONTROL_ENABLE      0x0080
   #define  FWOPT_SESSION_MODE               0x0040
   #define  FWOPT_INITIATOR_MODE             0x0020
   #define  FWOPT_TARGET_MODE                0x0010
   #define  FWOPT_FAST_POSTING               0x0008
   #define  FWOPT_DMA_INT_STATUS_ENABLE      0x0004
   #define  FWOPT_SENSE_BUF_DSCRPTR_ENABLE   0x0002

   UINT16   ExecThrottle ;        /* 04-05 */
   UINT8    RetryCount ;          /* 06 */
   UINT8    RetryDelay ;          /* 07 */
   UINT16   RES1;                 /* 08-09 */
   UINT16   AddFwOptions;         /* 0A-0B */
   UINT16   WakeupThreshold;      /* 0C-0D */
   UINT16   RES2;                 /* 0E-0F */
   UINT16   ReqQOutPtr;           /* 10-11 */
   UINT16   RspQInPtr;            /* 12-13 */
   UINT16   ReqQLen;              /* 14-15 */
   UINT16   RspQLen;              /* 16-17 */
   UINT32   ReqQAddrLo;           /* 18-1B */
   UINT32   ReqQAddrHi;           /* 1C-1F */
   UINT32   RspQAddrLo;           /* 20-23 */
   UINT32   RspQAddrHi;           /* 24-27 */
   UINT32   IntRegBufAddr0;       /* 28-2B */
   UINT32   IntRegBufAddr1;       /* 2C-2F */

   UINT16   iSCSIOptions;         /* 30-31 */
   #define  IOPT_RCV_ISCSI_MARKER_ENABLE     0x8000
   #define  IOPT_SEND_ISCSI_MARKER_ENABLE    0x4000
   #define  IOPT_HEADER_DIGEST_ENABLE        0x2000
   #define  IOPT_DATA_DIGEST_ENABLE          0x1000
   #define  IOPT_IMMEDIATE_DATA_ENABLE       0x0800
   #define  IOPT_INITIAL_R2T_ENABLE          0x0400
   #define  IOPT_DATA_PDU_IN_ORDER           0x0100

   UINT16 	TCPOptions;          /* 32-33 */
   #define  TOPT_AUTO_DISCOVERY_ENABLE       0x1000
   #define  TOPT_SLP_UA_ENABLE               0x0800
   #define  TOPT_SLP_SA_ENABLE               0x0400
   #define  TOPT_DHCP_ENABLE                 0x0200
   #define  TOPT_DNS_ENABLE                  0x0100
   #define  TOPT_NAGLEDISABLE                0x0020
   #define  TOPT_FRAGDISABLE                 0x0010
   #define  TOPT_TIMERSCALEMASK              0x000E
   #define  TOPT_TIMESTAMPENABLE             0x0001

   UINT16 	IPOptions;           /* 34-35 */
   #define  IPOPT_IPV6_ADDRESS_ENABLE        0x0002
   #define  IPOPT_IP_ADDRESS_VALID           0x0001

   UINT16 	MaxPDUSize;          /* 36-37 */
   UINT16 	RcvMarkerInt;        /* 38-39 */
   UINT16 	SndMarkerInt;        /* 3A-3B */
   UINT16 	InitMarkerlessInt;   /* 3C-3D */
   UINT16 	FirstBurstSize;      /* 3E-3F */
   UINT16 	LogOutMinTime;       /* 40-41 */
   UINT16 	LogOutMaxTime;       /* 42-43 */
   UINT16 	MaxOutStndngR2T;     /* 44-45 */
   UINT16 	KeepAliveTimeout;    /* 46-47 */
   UINT16 	PortNumber;          /* 48-49 */
   UINT16 	MaxBurstSize;        /* 4A-4B */
   UINT16 	MSS;                 /* 4C-4D */
   UINT16 	RES4;                /* 4E-4F */     /* special firmware configuration flags */
   #define 	RES4_TARGET_LOG              0x0001 /* enable Target Event log            */
   #define 	RES4_INITIATOR_LOG           0x0002 /* enable Initiator Event log         */
   #define 	RES4_ALL_TARGET_PDU_SHOW     0x0004 /* enable PDU log for all Targets (decode sent/rcvd PDUs)    */
   #define 	RES4_ALL_INITIATOR_PDU_SHOW  0x0008 /* enable PDU log for all Initiators (decode sent/rcvd PDUs) */
   #define 	RES4_SN_ERRLOG               0x0010 /* enable SeqNum Error log            */
   #define 	RES4_DISABLE_AUTOCONNECT     0x0020 /* disable autoconnect mode           */
   #define 	RES4_BSD_SOCKET_TASK_LOG     0x0040 /* enable bsd Socket Event log        */
   #define 	RES4_BSD_SOCKET_MASKS_SHOW   0x0080 /* enable bsd Socket R/W Masks log    */
   #define 	RES4_BSD_DATA_OUT_LOG        0x0100 /* enable bsd Socket Data-Out PDU log */
   #define 	RES4_RQSTQ_ENTRY_LOG         0x0200 /* enable requestQ entry log          */
   #define 	RES4_RSPQ_ENTRY_LOG          0x0400 /* enable responseQ entry log         */
   #define 	RES4_SKIP_TARGET_DDB_REINIT  0x0800 /* don't re-initialize target DDB values */
   
   UINT8  	IPAddr[16];             /* 50-5F */
   UINT8  	SubnetMask[16];         /* 60-6F */
   UINT8  	GatewayIPAddr[16];      /* 70-7F */
   UINT8  	DNSIPAddr[16];          /* 80-8F */
   UINT8  	Alias[32];              /* 90-AF */
   UINT8  	TargAddr[32];           /* B0-CF */
   UINT8  	SndScale;               /* D0*/
   UINT8  	RcvScale;               /* D1 */
   UINT8  	TCPFlags;               /* D2 */
   UINT8  	TypeOfService;          /* D3 */
   UINT8  	Time2Live;              /* D4 */
   UINT8  	RES3;                   /* D5 */
   UINT16 	VLANPriority;        	/* D6-D7 */
   UINT8  	RES6[40];               /* D8-FF */
   UINT8  	iSCSINameString[256];   /* 100-1FF */
} INIT_FW_CTRL_BLK;

typedef struct _DEVDB_ENTRY
{
   UINT8  options;            /* 00 */
   #define  DDB_OPT_DISABLE                  0x08  /* do not connect to device */
   #define  DDB_OPT_ACCESSGRANTED            0x04
   #define  DDB_OPT_TARGET                   0x02  /* device is a target */
   #define  DDB_OPT_INITIATOR                0x01  /* device is an initiator */


   UINT8  control;            /* 01 */
   #define  DDB_CTRL_SESSION_RECOVERY        0x10
   #define  DDB_CTRL_SENDING                 0x08
   #define  DDB_CTRL_XFR_PENDING             0x04
   #define  DDB_CTRL_QUEUE_ABORTED           0x02
   #define  DDB_CTRL_LOGGED_IN               0x01

   UINT16 exeThrottle;        /* 02-03 */
   UINT16 exeCount;           /* 04-05 */
   UINT8  retryCount;         /* 06    */
   UINT8  retryDelay;         /* 07    */
   UINT16 iSCSIOptions;       /* 08-09 */
   #define DDB_IOPT_RECV_ISCSI_MARKER_ENABLE 0x8000
   #define DDB_IOPT_SEND_ISCSI_MARKER_ENABLE 0x4000
   #define DDB_IOPT_HEADER_DIGEST_ENABLE     0x2000
   #define DDB_IOPT_DATA_DIGEST_ENABLE       0x1000
   #define DDB_IOPT_IMMEDIATE_DATA_ENABLE    0x0800
   #define DDB_IOPT_INITIAL_R2T_ENABLE       0x0400
   #define DDB_IOPT_DATA_SEQUENCE_IN_ORDER   0x0200
   #define DDB_IOPT_DATA_PDU_IN_ORDER        0x0100
   #define DDB_IOPT_AUTHENTICATION_ENABLE    0x0080
   #define DDB_IOPT_RESERVED2                0x007F

   UINT16 TCPOptions;         /* 0A-0B */
   #define DDB_TOPT_NAGLEDISABLE             0x0020
   #define DDB_TOPT_FRAGDISABLE              0x0010
   #define DDB_TOPT_TIMERSCALEMASK           0x000E
   #define DDB_TOPT_TIMESTAMPENABLE          0x0001

   UINT16 IPOptions;          /* 0C-0D */
   #define DDB_IPOPT_IPV6_ADDRESS_ENABLE     0x0002
   #define DDB_IPOPT_IP_ADDRESS_VALID        0x0001

   UINT16 maxPDUSize;         /* 0E-0F */
   UINT16 rcvMarkerInt;       /* 10-11 */
   UINT16 sndMarkerInt;       /* 12-13 */
   UINT16 initMarkerlessInt;  /* 14-15 */
   UINT16 firstBurstSize;     /* 16-17 */
   UINT16 logoutMinTime;      /* 18-19 */
   UINT16 logoutMaxTime;      /* 1A-1B */
   UINT16 maxOutstndngR2T;    /* 1C-1D */
   UINT16 keepAliveTimeout;   /* 1E-1F */
   UINT8 ISID[6];            /* 20-25 big-endian, must be converted to little-endian */
   UINT16 TSID;               /* 26-27 */
   UINT16 portNumber;         /* 28-29 */
   UINT16 maxBurstSize;       /* 2A-2B */
   UINT16 taskMngmntTimeout;  /* 2C-2D */
   UINT16 RES0;               /* 2E-2F */
   #define DDB_RES0_SEND_ZBUFS               0x0001 /* use zbufs when sending data                */
   #define DDB_RES0_RECV_ZBUFS               0x0002 /* use zbufs when receiving data              */
   #define DDB_RES0_TARGET_PDU_SHOW          0x0004 /* enable Target PDU log for this ddb (decode all sent/rcvd PDUs)    */
   #define DDB_RES0_INITIATOR_PDU_SHOW       0x0008 /* enable Initiator PDU log for this ddb (decode all sent/rcvd PDUs) */
   #define DDB_RES0_SECURITY_STAGE_LOGIN     0x0010 /* include security stage during login        */
   #define DDB_RES0_NEGOT_ALL_LOGIN_KEYS     0x0020 /* negotiate all keys during each login stage */
   #define DDB_RES0_QUERY_ALL_LOGIN_KEYS     0x0040 /* query all keys at end of each login stage  */
   #define DDB_RES0_DONT_PARSE_LOGIN_RSP     0x0080 /* don't parse parameters in login Resp PDU   */
   #define DDB_RES0_SWAP_CUR_NXT_LOGIN_STG   0x0100 /* swap current/next login stage fields       */
   #define DDB_RES0_NEGOTIATION_DETAIL_SHOW  0x0200 /* show iSCSI key negotiation details         */
   UINT8  ipAddr[0x10];       /* 30-3F */
   UINT8  iSCSIAlias[0x20];   /* 40-5F */
   UINT8  targetAddr[0x20];   /* 60-7F */
   UINT8  userID[0x20];       /* 80-9F */
   UINT8  password[0x20];     /* A0-BF */
   UINT8  iscsiName[0x100];   /* C0-1BF : xxzzy Make this a pointer to a string so we don't
                                                have to reserve soooo much RAM */
   UINT16 ddbLink;            /* 1C0-1C1 */
   UINT16 RES1;               /* 1C2-1C3 */
} DEVDB_ENTRY;


// Flash definitions
typedef struct _SYS_INFO_PHYS_ADDR
{
    UINT8            address[6]; 		/* 00-05 */
    UINT8            filler[2];		/* 06-07 */
} SYS_INFO_PHYS_ADDR;

typedef struct _FLASH_SYS_INFO
{
    UINT32           cookie;		/* 00-03 */
    UINT32           physAddrCount;		/* 04-07 */
    SYS_INFO_PHYS_ADDR physAddr[4];		/* 08-27 */
    UINT8            vendorId[128];		/* 28-A7 */
    UINT8            productId[128];	/* A8-127 */
    UINT32           serialNumber;		/* 128-12B */
   
    // PCI Configuration values
    UINT32           pciDeviceVendor; 	/* 12C-12F */
    UINT32           pciDeviceId;	  	/* 130-133 */
    UINT32           pciSubsysVendor; 	/* 134-137 */
    UINT32           pciSubsysId;	  	/* 138-13B */

    // This validates version 1.
    UINT32           crumbs;	  	/* 13C-13F */

    UINT32           enterpriseNumber;	/* 140-143 */

    // Leave this last in the struct so it is declared invalid if 
    // any new items are added.
    UINT32           crumbs2;	  	/* 144-147 */
} FLASH_SYS_INFO, *PFLASH_SYS_INFO;

#define FLASH_FW_IMG_PAGE_SIZE        0x20000
#define FLASH_FW_IMG_PAGE(addr)       (0xfffe0000 & (addr))

#define FLASH_OFFSET_SECONDARY_FW_IMG 0x01000000
#define FLASH_OFFSET_PRIMARY_FW_IMG   0x01800000
#define FLASH_OFFSET_SYS_INFO         0x02000000
#define FLASH_OFFSET_DRIVER_BLK       0x03000000
#define FLASH_OFFSET_INIT_FW_CTRL_BLK 0x04000000
#define FLASH_OFFSET_DEV_DB_AREA      0x05000000

#define MAX_FLASH_SZ            	  0x400000    /* 4M flash */
#define FLASH_DEFAULTBLOCKSIZE  	  0x20000
#define FLASH_EOF_OFFSET        	  FLASH_DEFAULTBLOCKSIZE - 8        /* 4 bytes for EOF signature */
#define FLASH_FILESIZE_OFFSET   	  FLASH_EOF_OFFSET - 4            /* 4 bytes for file size */
#define FLASH_CKSUM_OFFSET      	  FLASH_FILESIZE_OFFSET - 4              /* 4 bytes for chksum protection */


/*************************************************************************
 *
 *				IOCB Commands Structures and Definitions
 *
 *************************************************************************/
#define IOCB_MAX_CDB_LEN            16  /* Bytes in a CBD */
#define IOCB_MAX_SENSEDATA_LEN      32  /* Bytes of sense data */
#define IOCB_MAX_EXT_SENSEDATA_LEN  60  /* Bytes of extended sense data */
#define IOCB_MAX_DSD_CNT             1  /* DSDs per noncontinuation type IOCB */
#define IOCB_CONT_MAX_DSD_CNT        5  /* DSDs per Continuation */
#define CTIO_MAX_SENSEDATA_LEN    	24  /* Bytes of sense data in a CTIO*/

#define RESERVED_BYTES_MARKER       40  /* Reserved Bytes at end of Marker */
#define RESERVED_BYTES_INOT         28  /* Reserved Bytes at end of Immediate Notify */
#define RESERVED_BYTES_NOTACK       28  /* Reserved Bytes at end of Notify Acknowledge */
#define RESERVED_BYTES_CTIO          2  /* Reserved Bytes in middle of CTIO */

#define MAX_MBX_COUNT               14  /* Maximum number of mailboxes in MBX IOCB */

#define ISCSI_MAX_NAME_BYTECNT      256  /* Bytes in a target name */

#define IOCB_ENTRY_SIZE         	0x40


/* IOCB header structure */
typedef struct _HEADER
{
   UINT8 entryType;
   #define ET_STATUS                0x03
   #define ET_MARKER                0x04
   #define ET_CONT_T1               0x0A
   #define ET_INOT                  0x0D
   #define ET_NACK                  0x0E
   #define ET_STATUS_CONTINUATION   0x10
   #define ET_CMND_T4               0x15
   #define ET_ATIO                  0x16
   #define ET_CMND_T3               0x19
   #define ET_CTIO4                 0x1E
   #define ET_CTIO3                 0x1F
   #define ET_PERFORMANCE_STATUS	0x20
   #define ET_MAILBOX_CMD           0x38
   #define ET_MAILBOX_STATUS        0x39
   #define ET_PASSTHRU0             0x3A
   #define ET_PASSTHRU1             0x3B
   #define ET_PASSTHRU_STATUS       0x3C
   #define ET_ASYNCH_MSG            0x3D
   #define ET_CTIO5                 0x3E
   #define ET_CTIO6                 0x3F

   UINT8 entryCount;
   UINT8 systemDefined;
   UINT8 entryStatus;
    #define ES_MASK                 0x3E
    #define ES_BUSY                 0x02
    #define ES_INVALID_ENTRY_TYPE   0x04
    #define ES_INVALID_ENTRY_PARAM  0x08
    #define ES_INVALID_ENTRY_COUNT  0x10
    #define ES_INVALID_ENTRY_ORDER  0x20
	//KRH: Verify these

} HEADER ;

/* Genric queue entry structure*/
typedef struct QUEUE_ENTRY
{
   UINT8  data[60];
   UINT32 signature;
   #define RESPONSE_PROCESSED	0xDEADDEAD

}QUEUE_ENTRY;


/* 64 bit addressing segment counts*/

#define COMMAND_SEG_A64             1
#define CONTINUE_SEG_A64            5
#define CONTINUE_SEG_A64_MINUS1     4

/* 64 bit addressing segment definition*/

typedef struct DATA_SEG_A64
{
   struct
   {
      UINT32 addrLow;
      UINT32 addrHigh;

   } base;

   UINT32 count;

}DATA_SEG_A64;

/* Command Type 3 entry structure*/

typedef struct _COMMAND_T3_ENTRY
{
   HEADER  hdr;                    /* 00-03 */

   UINT32  handle;                 /* 04-07 */
   UINT16  target;                 /* 08-09 */
   UINT16  connection_id;          /* 0A-0B */

   UINT8   control_flags;          /* 0C */
   #define CF_NO_FAST_POST         0x80
   #define CF_DISABLE_INT          0x10
   
   /* data direction  (bits 5-6)*/
   #define CF_WRITE                0x20
   #define CF_READ                 0x40
   #define CF_NO_DATA              0x00
   #define CF_DIRECTION_MASK       0x60
   
   /* task attributes (bits 2-0) */
   #define CF_ACA_QUEUE            0x04
   #define CF_HEAD_TAG             0x03
   #define CF_ORDERED_TAG          0x02
   #define CF_SIMPLE_TAG           0x01
   #define CF_TAG_TYPE_MASK        0x07
   #define CF_ATTRIBUTES_MASK      0x67

   /* STATE FLAGS FIELD IS A PLACE HOLDER. THE FW WILL SET BITS IN THIS FIELD AS THE
      COMMAND IS PROCESSED. WHEN THE IOCB IS CHANGED TO AN IOSB THIS FIELD WILL HAVE
      THE STATE FLAGS SET PROPERLY.
   */
   UINT8   state_flags;            /* 0D */
   UINT8   cmdRefNum;              /* 0E */
   UINT8   reserved1;              /* 0F */
   UINT8   cdb[IOCB_MAX_CDB_LEN];  /* 10-1F */
   UINT8   lun[8];                 /* 20-27 */
   UINT32  cmdSeqNum;              /* 28-2B */
   UINT16  timeout;                /* 2C-2D */
   UINT16  dataSegCnt;             /* 2E-2F */
   UINT32  ttlByteCnt;             /* 30-33 */
   DATA_SEG_A64 dataseg[COMMAND_SEG_A64];  /* 34-3F */

} COMMAND_T3_ENTRY;

typedef struct _COMMAND_T4_ENTRY
{
   HEADER  hdr;                   /* 00-03 */
   UINT32  handle;                /* 04-07 */
   UINT16  target;                /* 08-09 */
   UINT16  connection_id;         /* 0A-0B */
   UINT8   control_flags;         /* 0C */

   /* STATE FLAGS FIELD IS A PLACE HOLDER. THE FW WILL SET BITS IN THIS FIELD AS THE
      COMMAND IS PROCESSED. WHEN THE IOCB IS CHANGED TO AN IOSB THIS FIELD WILL HAVE
      THE STATE FLAGS SET PROPERLY.
   */
   UINT8   state_flags;           /* 0D */
   UINT8   cmdRefNum;             /* 0E */
   UINT8   reserved1;                 /* 0F */
   UINT8   cdb[IOCB_MAX_CDB_LEN];    /* 10-1F */
   UINT8   lun[8];                /* 20-27 */
   UINT32  cmdSeqNum;             /* 28-2B */
   UINT16  timeout;               /* 2C-2D */
   UINT16  dataSegCnt;            /* 2E-2F */
   UINT32  ttlByteCnt;            /* 30-33 */

   /* WE ONLY USE THE ADDRESS FIELD OF THE FOLLOWING STRUCT. THE COUNT FIELD IS RESERVED */
   DATA_SEG_A64 dataseg[COMMAND_SEG_A64];  /* 34-3F */
} COMMAND_T4_ENTRY;

/* Continuation Type 1 entry structure*/
typedef struct _CONTINUATION_T1_ENTRY
{
   HEADER  hdr;

   DATA_SEG_A64 dataseg[CONTINUE_SEG_A64];

}CONTINUATION_T1_ENTRY;

/* Status Continuation Type entry structure*/
typedef struct _STATUS_CONTINUATION_ENTRY
{
   HEADER  hdr;

   UINT8 extSenseData[IOCB_MAX_EXT_SENSEDATA_LEN];

}STATUS_CONTINUATION_ENTRY;

/* Parameterize for 64 or 32 bits */
    #define COMMAND_SEG     COMMAND_SEG_A64
    #define CONTINUE_SEG    CONTINUE_SEG_A64

    #define COMMAND_ENTRY   COMMAND_T3_ENTRY
    #define CONTINUE_ENTRY  CONTINUATION_T1_ENTRY

    #define ET_COMMAND      ET_CMND_T3
    #define ET_CONTINUE     ET_CONT_T1



/* Marker entry structure*/
typedef struct _MARKER_ENTRY
{
   HEADER  hdr;                 /* 00-03 */

   UINT32  system_defined;      /* 04-07 */
   UINT16  target;              /* 08-09 */
   UINT16  modifier;            /* 0A-0B */
   #define MM_LUN_RESET         1
   #define MM_TARGET_WARM_RESET 0 
   #define MM_TARGET_COLD_RESET 2
   #define MM_CLEAR_TASK_SET    0
   #define MM_ABORT_TASK_SET    0

   UINT16  flags;               /* 0C-0D */
   UINT16  reserved1;           /* 0E-0F */
   UINT8   lun[8];              /* 10-17 */
   UINT64  reserved2;           /* 18-1F */
   UINT64  reserved3;           /* 20-27 */
   UINT64  reserved4;           /* 28-2F */
   UINT64  reserved5;           /* 30-37 */
   UINT64  reserved6;           /* 38-3F */
}MARKER_ENTRY;

/* Status entry structure*/
typedef struct _STATUS_ENTRY
{
   HEADER  hdr;                              /* 00-03 */

   UINT32  handle;                           /* 04-07 */

   UINT8   scsiStatus;                       /* 08 */
   #define SCSI_STATUS_MASK                  0xFF
   #define SCSI_STATUS                       0xFF
   #define SCSI_GOOD                         0x00

   UINT8   iscsiFlags;                       /* 09 */
   #define ISCSI_FLAG_RESIDUAL_OVER          0x02
   #define ISCSI_FLAG_RESIDUAL_UNDER         0x04
   #define ISCSI_FLAG_RESIDUAL_UNDER_BIREAD  0x08
   #define ISCSI_FLAG_RESIDUAL_OVER_BIREAD   0x10

   UINT8   iscsiResponse;                    /* 0A */
   #define ISCSI_RSP_COMPLETE                    0x00
   #define ISCSI_RSP_TARGET_FAILURE              0x01
   #define ISCSI_RSP_DELIVERY_SUBSYS_FAILURE     0x02
   #define ISCSI_RSP_UNSOLISITED_DATA_REJECT     0x03
   #define ISCSI_RSP_NOT_ENOUGH_UNSOLISITED_DATA 0x04
   #define ISCSI_RSP_CMD_IN_PROGRESS             0x05

   UINT8   completionStatus;                 /* 0B */
   #define SCS_COMPLETE                      0x00
   #define SCS_INCOMPLETE                    0x01
   #define SCS_DMA_ERROR                     0x02
   #define SCS_TRANSPORT_ERROR               0x03
   #define SCS_RESET_OCCURRED                0x04
   #define SCS_ABORTED                       0x05
   #define SCS_TIMEOUT                       0x06
   #define SCS_DATA_OVERRUN                  0x07
   #define SCS_DATA_DIRECTION_ERROR          0x08
   #define SCS_DATA_UNDERRUN                 0x15
   #define SCS_QUEUE_FULL                    0x1C
   #define SCS_DEVICE_UNAVAILABLE            0x28
   #define SCS_DEVICE_LOGGED_OUT             0x29
   #define SCS_DEVICE_CONFIG_CHANGED         0x2A

   UINT8   reserved1;                        /* 0C */

    /* state_flags MUST be at the same location as state_flags in the Command_T3/4_Entry */
   UINT8   state_flags;                      /* 0D */
   #define STATE_FLAG_SENT_COMMAND           0x01
   #define STATE_FLAG_TRANSFERRED_DATA       0x02
   #define STATE_FLAG_GOT_STATUS             0x04
   #define STATE_FLAG_LOGOUT_SENT            0x10

   UINT16  senseDataByteCnt;                 /* 0E-0F */
   UINT32  residualByteCnt;                  /* 10-13 */
   UINT32  bidiResidualByteCnt;              /* 14-17 */
   UINT32  expSeqNum;                        /* 18-1B */
   UINT32  maxCmdSeqNum;                     /* 1C-1F */
   UINT8   scsiSenseData[IOCB_MAX_SENSEDATA_LEN]; /* 20-3F */

}STATUS_ENTRY;

/*
 * Performance Status Entry where up to 30 handles can be posted in a
 * single IOSB. Handles are of 16 bit value.
 */
typedef struct  _PERFORMANCE_STATUS_ENTRY
{
   UINT8  entryType;
   UINT8  entryCount;
   UINT16 handleCount;

   #define MAX_STATUS_HANDLE  30
   UINT16 handleArray[ MAX_STATUS_HANDLE ];

} PERFORMANCE_STATUS_ENTRY;


typedef struct _IMMEDIATE_NOTIFY_ENTRY
{

   HEADER  hdr;
   UINT32  handle;
   UINT16  initiator;
   UINT16  InitSessionID;
   UINT16  ConnectionID;
   UINT16  TargSessionID;
   UINT16  inotStatus;
   #define INOT_STATUS_ABORT_TASK      0x0020
   #define INOT_STATUS_LOGGED_OUT      0x0029
   #define INOT_STATUS_RESTART_RECVD   0x0030
   #define INOT_STATUS_MSG_RECVD       0x0036
   #define INOT_STATUS_TSK_REASSIGN	   0x0037
   
   UINT16  taskFlags;
   #define TASK_FLAG_CLEAR_ACA         0x4000
   #define TASK_FLAG_COLD_RESET        0x2000
   #define TASK_FLAG_WARM_RESET        0x0800
   #define TASK_FLAG_LUN_RESET         0x1000
   #define TASK_FLAG_CLEAR_TASK_SET    0x0400
   #define TASK_FLAG_ABORT_TASK_SET    0x0200


   UINT32  refTaskTag;
   UINT8   lun[8];
   UINT32  inotTaskTag;
   UINT8   res3[RESERVED_BYTES_INOT];
} IMMEDIATE_NOTIFY_ENTRY ;

typedef struct _NOTIFY_ACK_ENTRY
{
   HEADER  hdr;
   UINT32  handle;
   UINT16  initiator;
   UINT16  res1;
   UINT16  flags;
   UINT8  	responseCode;
   UINT8	qualifier;
   UINT16  notAckStatus;
   UINT16  taskFlags;
   #define NACK_FLAG_RESPONSE_CODE_VALID 0x0010

   UINT32  refTaskTag;
   UINT8   lun[8];
   UINT32  inotTaskTag;
   UINT8   res3[RESERVED_BYTES_NOTACK];
} NOTIFY_ACK_ENTRY ;

typedef struct _ATIO_ENTRY
{
   HEADER  hdr;                           /* 00-03 */
   UINT32  handle;                        /* 04-07 */
   UINT16  initiator;                     /* 08-09 */
   UINT16  connectionID;                  /* 0A-0B */
   UINT32  taskTag;                       /* 0C-0f */
   UINT8   scsiCDB[IOCB_MAX_CDB_LEN];        /* 10-1F */
   UINT8   LUN[8];                        /* 20-27 */
   UINT8   cmdRefNum;                     /* 28 */

   UINT8   pduType;                       /* 29 */
   #define PDU_TYPE_NOPOUT                0x00
   #define PDU_TYPE_SCSI_CMD              0x01
   #define PDU_TYPE_SCSI_TASK_MNGMT_CMD   0x02
   #define PDU_TYPE_LOGIN_CMD             0x03
   #define PDU_TYPE_TEXT_CMD              0x04
   #define PDU_TYPE_SCSI_DATA             0x05
   #define PDU_TYPE_LOGOUT_CMD            0x06
   #define PDU_TYPE_SNACK                 0x10

   UINT16  atioStatus;                    /* 2A-2B */
   #define ATIO_CDB_RECVD                 0x003d

   UINT8   taskCode;                      /* 2C */
   #define ATIO_TASK_CODE_UNTAGGED        0x00
   #define ATIO_TASK_CODE_SIMPLE_QUEUE    0x01
   #define ATIO_TASK_CODE_ORDERED_QUEUE   0x02
   #define ATIO_TASK_CODE_HEAD_OF_QUEUE   0x03
   #define ATIO_TASK_CODE_ACA_QUEUE       0x04

   UINT8   reserved1;                     /* 2D */
   UINT16  reserved2;                     /* 2E-2F */
   UINT32  totalByteCnt;                  /* 30-33 */
   UINT32  cmdSeqNum;                     /* 34-37 */
   UINT64  reserved3;                     /* 38-3F */
} ATIO_ENTRY ;

typedef struct _CTIO3_ENTRY
{
   HEADER  hdr;                           /* 00-03 */
   UINT32  handle;                        /* 04-07 */
   UINT16  initiator;                     /* 08-09 */
   UINT16  connectionID;                  /* 0A-0B */
   UINT32  taskTag;                       /* 0C-0F */

   UINT8   flags;                         /* 10 */
   #define CTIO_FLAG_SEND_SCSI_STATUS     0x01
   #define CTIO_FLAG_TERMINATE_COMMAND    0x10
   #define CTIO_FLAG_FAST_POST            0x08
   #define CTIO_FLAG_FINAL_CTIO           0x80

   /*  NOTE:  Our firmware assumes that the CTIO_FLAG_SEND_DATA and CTIO_FLAG_GET_DATA flags are in the same  */
   /*         bit positions as the R and W bits in SCSI Command PDUs, so their values should not be changed!  */
   #define CTIO_FLAG_SEND_DATA            0x0040   /* (see note) Read Data Flag, send data to initiator       */
   #define CTIO_FLAG_GET_DATA             0x0020   /* (see note) Write Data Flag, get data from the initiator */

   UINT8   scsiStatus;                    /* 11 */
   UINT16  timeout;                       /* 12-13 */
   UINT32  offset;                        /* 14-17 */
   UINT32  r2tSN;                         /* 18-1B */
   UINT32  expCmdSN;                      /* 1C-1F */
   UINT32  maxCmdSN;                      /* 20-23 */
   UINT32  dataSN;                        /* 24-27 */
   UINT32  residualCount;                 /* 28-2B */
   UINT16  reserved;                      /* 2C-2D */
   UINT16  segmentCnt;                    /* 2E-2F */
   UINT32  totalByteCnt;                  /* 30-33 */
   DATA_SEG_A64 dataseg[COMMAND_SEG_A64]; /* 34-3F */
} CTIO3_ENTRY ;

typedef struct _CTIO4_ENTRY
{
   HEADER  hdr;                           /* 00-03 */
   UINT32  handle;                        /* 04-07 */
   UINT16  initiator;                     /* 08-09 */
   UINT16  connectionID;                  /* 0A-0B */
   UINT32  taskTag;                       /* 0C-0F */
   UINT8   flags;                         /* 10 */
   UINT8   scsiStatus;                    /* 11 */
   UINT16  timeout;                       /* 12-13 */
   UINT32  offset;                        /* 14-17 */
   UINT32  r2tSN;                         /* 18-1B */
   UINT32  expCmdSN;                      /* 1C-1F */
   UINT32  maxCmdSN;                      /* 20-23 */
   UINT32  dataSN;                        /* 24-27 */
   UINT32  residualCount;                 /* 28-2B */
   UINT16  reserved;                      /* 2C-2D */
   UINT16  segmentCnt;                    /* 2E-2F */
   UINT32  totalByteCnt;                  /* 30-33 */
   /* WE ONLY USE THE ADDRESS FROM THE FOLLOWING STRUCTURE THE COUNT FIELD IS RESERVED */
   DATA_SEG_A64 dataseg[COMMAND_SEG_A64]; /* 34-3F */
} CTIO4_ENTRY ;

typedef struct _CTIO5_ENTRY
{
   HEADER  hdr;                           /* 00-03 */
   UINT32  handle;                        /* 04-07 */
   UINT16  initiator;                     /* 08-09 */
   UINT16  connectionID;                  /* 0A-0B */
   UINT32  taskTag;                       /* 0C-0F */
   UINT8   response;                      /* 10 */
   UINT8   scsiStatus;                    /* 11 */
   UINT16  timeout;                       /* 12-13 */
   UINT32  reserved1;                     /* 14-17 */
   UINT32  expR2TSn;                      /* 18-1B */
   UINT32  expCmdSn;                      /* 1C-1F */
   UINT32  MaxCmdSn;                      /* 20-23 */
   UINT32  expDataSn;                     /* 24-27 */
   UINT32  residualCnt;                   /* 28-2B */
   UINT32  bidiResidualCnt;               /* 2C-2F */
   UINT32  reserved2;                     /* 30-33 */
   DATA_SEG_A64 dataseg[1];               /* 34-3F */
} CTIO5_ENTRY ;

typedef struct _CTIO6_ENTRY
{
   HEADER  hdr;                           /* 00-03 */
   UINT32  handle;                        /* 04-07 */
   UINT16  initiator;                     /* 08-09 */
   UINT16  connection;                    /* 0A-0B */
   UINT32  taskTag;                       /* 0C-0F */
   UINT16  flags;                         /* 10-11 */
   UINT16  timeout;                       /* 12-13 */
   UINT32  reserved1;                     /* 14-17 */
   UINT64  reserved2;                     /* 18-1F */
   UINT64  reserved3;                     /* 20-27 */
   UINT64  reserved4;                     /* 28-2F */
   UINT32  reserved5;                     /* 30-33 */
   DATA_SEG_A64 dataseg[1];               /* 34-3F */
} CTIO6_ENTRY ;

typedef struct _CTIO_STATUS_ENTRY
{
   HEADER  hdr;                           /* 00-03 */
   UINT32  handle;                        /* 04-07 */
   UINT16  initiator;                     /* 08-09 */
   UINT16  connectionID;                  /* 0A-0B */
   UINT32  taskTag;                       /* 0C-0F */
   UINT16  status;                        /* 10-11 */
   #define CTIO_STATUS_COMPLETE           0x0001
   #define CTIO_STATUS_ABORTED            0x0002
   #define CTIO_STATUS_DMA_ERROR          0x0003
   #define CTIO_STATUS_ERROR              0x0004
   #define CTIO_STATUS_INVALID_TAG        0x0008
   #define CTIO_STATUS_DATA_OVERRUN       0x0009
   #define CTIO_STATUS_CMD_TIMEOUT        0x000B
   #define CTIO_STATUS_PCI_ERROR          0x0010
   #define CTIO_STATUS_DATA_UNDERRUN      0x0015
   #define CTIO_STATUS_TARGET_RESET       0x0017
   #define CTIO_STATUS_NO_CONNECTION      0x0028
   #define CTIO_STATUS_LOGGED_OUT         0x0029
   #define CTIO_STATUS_CONFIG_CHANGED     0x002A
   #define CTIO_STATUS_UNACK_EVENT        0x0035
   #define CTIO_STATUS_INVALID_DATA_XFER  0x0036

   UINT16  timeout;                       /* 12-13 */
   UINT32  reserved1;                     /* 14-17 */
   UINT32  expR2TSN;                      /* 18-1B */
   UINT32  reserved2;                     /* 1C-1F */
   UINT32  reserved3;                     /* 20-23 */
   UINT64  expDataSN;                     /* 24-27 */
   UINT32  residualCount;                 /* 28-2B */
   UINT32  reserved4;                     /* 2C-2F */
   UINT64  reserved5;                     /* 30-37 */
   UINT64  reserved6;                     /* 38-3F */
} CTIO_STATUS_ENTRY ;

typedef struct _MAILBOX_ENTRY
{
   HEADER  hdr;
   UINT32  handle;
   UINT32  mbx[MAX_MBX_COUNT];
} MAILBOX_ENTRY ;

typedef struct MAILBOX_STATUS_ENTRY
{
   HEADER  hdr;
   UINT32  handle;
   UINT32  mbx[MAX_MBX_COUNT];
} MAILBOX_STATUS_ENTRY ;

/* Passthru structures */
typedef struct DSD
{
   UINT32  addr ;
   UINT32  byteCnt ;
} DSD_T ;

typedef struct _PASSTHRU0_ENTRY
{
   HEADER  hdr;
   UINT32  handle;
   UINT16  target;
   UINT16  connectionID;
   UINT16  controlFlags;
   UINT16  timeout;
   struct  DSD   sendBuffer[IOCB_MAX_DSD_CNT];
   UINT32  res1;
   struct  DSD   recvBuffer[IOCB_MAX_DSD_CNT];
   UINT8   res2[20];
} PASSTHRU0_ENTRY ;

typedef struct _PASSTHRU1_ENTRY
{
   HEADER  hdr;
   UINT32  handle;
   UINT16  target;
   UINT16  connectionID;

   UINT16  controlFlags;
   #define PT_FLAG_ETHERNET_FRAME         0x8000
   #define PT_FLAG_IP_DATAGRAM            0x4000
   #define PT_FLAG_TCP_PACKET             0x2000
   #define PT_FLAG_iSCSI_PDU              0x1000
   #define PT_FLAG_SEND_BUFFER            0x0200
   #define PT_FLAG_WAIT_4_REPONSE         0x0100
   #define PT_FLAG_NO_FAST_POST           0x0080

   UINT16  timeout;

   UINT64  outboundDSDlistAddr;
   UINT32  outboundByteCnt;
   UINT32  outboundSegmentCnt;

   UINT64  inboundDSDlistAddr;
   UINT32  inboundByteCnt;
   UINT32  inboundSegmentCnt;

} PASSTHRU1_ENTRY ;

typedef struct _PASSTHRU_STATUS_ENTRY
{
   HEADER  hdr;
   UINT32  handle;
   UINT16  target;
   UINT16  connectionID;
   UINT16  passThruStatus;
   #define PASSTHRU_STATUS_COMPLETE       0x0001
   #define PASSTHRU_STATUS_ERROR          0x0004
   #define PASSTHRU_STATUS_CMD_TIMEOUT    0x000B
   #define PASSTHRU_STATUS_PCI_ERROR      0x0010
   #define PASSTHRU_STATUS_NO_CONNECTION  0x0028


   UINT16  timeout;

   UINT64  res1;
   UINT32  outboundResidual;
   UINT32  res2;
   UINT64  res3;
   UINT32  inboundResidual;
   UINT32  res4;
} PASSTHRU_STATUS_ENTRY ;

typedef struct _ASYNCHMSG_ENTRY
{
   HEADER  hdr;
   UINT32  handle;
   UINT16  target;
   UINT16  connectionID;
   UINT8   lun[8];
   UINT16  iSCSIEvent;
   #define AMSG_iSCSI_EVENT_NO_EVENT                  0x0000
   #define AMSG_iSCSI_EVENT_TARG_RESET                0x0001
   #define AMSG_iSCSI_EVENT_TARGT_LOGOUT              0x0002
   #define AMSG_iSCSI_EVENT_CONNECTION_DROPPED        0x0003
   #define AMSG_ISCSI_EVENT_ALL_CONNECTIONS_DROPPED   0x0004

   UINT16  SCSIEvent;
   #define AMSG_NO_SCSI_EVENT                         0x0000
   #define AMSG_SCSI_EVENT                            0x0001

   UINT16  parameter1;
   UINT16  parameter2;
   UINT16  parameter3;
   UINT32  expCmdSn;
   UINT32  maxCmdSn;
   UINT16  senseDataCnt;
   UINT16  reserved;
   UINT32  senseData[IOCB_MAX_SENSEDATA_LEN];
} ASYNCHMSG_ENTRY ;

/* Timer entry structure, this is an internal generated structure
   which causes the QLA4000 initiator to send a NOP-OUT or the
   QLA4000 target to send a NOP-IN */

typedef struct _TIMER_ENTRY
{
   HEADER  hdr;                    /* 00-03 */

   UINT32  handle;                 /* 04-07 */
   UINT16  target;                 /* 08-09 */
   UINT16  connection_id;          /* 0A-0B */

   UINT8   control_flags;          /* 0C */

   /* STATE FLAGS FIELD IS A PLACE HOLDER. THE FW WILL SET BITS IN THIS FIELD AS THE
      COMMAND IS PROCESSED. WHEN THE IOCB IS CHANGED TO AN IOSB THIS FIELD WILL HAVE
      THE STATE FLAGS SET PROPERLY.
   */
   UINT8   state_flags;            /* 0D */
   UINT8   cmdRefNum;              /* 0E */
   UINT8   reserved1;              /* 0F */
   UINT8   cdb[IOCB_MAX_CDB_LEN];     /* 10-1F */
   UINT8   lun[8];                 /* 20-27 */
   UINT32  cmdSeqNum;              /* 28-2B */
   UINT16  timeout;                /* 2C-2D */
   UINT16  dataSegCnt;             /* 2E-2F */
   UINT32  ttlByteCnt;             /* 30-33 */
   DATA_SEG_A64 dataseg[COMMAND_SEG_A64];  /* 34-3F */

} TIMER_ENTRY;

#ifdef __cplusplus
}
#endif

#endif /* _QLA4X_FW_H */

