/********************************************************************************
*                  QLOGIC LINUX SOFTWARE
*
* QLogic ISP4xxx iSCSI driver
* Copyright (C) 2002 Qlogic Corporation
* (www.qlogic.com)
*
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2, or (at your option) any
* later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
**
******************************************************************************/

#ifndef _QLA4XXX_H           /* wrapper symbol for kernel use */
#define _QLA4XXX_H           /* subject to change without notice */

#if defined(__cplusplus)
extern "C" {
#endif

#define QLA4XXX_DRIVER_VERSION  "1.14"
#define DRIVER_NAME			"qla4000"

/*---------------------------------------------------------------------------*/

/*
 * Compile time Options:
 *            0 - Disable and 1 - Enable
 */

#define EXTENDED_DBG_ON_ERROR	0 /* Display more detailed debug msgs when error occurs, restore after recovery */
#define STOP_ON_ERROR               STOP_ON_ABORT | STOP_ON_RESET_DEVICE | STOP_ON_RESET_BUS | STOP_ON_RESET_FIRMWARE | STOP_ON_RESET_ADAPTER
#define STOP_ON_ABORT               0  /* Panic on abort */
#define STOP_ON_RESET_DEVICE        0  /* Panic on device reset */
#define STOP_ON_RESET_BUS           0  /* Panic on Bus reset */
#define STOP_ON_RESET_FIRMWARE      0  /* Panic on Reset firmware for a particular host adapter */
#define STOP_ON_RESET_ADAPTER       0  /* Panic on Hard reset of host adapter, write to bridge's PCI config space */
#define TRACK_SPURIOUS_INTRS	0
#define DISPLAY_SRBS_IN_PROC	0

/* 
 * The following compile time options are temporary, 
 * used for debug purposes only.
 */
#define IGNORE_LUN_18		0 /* Used for testing on storage box
					   * in the labe that has a bad lun 18 */
#define ISP_RESET_TEST		0 /* Issues BIG HAMMER (reset) every 3 seconds */

/*
 * The following compile time options are for features that are not
 * currently utilized in the driver, but may be needed in the future.
 */
#define STRING_STRING_COMMAND_LINE_OPTIONS	0 /* Used for options like verbose, etc. */

/*---------------------------------------------------------------------------*/

#ifndef KERNEL_VERSION
#  define KERNEL_VERSION(x,y,z) (((x)<<16)+((y)<<8)+(z))
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
#	error "This code supports kernel versions 2.4.0 and above"
#endif
/*---------------------------------------------------------------------------*/

#define MIN(x,y)			((x)<(y)?(x):(y))
#define MAX(x,y)			((x)>(y)?(x):(y))
#define SYS_DELAY(x)		udelay(x);barrier()
#define QLA4000_DELAY(sec)  	mdelay(sec * 1000)
#define NVRAM_DELAY() 		udelay(500) /* 2 microsecond delay */
//#define RESET_DELAY()		{int delay; for(delay=30*1000; delay!=0; delay-=500) mdelay(500);}  /* delay 30 seconds */
#define RESET_DELAY()		{int delay; for(delay=30; delay!=0; delay--) \
					{current->state = TASK_INTERRUPTIBLE; \
					schedule_timeout(1 * HZ);}}  /* delay 30 seconds */
	
#define MSB(x)     		(uint8_t)((uint16_t)(x) >> 8)
#define LSB(x)     		(uint8_t)(x)
#define MSW(x)		(uint16_t)((uint32_t)(x) >> 16)
#define LSW(x)     		(uint16_t)(x)
#if  BITS_PER_LONG <= 32
	#define LSL(x) 		(uint32_t)((uint32_t) x)
	#define MSL(x) 		(uint32_t)((uint32_t) 0x0)
	#define OFFSET(w)   	(((uint32_t) &w) & 0xFF)   /* 256 byte offsets */
#else
	#define LSL(x) 		(uint32_t)(0xffffffff & ((uint32_t)x))
	#define MSL(x) 		(uint32_t)(0xffffffff & (((uint32_t)x)>>32) )
	#define OFFSET(w)	(((uint64_t) &w) & 0xFF)   /* 256 byte offsets */
#endif

/*
 * I/O port macros
 */
#ifdef DEBUG
#	define RD_REG_BYTE(addr)         qla4000_getbyte((uint8_t *)addr)
#	define RD_REG_WORD(addr)         qla4000_getword((uint16_t *)addr)
#	define RD_REG_DWORD(addr)        qla4000_getdword((uint32_t *)addr)
#	define WRT_REG_BYTE(addr, data)  qla4000_putbyte((uint8_t *)addr, data)
#	define WRT_REG_WORD(addr, data)  qla4000_putword((uint16_t *)addr, data)
#	define WRT_REG_DWORD(addr, data) qla4000_putdword((uint32_t *)addr, data)
#else
#   define RD_REG_BYTE(addr)	 readb(addr)
#   define RD_REG_WORD(addr)         readw(addr)
#   define RD_REG_DWORD(addr)        readl(addr)
#   define WRT_REG_BYTE(addr, data)  writeb(data, addr)
#   define WRT_REG_WORD(addr, data)  writew(data, addr)
#   define WRT_REG_DWORD(addr, data) writel(data, addr)
#endif

/*---------------------------------------------------------------------------*/

#define STATIC static
/*
 * Return status codes for internal routines
 */
#define QLA_SUCCESS			0
#define QLA_ERROR			1


/*
 * Host adapter default definitions.
 */
#define MAX_BUSES       		1
#define MAX_TARGETS     		128
#define MAX_LUNS        		128
#define MAX_AEN_ENTRIES 		64 	/* same as EXT_DEF_MAX_AEN_QUEUE */
#define MAX_DDB_ENTRIES 		128	/* one ddb entry per target */
#define INVALID_ENTRY		0xFFFF

/*
 * Timers, Retry & Wait counts
 */
#define WDG_TIMER_INTERVAL 		1    	/* Watchdog timer interval (in seconds) */
#define MBOX_RETRY_COUNT		3	/* Max number of times to retry a mailbox command */
#define MBOX_WAIT_COUNT		30 	/* Max time (in seconds) to wait for a mailbox command to complete */
#define INIT_WAIT_COUNT		60 	/* Max time (in seconds) to wait for the adapter to initialize */
#define MAX_REQS_SERVICED_PER_INTR	16
#define RESET_TARGET_DELAY          20      /* (in seconds) */
#define RESET_FIRMWARE_DELAY        30      /* (in seconds) */

#define MAX_RELOGIN_RETRIES		8
#define RELOGIN_TIMEOUT 		20	/* Relogin timeout (time between relogins) */

#define MBOX_AEN_REG_COUNT		4	/* Number of mailbox registers used for AENs */
#define MAX_LINKED_CMDS_PER_LUN	3
#define MAX_OS_CDB_LENGTH 		MAX_COMMAND_SIZE /* Max CDB length supported by Linux (12)*/
#define INTERNAL_PASSTHRU_TIMEOUT	15
#define QLA_CMD_TIMER_DELTA		2	/* Number of seconds to subtract for internal command timer */
#define EXTEND_CMD_TIMEOUT		60	/* Amount of time to extend the OS timeout */

/*
 * Under heavy I/O on SMP systems (8-way and IA64) with many command
 * timeouts, the scsi mid-layer will sometimes not wake-up the
 * error-handling thread when an error-condition occurs.
 *
 * This workaround if enabled will wakeup the error-handler if it is
 * stuck in this condition for sixty seconds.
 *
 */
#undef EH_WAKEUP_WORKAROUND
//#undef EH_WAKEUP_WORKAROUND_REDHAT
#define  EH_WAKEUP_WORKAROUND_REDHAT




/*
 * Buffer sizes
 */
#define REQUEST_QUEUE_DEPTH       	1024    /* Max number of request queue entries. */
#define RESPONSE_QUEUE_DEPTH      	64      /* Max number of response queue entries. */

#define DMA_BUFFER_SIZE 		512	/* Size (in bytes) of general DMA buffer */
#define SRB_POOL_SIZE		REQUEST_QUEUE_DEPTH

/*---------------------------------------------------------------------------*/

/*
 * SCSI definitions not defined in Linux's scsi.h
 */

/* The SCSISTAT values are defined in scsi.h, 
 * but the values are shifted by one bit.
 * I re-define them here without bit shifting 
 * to minimize confusion */
#define SCSISTAT_GOOD		0x00
#define SCSISTAT_CHECK_CONDITION	0x02
#define SCSISTAT_CONDITION_GOOD	0x04
#define SCSISTAT_BUSY		0x08
#define SCSISTAT_INTERMEDIATE_GOOD  0x10
#define SCSISTAT_INTERMEDIATE_C_GOOD  0x14
#define SCSISTAT_RESERVATION_CONFLICT 0x18
#define SCSISTAT_COMMAND_TERMINATED   0x22
#define SCSISTAT_QUEUE_FULL           0x28

#define SENSE_KEY(sense_data) (sense_data[2] & 0x0f)
#define ASC(sense_data) sense_data[12]
#define ASCQ(sense_data) sense_data[13]

#define REPORT_LUNS			0xA0

/* SAM-II compliant lun structure */
typedef struct
{
    uint8_t bus_identifier:6;
	uint8_t address_method:2;

	uint8_t single_level_lun;
	uint16_t second_level_lun;
	uint16_t third_level_lun;
	uint16_t fourth_level_lun;
} single_level_lun_t;

typedef struct
{
    uint32_t lun_list_length;
	uint8_t reserved[4];
    single_level_lun_t lun[MAX_LUNS];
} report_luns_t;

typedef struct
{
    uint8_t data[4];
} inquiry_t;

/*---------------------------------------------------------------------------*/

/*
 * LUN structure
 */
typedef struct
{
	uint8_t  	lun_state;		/* 00 x00 */
	#define LS_LUN_READY		0	/* LUN is ready to accept commands */
	#define LS_LUN_SUSPENDED	1	/* LUN is suspended until  */
	#define LS_LUN_RETRY		2
	#define LS_LUN_TIMEOUT		3
	#define LUN_STATE_TBL()		  \
	{				  \
		"READY"			, \
            "SUSPENDED"		, \
            "RETRY"			, \
            "TIMEOUT"		, \
		NULL			  \
	}
	
	uint8_t		out_count;		/* 01 x01 */
	uint8_t 	lun;			/* 02 x02 */

	uint8_t 	retry_count;		/* 03 x03 */
	uint8_t 	max_retry_count;	/* 04 x04 */
	uint8_t 	max_recovery_retries;	/* 05 x05 */
	uint8_t		reserved[2];		/* 06 x06 */

	atomic_t	suspend_timer;		/* 08 x08 */
	struct list_head list_entry;		/* 12 x0c For suspended lun list */
	spinlock_t	lun_lock;		/* 20 x14 */
} lun_entry_t;

/* Never set this to Zero */
#define SUSPEND_SECONDS	6		
#define SUSPEND_RETRIES	1

/*---------------------------------------------------------------------------*/

/*
 * Device Database (DDB) structure
 */

typedef struct
{
	struct list_head   list_entry;		/* 00 x00 */
	uint16_t      	bus;                    /* 08 x08 */
	uint16_t      	target;                 /* 10 x0a */
	uint16_t      	fw_ddb_index;		/* 12 x0c */
	uint16_t      	out_count;		/* 14 x0e */

	/* refer to MBOX_CMD_GET_DATABASE_ENTRY for device_state definitions   */
	uint32_t      	device_state;           /* 16 x10 */
	uint32_t        CmdSn;                  /* 20 x14 */
	uint16_t      	target_session_id;      /* 24 x18 */
	uint16_t      	connection_id;          /* 26 x1a */
	uint16_t      	exe_throttle;           /* 28 x1c */
	uint16_t      	task_mgmt_timeout;	/* 30 x1e */
	uint8_t       	reserved;    		/* 32 x20 */
	uint8_t       	num_valid_luns;		/* 33 x21 */
	uint16_t	relogin_timeout;     /* 34 x22 Max time to wait for relogin */
	atomic_t	port_down_timer;        /* 36 x24 Device down time */
	atomic_t	relogin_timer;          /* 40 x28 Wait time for relogin to fail */
	atomic_t	relogin_retry_count;    /* 44 x2c Wait time between relogins */
	atomic_t       	state;			/* 48 x30 */
	#define DEV_STATE_DEAD		0	/* We can no longer talk to this device */
	#define DEV_STATE_ONLINE	1	/* Device ready to accept commands */
	#define DEV_STATE_MISSING	2	/* Device logged off, trying to re-login */
	#define DEV_STATE_TBL()		  \
	{				  \
		"DEAD"			, \
            "ONLINE"		, \
            "MISSING"		, \
		NULL			  \
	}
	uint32_t      	flags;           	/* 52 x34 */
	#define DEV_FLAG_RELOGIN	0	
	#define DEV_FLAG_RESET		1	

	lun_entry_t   	*lun_table[MAX_LUNS];  	/* 56 x38*/
} ddb_entry_t;                   		

/*---------------------------------------------------------------------------*/

/*
 * Asynchronous Event Queue structure
 */
typedef struct
{
	uint32_t mbox_sts[MBOX_AEN_REG_COUNT];
}aen_t;

/*
 * DMA buffer structure
 */
typedef struct {
	    void      *virt_addr;
	    dma_addr_t phys_addr;
	    spinlock_t spinlock;
	    uint32_t   buf_len;
} dma_buf_t;

/* We use the Scsi_Pointer structure that's included with each command
 * SCSI_Cmnd as a scratchpad for our SRB.
 *
 * SCp will always point to the SRB structure (defined in qla4xxx.h).
 * It is defined as follows: (see srb_t definition)
 *  - SCp.ptr  -- > pointer back to the cmd
 *  - SCp.this_residual --> used as forward pointer to next srb
 *  - SCp.buffer --> used as backward pointer to next srb
 *  - SCp.buffers_residual --> used as flags field
 *  - SCp.have_data_in --> not used
 *  - SCp.sent_command --> not used
 *  - SCp.phase --> not used
 */

//#define CMD_SP(Cmnd)	(&(Cmnd)->SCp)

#define CMD_SP(Cmnd)	    ((Cmnd)->SCp.ptr)
/* Additional fields used by ioctl passthru */
#define CMD_PASSTHRU_TYPE(Cmnd) ((int) (Cmnd)->SCp.buffer)
#define CMD_COMPL_STATUS(Cmnd)  ((Cmnd)->SCp.this_residual)
#define CMD_RESID_LEN(Cmnd)     ((Cmnd)->SCp.buffers_residual)
#define CMD_SCSI_STATUS(Cmnd)   ((Cmnd)->SCp.Status)
#define CMD_ACTUAL_SNSLEN(Cmnd) ((Cmnd)->SCp.have_data_in)
#define CMD_HOST_STATUS(Cmnd)   ((Cmnd)->SCp.Message)
#define CMD_ISCSI_RESPONSE(Cmnd)((Cmnd)->SCp.sent_command)
#define CMD_STATE_FLAGS(Cmnd)   ((Cmnd)->SCp.phase)

#define CMD_XFRLEN(Cmnd)	(Cmnd)->request_bufflen
#define CMD_CDBLEN(Cmnd)	(Cmnd)->cmd_len
#define CMD_CDBP(Cmnd)	(Cmnd)->cmnd
#define CMD_SNSP(Cmnd)	(Cmnd)->sense_buffer
#define CMD_SNSLEN(Cmnd)	(sizeof (Cmnd)->sense_buffer)
#define CMD_RESULT(Cmnd)	((Cmnd)->result)
#define CMD_HANDLE(Cmnd)	((Cmnd)->host_scribble)
#define CMD_TIMEOUT(Cmnd) 	((Cmnd)->timeout_per_command)

#define SCSI_BUS_32(scp)	((scp)->channel)
#define SCSI_TCN_32(scp)	((scp)->target)
#define SCSI_LUN_32(scp)	((scp)->lun)

/*
 * SCSI Request Block structure  (sp)  that is placed
 * on cmd->SCp location of every I/O     [We have 22 bytes available]
 */
typedef struct _srb_t
{
    struct list_head   list_entry;	/* (8)   */
    uint8_t     flags;              /* (1) Status flags. */
	#define SRB_TIMEOUT          0x01   /* Command timed out */
	#define SRB_ABORT_PENDING    0x02   /* Command abort sent to device */
	#define SRB_ABORTED          0x04   /* Command aborted command already */
	#define SRB_DMA_VALID        0x08   /* The DMA Buffer has been mapped */
	#define SRB_GOT_SENSE        0x10   /* Sense data was recieved */

    uint8_t     state;              /* (1) Status flags. */
	#define SRB_NO_QUEUE_STATE	 0
	#define SRB_FREE_STATE		 1
	#define SRB_PENDING_STATE	 2
	#define SRB_ACTIVE_STATE	 3
	#define SRB_ACTIVE_TIMEOUT_STATE 4
	#define SRB_RETRY_STATE	 	 5
	#define SRB_DONE_STATE	 	 6
	#define SRB_STATE_TBL()	  	  \
	{	    			  \
            "NO_QUEUE"	        , \
            "FREE"		        , \
            "PENDING"	        , \
		"ACTIVE"	        , \
		"ACTIVE_TIMEOUT"        , \
		"RETRY"	        	, \
		"DONE"	        	, \
		NULL			  \
	}
	
	uint8_t     entry_count;	/* (1) number of request queue entries used */
	uint8_t     reserved1;
	uint16_t    reserved2;
	uint16_t    active_array_index;

    Scsi_Cmnd  *cmd;                /* (4) SCSI command block */
    dma_addr_t  saved_dma_handle;   /* (4) for unmap of single transfers */
	ddb_entry_t *ddb_entry;
	lun_entry_t *lun_entry;
    struct      timer_list   timer;  /* used to timeout command */

	#if 1
	//#ifdef DEBUG
	uint8_t     os_timeout;
	uint8_t     iocb_timeout;
	uint8_t     driver_timeout;
	uint8_t	    task_mgmt_timeout;
	#else
	uint8_t     reserved3[4];
	#endif
	uint16_t    iocb_cnt;
	uint16_t    cc_stat;
	u_long      r_start;  /* Time we recieve a cmd from OS*/
	u_long      u_start;  /* Time when we handed the cmd to F/W */
} srb_t;

/*
 * Linux Host Adapter structure
 */
typedef struct scsi_qla_host {
    /* Linux adapter configuration data */
    struct Scsi_Host *host;	/* pointer to host data */
    struct scsi_qla_host *next;

	//FIXME: Get rid of adapter state.  Use adapter flag to check for reset
	atomic_t        adapter_state;
	#define AS_DOWN   	0               // Adapter not ready
	#define AS_RESET  	1               // Adapter being reset
	#define AS_READY  	2               // Adapter ready
	#define ADAPTER_STATE_TBL()	\
	{				\
		"DOWN"		       ,\
		"RESET"		       ,\
		"READY"		       ,\
		NULL			\
	}
	
	uint32_t	firmware_state;

	uint32_t flags;
	#define AF_INIT_DONE		0  /* 0x0001 */
    #define AF_MBOX_COMMAND 	1  /* 0x0002 */
	#define AF_MBOX_COMMAND_DONE 	2  /* 0x0004 */
	#define AF_SCHEDULE_DPC		3  /* 0x0008 */
	#define AF_DPC_SCHEDULED	4  /* 0x0010 */
	#define AF_INTERRUPTS_ON	5  /* 0x0020 Not Used */
	#define AF_IN_ISR      		6  /* 0x0040 Not Used */

	uint32_t dpc_flags;
	#define DPC_RESET_HA		1  /* 0x0002 */
	#define DPC_RESET_DEVICE	2  /* 0x0004 */
	#define DPC_RETRY_RELOGIN	3  /* 0x0008 */
	#define DPC_RESET_HA_DESTROY_DDB_LIST 4 
	
	uint16_t    iocb_cnt;
	uint16_t    iocb_hiwat;

	/* pci information */
    uint32_t	pci_address;
    struct pci_dev  *pdev;
	struct pci_dev *pci_brgdev; /* Pointer to Intel PCI Brigde on QLA4000*/
    uint32_t        pci_device_id;
    uint8_t         pci_bus;
    uint8_t         pci_device_fn;

    uint8_t         devnum;			/* Index into board structure for type of QLA4xxx adapter supported */
    uint8_t         ports;			/* Maximum number of busses/ports supported */
    uint16_t   	host_no;		/* adapter instance w.r.t. all scsi hosts in OS */
    uint16_t   	instance;               /* adapter instance w.r.t. this driver */

    uint32_t	memory_mapped_address;
    ISP_REG_T     	*base_address;	/* Base Memory-mapped I/O address */

	/* Counters for general statistics */
	uint64_t	adapter_error_count;
	uint64_t	device_error_count;
	uint64_t	total_io_count;
	uint64_t	total_mbytes_xferred;
    uint64_t   	isr_count;	/* Interrupt count */
	uint64_t	link_failure_count;
	uint64_t	invalid_crc_count;

    uint32_t   	spurious_int_count;
	uint32_t	aborted_io_count;
	uint32_t	io_timeout_count;
	uint32_t	mailbox_timeout_count;
	uint32_t	seconds_since_last_intr;
	
	/* Info Needed for Management App */
		// From GetFwVersion
	uint32_t firmware_version[2];
	uint32_t patch_number;
	uint32_t build_number;
		// From Init_FW
	uint32_t firmware_options;
	uint8_t ip_address[16];

	/* Passthru cmd/completion */
	struct semaphore ioctl_cmpl_sem;
	struct timer_list ioctl_cmpl_timer;
	uint32_t	ioctl_tov;
	uint8_t	ioctl_scsi_pass_in_progress;

	/* Linux kernel thread */
	uint8_t dpc_active;      	/* DPC routine is active */
	struct task_struct *dpc_handler; /* kernel thread */
	struct semaphore *dpc_wait;    /* DPC waits on this semaphore */
	struct semaphore *dpc_notify;  /* requester waits for DPC on this
	                                  * semaphore */

    /* Linux timer thread */
	struct timer_list watchdog_timer;

	/* Recovery Timers */
	uint32_t port_down_retry_count;
	atomic_t check_relogin_timeouts;

#if ISP_RESET_TEST
	uint32_t isp_reset_timer;
#endif
	int 	 eh_start; /* To wake up the mid layer error handler thread */ 	

	/* This spinlock must be held with irqs disabled in order to access
	 * the pending, retry and free srb queues. 
	 *  
	 * The list_lock spinlock is of lower priority than the io_request lock.
	 *-----------------------------------------------------------------------*/
	spinlock_t	list_lock;

	/* internal srb queues */
	struct list_head pending_srb_q;
	struct list_head retry_srb_q;
	struct list_head free_srb_q;
	uint16_t pending_srb_q_count;
	uint16_t retry_srb_q_count;
	uint16_t free_srb_q_count;
	uint16_t num_srbs_allocated;

	/* This spinlock must be held with irqs disabled in order to access
	 * the done srb queue and suspended_lun	queue.
	 * 
	 * The adapter_lock spinlock is of lower priority than the io_request lock.
	 *-----------------------------------------------------------------------*/
	spinlock_t adapter_lock;

	/* Done queue
	 * In order to avoid deadlocks with the list_lock,
	 * place all srbs to be returned to OS on this list.
	 * After the list_lock is released, return all of
	 * these commands to the OS */

	struct list_head done_srb_q;
	uint16_t	done_srb_count;

	/* Suspended LUN queue (uses adapter_lock) */
	struct list_head suspended_lun_q;
	uint32_t suspended_lun_q_count;


	/* This spinlock is used to protect "io transactions", you must	
	 * aquire it before doing any IO to the card, eg with RD_REG*() and
	 * WRT_REG*() for the duration of your entire command transaction.
	 * It is also used to protect the active_srb_array.
	 *
	 * This spinlock is of lower priority than the io request lock.
	 *-----------------------------------------------------------------------*/
	spinlock_t	hardware_lock;
	
	/* Active array */
	srb_t *active_srb_array[QLA4000_MAX_CMDS_TO_RISC];
	uint16_t active_srb_count;
	uint16_t current_active_index;

	/* request and response queue variables */
	dma_buf_t	req_queue;		/* Physical & Virtual addresses*/
	dma_buf_t	rsp_queue;

	QUEUE_ENTRY     *request_ptr;		/* Current address. */
    QUEUE_ENTRY     *response_ptr;

	uint16_t        request_in; 		/* Current indexes. */
	uint16_t        request_out;
    uint16_t        response_in;
    uint16_t        response_out;

    uint16_t        req_q_count;		/* Number of available entries. */

	/* This spinlock must be held with irqs disabled in order to access
	 * the aen queue.
	 * 
	 * The aen_q_spinlock is of lower priority than the io_request lock.
	 *-----------------------------------------------------------------------*/
	spinlock_t	aen_q_spinlock;

	/* aen queue variables */
	uint16_t	aen_q_count;		/* Number of available aen_q entries */
	uint16_t	aen_in;			/* Current indexes */
	uint16_t	aen_out;
	aen_t 		aen_q[MAX_AEN_ENTRIES];

    /* temporary mailbox status registers */
	volatile uint8_t  mbox_status_count;
    volatile uint32_t mbox_status[MBOX_REG_COUNT];

	/* This semaphore protects several threads to do mailbox commands
	 * concurrently.
     */
	struct semaphore	mbox_sem;
	wait_queue_head_t mailbox_wait_queue;

	/* This semaphore protects several threads to do ioctl commands
	 * concurrently.
     */
	struct semaphore	ioctl_sem;

	/* General DMA buffer */
	dma_buf_t dma_buf;

	/* local device database list (contains internal ddb entries)*/
	struct list_head ddb_list;

	/* Map ddb_list entry by SCSI target id */
    ddb_entry_t *target_map[MAX_TARGETS];

	/* Map ddb_list entry by FW ddb index */
	ddb_entry_t *fw_ddb_index_map[MAX_DDB_ENTRIES];

}scsi_qla_host_t;

//----------------------------------------------------------------------------



/*
 * Prototypes for Global Functions
 */
scsi_qla_host_t *qla4000_get_adapter_handle(uint16_t instance);
uint8_t qla4000_mailbox_command(scsi_qla_host_t *ha, uint8_t inCount, uint8_t outCount, uint32_t *mbx_cmd, uint32_t *mbx_sts);
uint8_t qla4000_alloc_physical_memory(struct pci_dev *pdev, dma_buf_t *dma_buf, uint32_t size);
inline void qla4000_free_physical_memory(struct pci_dev *pdev, dma_buf_t *dma_buf);
uint8_t qla4000_resize_dma_buf(scsi_qla_host_t *ha, dma_buf_t *dma_buf, uint32_t size);
uint8_t qla4000_get_ddb_entry(scsi_qla_host_t *, uint16_t, DEVDB_ENTRY *,  uint32_t *, uint32_t *, uint32_t *);
uint8_t qla4000_set_ddb_entry(scsi_qla_host_t *, uint16_t, DEVDB_ENTRY *);
inline uint32_t qla4000_get_hba_count(void);
ddb_entry_t *qla4000_lookup_ddb_by_SCSIID(scsi_qla_host_t *ha, uint32_t bus, uint32_t target);
ddb_entry_t *qla4000_lookup_ddb_by_fw_index(scsi_qla_host_t *ha, uint32_t fw_ddb_index);
int         qla4000_queuecommand(Scsi_Cmnd *, void (* done)(Scsi_Cmnd *));
uint8_t qla4000_login_device(scsi_qla_host_t *ha, uint16_t fw_ddb_index, uint16_t connection_id);
uint8_t qla4000_logout_device(scsi_qla_host_t *ha, uint16_t fw_ddb_index, uint16_t connection_id);
uint8_t qla4000_send_command_to_isp(scsi_qla_host_t *ha, srb_t *srb);
void qla4000_cmd_timeout(srb_t *srb);
void qla4000_add_timer_to_cmd(srb_t *srb, int timeout);
void qla4000_delete_timer_from_cmd(srb_t *srb);

#if defined(__cplusplus)
}
#endif

#endif // _QLA4XXX_H

/*
 * Overrides for Emacs so that we get a uniform tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
