/************************************************************************/
/*                                                                      */
/* Source File Name: QLINIOCTL.H                                        */
/*                                                                      */
/* Function: Defines the internal IOCTL interface to QLogic drivers     */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*                              NOTICE                                  */
/*                                                                      */
/*               COPYRIGHT 2000-2002 QLOGIC CORPORATION                 */
/*                        ALL RIGHTS RESERVED                           */
/*                                                                      */
/*     This computer program  is  CONFIDENTIAL  and a TRADE SECRET      */
/*     of  QLOGIC CORPORATION.  The  receipt  or possesion of this      */
/*     program does not convey any rights to reproduce or disclose      */
/*     its contents, or to manufacture, use, or sell anything that      */
/*     it may describe, in whole or in part, without the  specific      */
/*     written consent of QLOGIC CORPORATION.  Any reproduction of      */
/*     this program  without the express written consent of QLOGIC      */
/*     CORPORATION  is a violation  of the copyright laws  and may      */
/*     subject you to criminal prosecution.                             */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/* !!!!!!!!!!!!!!!!! VERY VERY VERY ... IMPORTANT !!!!!!!!!!!!!!!!!!!!!!*/
/* This file is formated to confirm to the guidelines for Solaris	*/
/* Please try to follow these standards before making any further	*/
/* changes to it.							*/
/* Here are few important ones:						*/
/* 1. Use 8 spaces for tab and always include tabs in your file. Some	*/
/*    editors allow you replace tabs with spaces while saving the file  */
/*    Please do not use or disable that option.				*/
/* 2. Use 8 spaces for indentation					*/
/* 3, Do not use more than 80 characters in a single line		*/
/* 4. while defining structures use the following format:		*/
/*									*/
/*	typedef<Space>struct<Space>Name<space>{				*/
/*	<tab>TypeOfElement<tab>ElementName;<tabs>			*/
/*	<tab>TypeOfElement<tab>ElementName;<tabs>			*/
/*	:								*/
/*	}<Space>StructureName,<Space>AnyMoreNames;			*/
/*									*/
/*	The number of Tabs after ElementName should be such that	*/
/*	comments for all the Element Names are aligned			*/
/*									*/
/* 5. Please consult Solaris Folks if you have any more questions. The	*/
/*    guidelines should be sufficient for almost all changes		*/
/************************************************************************/


/************************************************************************/
/* Revision history:                                                    */
/*                                                                      */
/*  01/07/02 MRS    Created                                             */
/*                                                                      */
/************************************************************************/


#ifndef _QLINIOCT_H_
#define _QLINIOCT_H_

/*
   Ioctl
*/

/*
  General
*/

/*
 * Command Codes definitions
 */
#define INT_CC_GET_DATA                         0xF0002001
#define INT_CC_SET_DATA                         0xF0002002
#define INT_CC_DIAG_PING                        0xF0002003
#define INT_CC_ISCSI_LOOPBACK                   0xF0002004
#define INT_CC_HBA_RESET                        0xF0002005
#define INT_CC_COPY_FW_FLASH                    0xF0002006
#define INT_CC_LOGOUT_ISCSI                     0xF0002007


/*
 * Sub codes for Get Data.
 * Use in combination with INT_GET_DATA as the ioctl code
 */
#define INT_SC_GET_FLASH                        1

/*
 * Sub codes for Set Data.
 * Use in combination with INT_SET_DATA as the ioctl code
 */
#define INT_SC_SET_FLASH                        1
                             
/*
 * ***********************************************************************
 * INIT_FW_ISCSI_ALL
 * ***********************************************************************
 */
typedef struct _INT_INIT_FW_ISCSI_ALL
{
	UINT8	Version;					/* 1   */
	UINT8	Reserved0;					/* 1   */
	UINT16	FWOptions;					/* 2   */
	UINT16	exeThrottle;					/* 2   */
	UINT8	retryCount;					/* 1   */
	UINT8	retryDelay;					/* 1   */
	UINT16	Reserved1;					/* 2   */
	UINT16	addFWOptions;					/* 2   */
	UINT16	wakeupThreshold;				/* 2   */
	UINT16	Reserved2;					/* 2   */
	UINT16	ReqQOutPtr;					/* 2   */
	UINT16	RespQInPtr;					/* 2   */
	UINT16	ReqQLen;					/* 2   */
	UINT16	RespQLen;					/* 2   */
	UINT32	ReqQAddr[2];					/* 8   */
	UINT32	RespQAddr[2];					/* 8   */
	UINT32	IntRegBufAddr[2];				/* 8   */
	UINT16	iSCSIOptions;					/* 2   */
	UINT16	TCPOptions;					/* 2   */
	UINT16	IPOptions;					/* 2   */
	UINT16	maxPduSize;					/* 2   */
	UINT16	recvMarkerInt;					/* 2   */
	UINT16	sendMarkerInt;					/* 2   */
	UINT16	initMarkerlessInt;				/* 2   */
	UINT16	firstBurstSize;					/* 2   */
	UINT16	logoutMinTime;					/* 2   */
	UINT16	logoutMaxTime;					/* 2   */
	UINT16	maxOutstandingR2T;				/* 2   */
	UINT16	keepAliveTimeout;				/* 2   */
	UINT16	portNumber;					/* 2   */
	UINT16	maxBurstSize;					/* 2   */
	UINT16	MSS;						/* 2   */
	UINT16	Reserved4;					/* 6   */
	UINT8	IPAddr[16];					/* 16  */
	UINT8	SubnetMask[16];					/* 16  */
	UINT8	IPGateway[16];					/* 16  */
	UINT8	DNSsvrIP[16];					/* 16  */
	UINT8	Alias[EXT_DEF_ISCSI_ALIAS_LEN];			/* 32  */
	UINT8	targetAddr[EXT_DEF_ISCSI_TADDR_SIZE];		/* 32  */
	UINT8	SendScale;					/* 1   */
	UINT8	RecvScale;					/* 1   */
	UINT8	TCPFlags;					/* 1   */
	UINT8	TypeOfService;					/* 1   */
	UINT8	Time2Live;					/* 1   */
	UINT8	Reserved3;					/* 1   */
	UINT16	VLANPriority;					/* 2   */
	UINT8	Reserved6[40];					/* 40  */
	UINT8	iSCSIName[EXT_DEF_ISCSI_NAME_LEN];		/* 256 */
} INT_INIT_FW_ISCSI_ALL, *PINT_INIT_FW_ISCSI_ALL;		/* 512 */                                      
                                          
/*
 * ***********************************************************************
 * INT_DEVICE_ENTRY_ISCSI_ALL
 * ***********************************************************************
 */
typedef struct _INT_DEVICE_ENTRY_ISCSI_ALL
{
	UINT8	Options;					/* 1 */
	UINT8	Control;					/* 1 */
	UINT16	exeThrottle;					/* 2 */
	UINT16	exeCount;					/* 2 */
	UINT8	retryCount;					/* 1 */
	UINT8	retryDelay;					/* 1 */
	UINT16	iSCSIOptions;					/* 2 */
	UINT16	TCPOptions;					/* 2 */
	UINT16	IPOptions;					/* 2 */
	UINT16	maxPDUSize;					/* 2 */
	UINT16	RecvMarkerInterval;				/* 2 */
	UINT16	SendMarkerInterval;				/* 2 */
	UINT16	initMarkerlessInt;				/* 2 */
	UINT16	firstBurstSize;					/* 2 */
	UINT16	logoutMinTime;					/* 2 */
	UINT16	logoutMaxTime;					/* 2 */
	UINT16	maxOutstandingR2T;				/* 2 */
	UINT16	keepAliveTimeout;				/* 2 */
	UINT8	InitiatorSessID[EXT_DEF_ISCSI_ISID_SIZE];	/* 6 */
	UINT16	TargetSessID;					/* 2 */
	UINT16	portNumber;					/* 2 */
	UINT16	maxBurstSize;					/* 2 */
	UINT16	taskMngmntTimeout;				/* 2 */
	UINT16	Reserved0;					/* 2 */
	UINT8	IPAddress[16];					/* 16  */
	UINT8	Alias[EXT_DEF_ISCSI_ALIAS_LEN];			/* 32  */
	UINT8	targetAddr[EXT_DEF_ISCSI_TADDR_SIZE];		/* 32  */
	UINT8	userID[EXT_DEF_ISCSI_USER_ID_SIZE];		/* 32  */
	UINT8	password[EXT_DEF_ISCSI_PASSWORD_SIZE];		/* 32  */
	UINT8	iSCSIName[EXT_DEF_ISCSI_NAME_LEN];		/* 256 */   
	UINT16	ddbLink;					/* 2   */
	UINT16	Reserved1;					/* 2   */
} INT_DEVICE_ENTRY_ISCSI_ALL, *PINT_DEVICE_ENTRY_ISCSI_ALL;	/* 452 */

/*
 * ****************************************************************************
 * INT_DEVDDB_ENTRY
 * ****************************************************************************
 */

typedef struct _FLASH_DEVDB_ENTRY
{
	INT_DEVICE_ENTRY_ISCSI_ALL	entryData;		/* 0-1C3   */
	UINT8				RES0[0x38];		/* 1C4-1FB */
	UINT16				ddbValidCookie;		/* 1FC-1FD */
	UINT16				ddbValidSize;		/* 1FE-1FF */
} FLASH_DEVDB_ENTRY, *PFLASH_DEVDB_ENTRY;

/*
 * ****************************************************************************
 * INT_FLASH_INITFW
 * ****************************************************************************
 */

typedef struct _FLASH_INITFW
{
	INT_INIT_FW_ISCSI_ALL	initFWData;
	UINT32			validCookie;
} FLASH_INITFW, *PFLASH_INITFW;            


/*
 * ***********************************************************************
 * INT_ACCESS_FLASH
 * ***********************************************************************
 */

#define INT_DEF_AREA_TYPE_FW_IMAGE1		0x01
#define INT_DEF_AREA_TYPE_FW_IMAGE2		0x02
#define INT_DEF_AREA_TYPE_DRIVER		0x03
#define INT_DEF_AREA_TYPE_DDB			0x04
#define INT_DEF_AREA_TYPE_INIT_FW		0x05
#define INT_DEF_AREA_TYPE_SYS_INFO		0x06

#define INT_DEF_FLASH_BLK_SIZE			0x4000
#define INT_DEF_FLASH_PHYS_BLK_SIZE		0x20000

#define INT_ISCSI_FW_IMAGE1_FLASH_OFFSET	0x01800000
#define INT_ISCSI_FW_IMAGE2_FLASH_OFFSET	0x01000000
#define INT_ISCSI_SYSINFO_FLASH_OFFSET		0x02000000
#define INT_ISCSI_DRIVER_FLASH_OFFSET		0x03000000
#define INT_ISCSI_INITFW_FLASH_OFFSET		0x04000000
#define INT_ISCSI_DDB_FLASH_OFFSET		0x05000000
                                                     
/* WRITE_FLASH option definitions */
#define INT_WRITE_FLASH_OPT_CLEAR_REMAINING	0 /* Write data to FLASH but
						     do not Commit any data 
						     not written before 
						     commit will be cleared 
						     (set to 0xFF)	*/
#define INT_WRITE_FLASH_OPT_COMMIT_DATA		2 /* Commit (Burn) data to 
						     FLASH */


typedef struct _INT_ACCESS_FLASH {
	UINT32	AreaType;					/* 4   */
	UINT32	DataLen;					/* 4   */
	UINT32	DataOffset;					/* 4   */
	UINT8	FlashData[INT_DEF_FLASH_BLK_SIZE];		/* 0x4000 */
	UINT32	Options;					/* 4   */
} INT_ACCESS_FLASH, *PINT_ACCESS_FLASH;				/* 0x4010 */

/*
 * ****************************************************************************
 * INT_FLASH_DRIVER_PARAM
 * ****************************************************************************
 */

typedef struct _INT_FLASH_DRIVER_PARAM {
	UINT16	DiscoveryTimeOut;				/* 2   */
	UINT16	PortDownTimeout;				/* 2   */
	UINT32	Reserved[32];					/* 128 */
} INT_FLASH_DRIVER_PARAM, *PINT_FLASH_DRIVER_PARAM;		/* 132 */


#define VALID_FLASH_INITFW  0x11BEAD5A
                                        
#define FLASH_ISCSI_MAX_DDBS		64
#define FLASH_DDB_VALID_COOKIE		0x9034 /* this value indicates this 
						  entry in flash is valid */
#define FLASH_DDB_INVALID_COOKIE	0x0    /* this value is used to set 
						  the entry to invalid    */

/*
 * ****************************************************************************
 * INT_HBA_SYS_INFO
 * ****************************************************************************
 */

typedef struct _INT_HBA_SYS_INFO {
	UINT32	cookie;						/* 4   */
	UINT32	physAddrCount;					/* 4   */
	UINT8	macAddr0[6];					/* 6   */
	UINT8	macAddr1[6];					/* 6   */
	UINT8	macAddr2[6];					/* 6   */
	UINT8	macAddr3[6];					/* 6   */
	UINT8	vendorId[128];					/* 128 */
	UINT8	productId[128];					/* 128 */
	UINT32	serialNumber;					/* 4   */
	UINT32	pciDeviceVendor;				/* 4   */
	UINT32	pciDeviceId;					/* 4   */
	UINT32	pciSubsysVendor;				/* 4   */
	UINT32	pciSubsysId;					/* 4   */
	UINT32	crumbs;						/* 4   */
	UINT32	enterpriseNumber;				/* 4   */
	UINT32	crumbs2;					/* 4   */
} INT_HBA_SYS_INFO, *PINT_HBA_SYS_INFO;				/* 320 */      

/*
 * ****************************************************************************
 * INT_FW_DW_HDR
 * ****************************************************************************
 */

/* File header */
typedef struct _INT_FW_DL_HDR {
	UINT32	Size;		/* download size, excluding DL_HDR & EXT_HDR*/
	UINT32	Checksum;	/* Checksum of download file, excluding DL_HDR 
				   & EXT_HDR */
	UINT32	HdrChecksum;	/* Checksum of header area should be zero */
	UINT32	Flags;		/* See Flags bits defined above */
	UINT32	Cookie;		/* Target specific identifier */
	UINT32	Target;		/* Target specific identifier */
	UINT32	Reserved0;	/* Reserved */
	UINT32	Reserved1;	/* Reserved */
	UINT8	Copyright[64];	/* Copyright */
	UINT8	Version[32];	/* Version String */
} INT_FW_DL_HDR, *PINT_FW_DL_HDR;

/*
 * ****************************************************************************
 * INT_HBA_RESET
 * ****************************************************************************
 */

typedef struct _INT_HBA_RESET {
	UINT32	Reserved[2];					/* 8  */
} INT_HBA_RESET, *PINT_HBA_RESET;				/* 8  */

/*
 * ****************************************************************************
 * INT_COPY_FW_FLASH
 * ****************************************************************************
 */

typedef struct _INT_COPY_FW_FLASH {
	UINT32	Options;					/* 4  */  
} INT_COPY_FW_FLASH, *PINT_COPY_FW_FLASH;			/* 4  */

#define INT_COPY_FLASH_PRIMARY_TO_SECONDARY	0
#define INT_COPY_FLASH_SECONDARY_TO_PRIMARY	1

/*
 * ****************************************************************************
 * INT_LOGOUT_ISCSI
 * ****************************************************************************
 */

/* Logout Options */
#define INT_DEF_RELOGIN_CONNECTION		0x0002
#define INT_DEF_CLOSE_SESSION			0x0001

typedef struct _INT_LOGOUT_ISCSI {
	UINT16    TargetID;					/* 2   */
	UINT16    ConnectionID;					/* 2   */
	UINT16    Options;					/* 2   */
	UINT32    Reserved;					/* 4   */
} INT_LOGOUT_ISCSI, *PINT_LOGOUT_ISCSI;				/* 10  */

/*
 * ****************************************************************************
 * INT_PING
 * ****************************************************************************
 */

typedef struct _INT_PING {
	EXT_ISCSI_IP_ADDR	IPAddr;				/* 20  */
	UINT16			PacketCount;			/* 2   */
	UINT16			Reserved;			/* 2   */
} INT_PING, *PINT_PING;						/* 24  */

#endif /* _QLINIOCT_H_ */
