/******************************************************************************
 *                  QLOGIC LINUX SOFTWARE                                     *
 *                                                                            *
 * QLogic ISP4xxx device driver for Linux 2.4.x                               *
 * Copyright (C) 2002 Qlogic Corporation                                      *
 * (www.qlogic.com)                                                           *
 *                                                                            *
 * This program is free software; you can redistribute it and/or modify it    *
 * under the terms of the GNU General Public License as published by the      *
 * Free Software Foundation; either version 2, or (at your option) any        *
 * later version.                                                             *
 *                                                                            *
 * This program is distributed in the hope that it will be useful, but        *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                 *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 * General Public License for more details.                                   *
 *                                                                            *
 ******************************************************************************
 *             Please see release.txt for revision history.                   *
 *                                                                            *
 ******************************************************************************
 * Function Table of Contents:
 *        qla4000_detect
 *        qla4000_get_token
 *        qla4000_setup
 *        qla4000_alloc_physical_memory
 *        qla4000_free_physical_memory
 *        qla4000_resize_dma_buf
 *        qla4000_alloc_srb_pool
 *        qla4000_free_srb_pool
 *        qla4000_mem_alloc
 *        qla4000_mem_free
 *        qla4000_register_resources
 *        qla4000_select_queue_depth
 *        qla4000_proc_info
 *        qla4000_set_info
 *        qla4000_release
 *        apidev_open
 *        apidev_close
 *        apidev_ioctl
 *        apidev_init
 *        apidev_cleanup
 *        qla4000_info
 *        qla4000_biosparam
 *        qla4000_init_rings
 *        qla4000_initialize_fw_cb
 *        qla4000_get_fw_version
 *        qla4000_validate_mac_address
 *        qla4000_get_firmware_state
 *        qla4000_init_firmware
 *        qla4000_login_device
 *        qla4000_logout_device
 *        qla4000_relogin_device
 *        qla4000_send_inquiry_cmd
 *        qla4000_send_report_luns_cmd
 *        qla4000_get_ddb_entry
 *        qla4000_set_ddb_entry
 *        qla4000_alloc_ddb
 *        qla4000_free_ddb
 *        qla4000_free_ddb_list
 *        qla4000_initialize_devices
 *        qla4000_initialize_adapter
 *        qla4000_get_req_pkt
 *        qla4000_send_marker
 *        qla4000_send_command_to_isp
 *        qla4000_process_pending_srb_q
 *        qla4000_lookup_ddb_by_SCSIID
 *        qla4000_lookup_ddb_by_fw_index
 *        qla4000_queuecommand
 *        qla4000_complete_request
 *        qla4000_status_entry
 *        qla4000_process_response_queue
 *        qla4000_interrupt_service_routine
 *        qla4000_mailbox_command
 *        qla4000_process_ddb_changed
 *        qla4000_process_aen
 *        qla4000_intr_handler
 *        qla4000_watchdog
 *        qla4000_do_dpc
 *        qla4000_panic
 *        qla4000_eh_abort
 *        qla4000_reset_adapter
 *        qla4000_reset_firmware
 *        qla4000_recover_adapter
 *        qla4000_reset_target
 *        qla4000_warm_reset_target
 *        qla4000_cold_reset_target
 *        qla4000_eh_device_reset
 *        qla4000_eh_bus_reset
 *        qla4000_eh_host_reset
 *        qla4000_enable_intrs
 *        qla4000_disable_intrs
 *        qla4000_get_adapter_handle
 *        qla4000_get_hba_count
 *
 *
 *****************************************************************************/

#include <linux/config.h>
#include <linux/module.h>
#include <stdarg.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/segment.h>
#include <asm/byteorder.h>
#include <linux/spinlock.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>
#include <linux/blk.h>
#include <linux/tqueue.h>
#include <linux/wait.h>

#include <linux/init.h>

#include <../drivers/scsi/scsi.h>
#include <../drivers/scsi/sd.h>
#include <../drivers/scsi/hosts.h>

#include <linux/stat.h>
#include <linux/slab.h>  /* in lieu of malloc.h */

#define __KERNEL_SYSCALLS__
#include <linux/unistd.h>
#include <linux/smp_lock.h>
#include <asm/system.h>
#define SHUTDOWN_SIGS	(sigmask(SIGHUP))

void code_start(void)
{
}
#include "qlisioct.h"
#include "qla4x_fw.h"
#include "qla4xxx.h"
#include "ql4print.h"
#include "ql4lists.h"

/****************************************************************************/
/*    			     Function Prototypes			*/
/****************************************************************************/

/*
 *  Linux - iSCSI Driver Interface Function Prototypes.
 */
int         qla4000_detect(Scsi_Host_Template *);
int         qla4000_proc_info( char *, char **, off_t, int, int, int);
int         qla4000_release(struct Scsi_Host *);
const char *qla4000_info(struct Scsi_Host *);
int         qla4000_queuecommand(Scsi_Cmnd *, void (* done)(Scsi_Cmnd *));
int         qla4000_eh_abort(Scsi_Cmnd *);
int         qla4000_eh_device_reset(Scsi_Cmnd *);
int         qla4000_eh_bus_reset(Scsi_Cmnd *);
int         qla4000_eh_host_reset(Scsi_Cmnd *);
int         qla4000_biosparam(Disk *, kdev_t, int[]);
void        qla4000_intr_handler(int, void *, struct pt_regs *);
void        qla4000_select_queue_depth(struct Scsi_Host *, Scsi_Device *);
int         qla4000_set_info(char *, int , struct Scsi_Host *);
void        qla4000_watchdog(unsigned long data);

/*
 * Routines referenced from other modules
 */
extern int  qla4000_ioctl(Scsi_Device *dev, int cmd, void *arg);
extern void qla4000_ioctl_sem_init (scsi_qla_host_t *ha);
extern void qla4000_scsi_pass_done(Scsi_Cmnd *cmd);

/*
 *  QLogic Driver Support Function Prototypes.
 */
STATIC uint8_t  qla4000_initialize_adapter(scsi_qla_host_t *ha, uint8_t ddb_list_todo);
STATIC void     qla4000_mem_free(scsi_qla_host_t *ha);
static inline void qla4000_enable_intrs(scsi_qla_host_t *);
static inline void qla4000_disable_intrs(scsi_qla_host_t *);
STATIC uint8_t  qla4000_interrupt_service_routine(scsi_qla_host_t *ha);
STATIC uint8_t  qla4000_mem_alloc(scsi_qla_host_t *ha);
STATIC uint8_t  qla4000_register_resources(scsi_qla_host_t *);
STATIC int      apidev_cleanup(void);
void qla4000_do_dpc(unsigned long p);
static uint8_t qla4000_complete_request(scsi_qla_host_t *ha, srb_t *srb);

inline lun_entry_t *qla4000_lookup_lun_handle(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, uint16_t lun);
inline ddb_entry_t *qla4000_lookup_ddb_by_SCSIID(scsi_qla_host_t *ha, uint32_t bus, uint32_t target);
inline ddb_entry_t *qla4000_lookup_ddb_by_fw_index(scsi_qla_host_t *ha, uint32_t fw_ddb_index);
uint8_t qla4000_reset_target(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, int flag);
uint8_t qla4000_warm_reset_target(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry);
uint8_t qla4000_cold_reset_target(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry);
void qla4xxx_start_io(scsi_qla_host_t *ha);
uint8_t qla4000_reset_firmware(scsi_qla_host_t *ha);
//STATIC void qla4000_flush_queued_srbs(scsi_qla_host_t *ha, int cmd_status);
STATIC void qla4000_flush_active_srbs(scsi_qla_host_t *ha);
uint8_t qla4000_recover_adapter(scsi_qla_host_t *ha, uint8_t ddb_list_todo);
static void
qla4xxx_extend_timeout(Scsi_Cmnd *cmd, int timeout);

/****************************************************************************/
/*    					Global Data											*/
/****************************************************************************/


/*
 *   ISP Board(s) supported by this driver
 *---------------------------------------------------------------------------*/
#define NUM_OF_ISP_DEVICES 	1
typedef struct
{
	unsigned long   device_id;  /* Device PCI ID   */
	unsigned char   bdName[9];  /* Board ID String */
	unsigned char   numPorts;   /* Number of SCSI ports */
} qla_boards_t;

qla_boards_t   QL4000BoardTbl[NUM_OF_ISP_DEVICES] =
{
/* Board PCI Device ID, Name,  Number of ports */
	{QLA4000_DEVICE_ID, "QLA4000 ", 1}
};


/* 
 * Command line options 
 *---------------------------------------------------------------------------*/
static int ql4xportdownretrycount= 60;
static int ql4xcmdretrycount = 20;
static int ql4xmaxqdepth = 2;
static int ql4xcmdtimeout = 5;

	    #if STRING_COMMAND_LINE_OPTIONS
static unsigned long qla4000_verbose = 1L;
	    #endif

#ifdef MODULE

	    #if STRING_COMMAND_LINE_OPTIONS
static char *options = NULL;
MODULE_PARM(options, "s");  /* insmod qla4000 options=verbose" */
/*
 * Just in case someone uses commas to separate items on the insmod
 * command line, we define a dummy buffer here to avoid having insmod
 * write wild stuff into our code segment
 */
static char dummy_buffer[60] = 
"Please don't add commas in your insmod command!!\n";
	    #endif

MODULE_PARM(ql4xportdownretrycount, "i");
MODULE_PARM_DESC(ql4xportdownretrycount,
		 "Port down retry count");

MODULE_PARM(ql4xcmdretrycount, "i");
MODULE_PARM_DESC(ql4xcmdretrycount,
		 "Command retry count");

MODULE_PARM(ql4xmaxqdepth, "i");
MODULE_PARM_DESC(ql4xmaxqdepth,
		"Maximum queue depth to report for target devices.");

MODULE_PARM(ql4xcmdtimeout, "i");
MODULE_PARM_DESC(ql4xcmdtimeout,
		 "Command timeout");

#endif

#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif

/* 
 * Detected host adapter variables 
 *---------------------------------------------------------------------------*/
int qla4000_hba_count = 0;
static scsi_qla_host_t *qla4000_hostlist = NULL;

/* 
 * Proc info variables
 *---------------------------------------------------------------------------*/
static int qla4000_buffer_size = 0;
static char *qla4000_buffer = NULL;

/* 
 * Declare global semaphore
 *---------------------------------------------------------------------------*/
DECLARE_MUTEX_LOCKED(qla4000_detect_sem);
DECLARE_MUTEX_LOCKED(qla4000_dpc_sem);

/* 
 * String messages for various state values (used for print statements)
 *---------------------------------------------------------------------------*/
const char *host_sts_msg[] ={
	"DID_OK",
	"DID_NO_CONNECT",
	"DID_BUS_BUSY",
	"DID_TIME_OUT",
	"DID_BAD_TARGET",
	"DID_ABORT",
	"DID_PARITY",
	"DID_ERROR",
	"DID_RESET",
	"DID_BAD_INTR",
	NULL};

const char *ddb_state_msg[] ={
	"UNASSIGNED",
	"NO_CONNECTION_ACTIVE",
	"DISCOVERY",
	"NO_SESSION_ACTIVE",
	"SESSION_ACTIVE",
	"LOGGING_OUT",
	"SESSION_FAILED",
	NULL};

const char *srb_state_msg[] = SRB_STATE_TBL();
#if 0
const char *dev_state_msg[] = DEV_STATE_TBL();
const char *lun_state_msg[] = LUN_STATE_TBL();
const char *adapter_state_msg[] = ADAPTER_STATE_TBL();
#endif

/****************************************************************************/
/*  LINUX -  Loadable Module Functions.                                     */
/****************************************************************************/


/**************************************************************************
 * qla4000_detect
 *    This routine will probe for Qlogic 4000 iSCSI host adapters.
 *    It returns the number of host adapters of a particular
 *    type that were found.	 It also initialize all data necessary for
 *    the driver.  It is passed-in the host number, so that it
 *    knows where its first entry is in the scsi_hosts[] array.
 *
 * Input:
 *     template - pointer to SCSI template
 *
 * Returns:
 *  num - number of host adapters found.
 **************************************************************************/
int
qla4000_detect(Scsi_Host_Template *template)
{
	qla_boards_t *bdp;
	int i;
	struct  list_head       *list;
	struct  pci_bus         *b_bus;
	struct  pci_dev         *dev;

	ENTER("qla4000_detect");
	printk(/*KERN_INFO*/ "qla4000: code starts at address = %p\n", code_start);
	QL4PRINT(QLP2|QLP7,printk("qla4000: code starts at address = %p\n", code_start));

	spin_unlock_irq(&io_request_lock);

	#if STRING_COMMAND_LINE_OPTIONS  /* currently not used */

	//#ifdef MODULE
	/*
	* If we are called as a module, the qla4000 pointer may not be null
	* and it would point to our bootup string, just like on the lilo
	* command line.  IF not NULL, then process this config string with
	* qla4000_setup
	*
	* Boot time Options
	* To add options at boot time add a line to your lilo.conf file like:
	* append="qla4000=verbose,max_tags:{{255,255,255,255},{255,255,255,255}}"
	* which will result in the first four devices on the first two
	* controllers being set to a tagged queue depth of 32.
	*/
	if (options)
		qla4000_setup(options, NULL);
	if (dummy_buffer[0] != 'P')
		printk(KERN_WARNING
		       "qla4000: Please read the file /usr/src/linux/drivers/scsi/%s/README.qla4xxx\n"
		       "qla4000: to see the proper way to specify options to the qla4000 module\n"
		       "qla4000: Specifically, don't use any commas when passing arguments to\n"
		       "qla4000: insmod or else it might trash certain memory areas.\n", DRIVER_NAME);
	#endif

	if (!pci_present())
	{
		printk("qla4000_detect: PCI not present\n");
		spin_lock_irq(&io_request_lock);
		return(0);
	}

	bdp = &QL4000BoardTbl[0];
	qla4000_hostlist = NULL;
	template->proc_name = DRIVER_NAME;

	/* Try and find each different type of adapter we support */
	for (i=0;bdp->device_id != 0 && i < NUM_OF_ISP_DEVICES;i++,bdp++)
	{
		struct pci_dev *pdev = NULL;

		QL4PRINT(QLP7, printk("qla4000_detect: searching for adapter %d\n", i));

		while ((pdev = pci_find_subsys(QLA4XXX_VENDOR_ID,
					       bdp->device_id,
					       PCI_ANY_ID,
					       PCI_ANY_ID,
					       pdev))
		      )
		{
			struct Scsi_Host *host;
			scsi_qla_host_t *ha, *cur_ha;

			/* Found an adapter */
			QL4PRINT(QLP7, printk("ql4000: Supported Device Found VID=%X DID=%X SSVID=%X SSDID=%X\n",
					      pdev->vendor, pdev->device,
					      pdev->subsystem_vendor, pdev->subsystem_device));

			if (pci_enable_device(pdev) != PCIBIOS_SUCCESSFUL)
			{
				QL4PRINT(QLP1|QLP7, printk("already found-- skipping\n"));
				continue;
			}

			spin_lock_irq(&io_request_lock);

			host = scsi_register(template, sizeof(scsi_qla_host_t));

			if (host == NULL)
			{
				printk("qla4000: Failed to get memory for host\n");
				spin_unlock_irq(&io_request_lock);
				continue;
			}

			/* Clear our data area */
			ha = (scsi_qla_host_t *) host->hostdata;
			memset(ha, 0, sizeof(scsi_qla_host_t));

			/* Save the information from PCI BIOS.  */
			ha->pdev = pdev;
			ha->pci_bus = pdev->bus->number;
			ha->pci_device_fn = pdev->devfn;
			ha->pci_device_id = bdp->device_id;
			ha->devnum = i;
			ha->ports = bdp->numPorts;
			ha->host = host;
			ha->host_no = host->host_no;
			ha->pci_address = pci_resource_start(pdev, 0);
			ha->instance = qla4000_hba_count;

			/* Save host information */
			host->can_queue   = REQUEST_QUEUE_DEPTH/4;    /*   */
			host->cmd_per_lun = MAX_LINKED_CMDS_PER_LUN;
			host->select_queue_depths = qla4000_select_queue_depth;
			host->base        = ha->pci_address;
			host->irq         = pdev->irq;
			host->max_channel = bdp->numPorts-1;
			host->max_lun     = MAX_LUNS-1;
			host->max_id      = MAX_TARGETS-1;
			host->unique_id   = ha->instance;

			/* Search for pointer to the Intel PCI Brigde on the QLA4000 */
#if 0
			printk("%s class=0x%x,"
			       "vendor_id=0x%x,device_id=0x%x bus_no=%d\n",
			       __func__, pdev->bus->parent->self->class,
			       pdev->bus->parent->self->vendor, 
			       pdev->bus->parent->self->device,
			       pdev->bus->parent->self->bus->number);
#else
#define QLPCI_BRIDGE_CLASS	0x060400
			b_bus=pdev->bus;
			list_for_each(list,&b_bus->devices)
			if ((dev = pci_dev_b(list)) != NULL)
			{
				/*printk("%s pci_dev_class=0x%x pci_device_device_id=0x%x,subordinate=%p,devfn=%d,PCI_SLOT:%02x PCI_FUNC:%x dev=%p self=%p\n",
				       __func__, dev->class, dev->device, 
				       dev->subordinate,dev->devfn,
				       PCI_SLOT(dev->devfn),
				       PCI_FUNC(dev->devfn), 
				       dev, dev->bus->self);*/

				if ((dev->class = QLPCI_BRIDGE_CLASS) && 
				    PCI_SLOT(dev->devfn) == PCI_SLOT(pdev->devfn) && 
				    PCI_FUNC(dev->devfn) != PCI_SLOT(pdev->devfn))
				{
					ha->pci_brgdev=dev;
				}
			}
#endif
			spin_unlock_irq(&io_request_lock);
			/* Allocate memory for dma buffers */
			if (qla4000_mem_alloc(ha) == QLA_ERROR)
			{
				spin_lock_irq(&io_request_lock);
				scsi_unregister(host);
				printk(KERN_INFO "qla4000: Failed to get DMA memory\n");
				spin_unlock_irq(&io_request_lock);
				continue;
			}
			spin_lock_irq(&io_request_lock);

			/* Initialize lists and spinlocks */
			INIT_LIST_HEAD(&ha->ddb_list);
			INIT_LIST_HEAD(&ha->pending_srb_q);
			INIT_LIST_HEAD(&ha->retry_srb_q);
			INIT_LIST_HEAD(&ha->done_srb_q);
			INIT_LIST_HEAD(&ha->suspended_lun_q);
			init_MUTEX(&ha->mbox_sem);
			init_waitqueue_head(&ha->mailbox_wait_queue);
			init_MUTEX(&ha->ioctl_sem);
			init_MUTEX_LOCKED(&ha->ioctl_cmpl_sem);

			spin_lock_init(&ha->hardware_lock);
			spin_lock_init(&ha->aen_q_spinlock);
			spin_lock_init(&ha->adapter_lock);
			spin_lock_init(&ha->list_lock);

			/* Register our resources (i.e. IRQ) with Linux */
			if (qla4000_register_resources(ha) == QLA_ERROR)
			{
				printk(KERN_INFO "qla4000: Failed to register resources\n");
				qla4000_mem_free(ha);
				scsi_unregister(host);
				spin_unlock_irq(&io_request_lock);
				continue;
			}

			spin_unlock_irq(&io_request_lock);

			/* Initialize the Host adapter request/response queues and firmware */
			if (qla4000_initialize_adapter(ha, 1) == QLA_ERROR)
			{
				printk(KERN_INFO "qla4000: Failed to initialize adapter\n");
				qla4000_mem_free(ha);
				spin_lock_irq(&io_request_lock);
				scsi_unregister(host);
				spin_unlock_irq(&io_request_lock);
				continue;
			}

			/* Insert new entry into the list of adapters */
			ha->next = NULL;
			if (qla4000_hostlist == NULL)
			{
				cur_ha = qla4000_hostlist = ha;
			}
			else
			{
				cur_ha = qla4000_hostlist;
				while (cur_ha->next != NULL)
					cur_ha = cur_ha->next;
				cur_ha->next = ha;
			}

			set_bit(AF_INIT_DONE, &ha->flags);

			/* Start watchdog thread */
			QL4PRINT(QLP3|QLP6, printk("qla4000_detect: Starting watchdog timer for adapter %d, interval = %d\n",
						   ha->instance, WDG_TIMER_INTERVAL));
			init_timer(&ha->watchdog_timer);
			ha->watchdog_timer.function = qla4000_watchdog;
			ha->watchdog_timer.data     = (unsigned long) ha;
			ha->watchdog_timer.expires  = jiffies + (WDG_TIMER_INTERVAL * HZ);
			add_timer(&ha->watchdog_timer);

			/* Startup the kernel thread for this host adapter */
			QL4PRINT(QLP3|QLP6, printk("qla4000_detect: Starting kernel thread for qla4000_dpc_%d\n", ha->host_no));
			ha->dpc_notify = &qla4000_detect_sem;
			kernel_thread((int (*)(void *))qla4000_do_dpc, (void *) ha, 0);

			/* Now wait for the kernel dpc thread to initialize
			 * and go to sleep. */
			down(&qla4000_detect_sem);
			ha->dpc_notify = NULL;

			qla4000_hba_count++;
		}  /* end of WHILE */
	} /* end of FOR */

	spin_lock_irq(&io_request_lock);

	QL4PRINT(QLP7, printk("qla4000_detect: Found %d hosts ...\n", qla4000_hba_count));
	LEAVE("qla4000_detect");
	return(qla4000_hba_count);
}

#if STRING_COMMAND_LINE_OPTIONS  /* Currently not used */
STATIC char *
qla4000_get_token(char *cmdline, char *str )
{
	register    char    *cp = cmdline;

	/* skip preceeding spaces */
	while (strchr(cp,' '))
		++cp;

	/* symbol starts here */
	str = cp;

	/* skip char if not a space or : */
	while (*cp && !( strchr(cp,' ') || strchr(cp,':')))
		cp++;

	*cp = '\0';
	return( cp );
}

/**************************************************************************
 *   qla4000_setup
 *
 *   Handle Linux boot parameters. This routine allows for assigning a value
 *   to a parameter with a ':' between the parameter and the value.
 *   ie. qla4000=max_reqs:0x0A,verbose
 **************************************************************************/
STATIC void
qla4000_setup(char *s, int *dummy)
{
	char *end, *str, *cmd;

	QL4PRINT(QLP7, printk(KERN_DEBUG "qla4000: Processing Option str = %s\n", s));
	end = strchr(s, '\0');

	/* locate command */
	str = s;
	for (cmd = s; *cmd && cmd != end; cmd++)
	{
		cmd = qla4000_get_token(cmd, str);
		if (strcmp (str, "verbose") == 0)
		{
			QL4PRINT(QLP7, printk(KERN_DEBUG "qla4000: verbose\n"));
			qla4000_verbose++;
		}
	}
}
#endif

inline uint8_t
qla4000_alloc_physical_memory(struct pci_dev *pdev, dma_buf_t *dma_buf, uint32_t size)
{
	dma_buf->virt_addr = pci_alloc_consistent(pdev, size, &dma_buf->phys_addr);
	if (dma_buf->virt_addr)
	{
		dma_buf->buf_len = size;
		spin_lock_init(&dma_buf->spinlock);
		return(QLA_SUCCESS);
	}
	else
	{
		return(QLA_ERROR);
	}
}

inline void
qla4000_free_physical_memory(struct pci_dev *pdev, dma_buf_t *dma_buf)
{
	pci_free_consistent(pdev, dma_buf->buf_len, dma_buf->virt_addr, dma_buf->phys_addr);
	dma_buf->buf_len = 0;
	dma_buf->phys_addr = 0;
	dma_buf->virt_addr = 0;
	dma_buf->spinlock = SPIN_LOCK_UNLOCKED;
}

/*
 * qla4000_resize_dma_buf
 * 	  Allocate dma memory of the specified size.  Replaces any previously
 * 	  allocated dma memory.	 DMA buffer contains all zeros upn return.
 */
uint8_t
qla4000_resize_dma_buf(scsi_qla_host_t *ha, dma_buf_t *dma_buf, uint32_t size)
{
	uint8_t status = QLA_ERROR;

	ENTER("qla4000_resize_dma_buf");

	if (dma_buf->buf_len)
	{
		QL4PRINT(QLP3, printk("qla4000_resize_dma_buf: deallocate old dma_buf, size=0x%x\n", dma_buf->buf_len));
		qla4000_free_physical_memory(ha->pdev, dma_buf);
	}

	QL4PRINT(QLP3, printk("qla4000_resize_dma_buf: allocate new dma_buf, size=0x%x\n", size));
	status =  qla4000_alloc_physical_memory(ha->pdev, dma_buf, size);

	if (status == QLA_ERROR)
		QL4PRINT(QLP3, printk("qla4000_resize_dma_buf: ERROR allocating new dma_buf, size=0x%x\n", size));

	LEAVE("qla4000_resize_dma_buf");
	return(status);
}

/*
 * qla4000_alloc_srb_pool
 * 	  This routine is called during driver initialization to allocate
 * 	  memory for the local srb pool
 */
uint8_t qla4000_alloc_srb_pool(scsi_qla_host_t *ha)
{
	srb_t *srb;
	int i;
	uint8_t status = QLA_ERROR;
	unsigned long flags;
	ENTER("qla4000_alloc_srb_pool");

	INIT_LIST_HEAD(&ha->free_srb_q);
	spin_lock_init(&ha->list_lock);
	spin_lock_irqsave(&ha->list_lock, flags);
	ha->num_srbs_allocated = 0;
	ha->free_srb_q_count = 0; /* incremented in add_to_free_srb_q routine */

	/*
	 * NOTE: Need to allocate each SRB separately, as Kernel 2.4.4
	 *       seems to have an error when allocating a large amount
	 *       of memory.
	 */
	for (i=0; i < SRB_POOL_SIZE; i++)
	{
		srb = (srb_t *) kmalloc(sizeof(srb_t), GFP_KERNEL);
		if (srb == NULL)
		{
			printk("qla4000_alloc_srb_pool(%d): failed to allocate memory, count = %d\n",
			       ha->host_no, i);
		}
		else
		{
			ha->num_srbs_allocated++;
			__add_to_free_srb_q(ha, srb);
			status = QLA_SUCCESS;
		}
	}

	if (ha->free_srb_q_count)
		status = QLA_SUCCESS;

	spin_unlock_irqrestore(&ha->list_lock, flags);

	QL4PRINT(QLP7, printk("qla4000_alloc_srb_pool(%d): Allocated %d SRB(s)\n",
			      ha->host_no, ha->free_srb_q_count));

	LEAVE("qla4000_alloc_srb_pool");
	return(status);
}

void qla4000_free_srb_pool(scsi_qla_host_t *ha)
{
	struct list_head *ptr, *next;
	srb_t *srb;
	int cnt_free_srbs = 0;
	unsigned long flags;

	ENTER("qla4000_free_srb_pool");
	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_safe(ptr, next, &ha->free_srb_q)
	{
		srb = list_entry(ptr, srb_t, list_entry);
		/* Remove srb from LUN queue. */
		__del_from_free_srb_q(ha, srb);
		kfree(srb);
		cnt_free_srbs++;
	}

	if (cnt_free_srbs != ha->num_srbs_allocated)
	{
		printk(KERN_WARNING "qla4000(%d): Did not free all srbs, "
		       "Free'd srb count = %d, Alloc'd srb count %d\n",
		       ha->host_no, cnt_free_srbs, ha->num_srbs_allocated);
	}

	spin_unlock_irqrestore(&ha->list_lock, flags);
	LEAVE("qla4000_free_srb_pool");
}

/*
 * qla4000_mem_alloc
 *      Allocates adapter memory.
 *
 * Returns:
 *      0  = success.
 *      1  = failure.
 */
STATIC uint8_t
qla4000_mem_alloc(scsi_qla_host_t *ha)
{

	uint8_t   status = QLA_ERROR;
	uint32_t  page_offset, base;
	uint32_t   mmapbase;

	ENTER("qla4000_mem_alloc");

	/*
	 * Allocae consistent memory for request response queue
	 */
	if (qla4000_alloc_physical_memory(ha->pdev, &ha->req_queue, (REQUEST_QUEUE_DEPTH+1)*(sizeof(QUEUE_ENTRY))) == QLA_SUCCESS)
	{
		QL4PRINT(QLP7, printk("qla4000_mem_alloc: req_queue.virt_addr = 0x%p\n", ha->req_queue.virt_addr));
		QL4PRINT(QLP7, printk("qla4000_mem_alloc: req_queue.phys_addr = 0x%X\n", ha->req_queue.phys_addr));
	}

	/*
	 * Allocate consistent memory for response queue
	 */
	if (qla4000_alloc_physical_memory(ha->pdev, &ha->rsp_queue, (RESPONSE_QUEUE_DEPTH+1)*(sizeof(QUEUE_ENTRY))) == QLA_SUCCESS)
	{
		QL4PRINT(QLP7, printk("qla4000_mem_alloc: rsp_queue.virt_addr = 0x%p\n", ha->rsp_queue.virt_addr));
		QL4PRINT(QLP7, printk("qla4000_mem_alloc: rsp_queue.phys_addr = 0x%X\n", ha->rsp_queue.phys_addr));
	}

	/*
	 * Allocate consistent memory for dma buffer
	 */
	if (qla4000_alloc_physical_memory(ha->pdev, &ha->dma_buf, DMA_BUFFER_SIZE) == QLA_SUCCESS)
	{
		QL4PRINT(QLP10, printk("qla4000_mem_alloc: dma buff.virt_addr = 0x%p\n", ha->dma_buf.virt_addr));
		QL4PRINT(QLP10, printk("qla4000_mem_alloc: dma buff.phys_addr = 0x%X\n", ha->dma_buf.phys_addr));
	}

	/*
	 * Allocate memory for srb pool
	 */
	qla4000_alloc_srb_pool(ha);

	/*
	 * Map the Memory I/O register
	 */
	QL4PRINT(QLP7, printk("qla4000_mem_alloc: pci_address = 0x%X\n", ha->pci_address));
	mmapbase = ha->pci_address & PCI_BASE_ADDRESS_MEM_MASK;

	/* 
	 * Find proper memory chunk for memory
	 * map I/O reg. 
	 */
	base = mmapbase & PAGE_MASK;
	page_offset = mmapbase - base;

	/*
	 * Get virtual address for I/O
	 * registers. 
	 */
	ha->memory_mapped_address = (uint32_t )ioremap(base, page_offset + sizeof(*ha->base_address));
	if( ha->memory_mapped_address ) 
	{
		ha->memory_mapped_address += page_offset;
		ha->base_address = (ISP_REG_T *)ha->memory_mapped_address;

            QL4PRINT(QLP7, printk("qla4000_mem_alloc: SUCCESS: memory_mapped_address = 0x%X\n", ha->memory_mapped_address));
		status = QLA_SUCCESS;
	}
	else
	{
		QL4PRINT(QLP2, printk("qla4000_mem_alloc: I/O Remap Failed\n"));
		qla4000_mem_free(ha);
	}

	LEAVE("qla4000_mem_alloc");
	return(status);
}

/*
 * qla4000_free_lun
 *	  This function enables the specified lun in the ddb_entry's lun table.
 */
static inline void
qla4000_free_lun(ddb_entry_t *ddb_entry, uint16_t lun)
{
	lun_entry_t *lun_entry = ddb_entry->lun_table[lun];

	ENTER("qla4000_free_lun");
	if (lun_entry != 0)
	{
		QL4PRINT(QLP3, printk("qla4000_free_lun: target %d, free lun %d\n",
				      ddb_entry->target, lun));

		ddb_entry->lun_table[lun] = NULL;
		ddb_entry->num_valid_luns--;
		kfree(lun_entry);
	}

	LEAVE("qla4000_free_lun");
}

STATIC void
qla4000_free_ddb(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry)
{
	uint16_t lun;

	ENTER("qla4000_free_ddb");

	/* Remove device entry from list */
	list_del_init(&ddb_entry->list_entry);

	/* Remove device pointer from index mapping arrays */
	ha->fw_ddb_index_map[ddb_entry->fw_ddb_index] = NULL;
	ha->target_map[ddb_entry->target] = NULL;

	/* Free memory allocated for all luns */
	for (lun = 0; lun < MAX_LUNS; lun++)
		if (ddb_entry->lun_table[lun])
			qla4000_free_lun(ddb_entry, lun);

		/* Free memory for device entry */
	kfree(ddb_entry);
	LEAVE("qla4000_free_ddb");
}

STATIC void
qla4000_free_ddb_list(scsi_qla_host_t *ha)
{
	struct list_head *ptr;
	ddb_entry_t *ddb_entry;

	ENTER("qla4000_free_ddb_list");

	while (!list_empty(&ha->ddb_list))
	{
		/* Remove device entry from head of list */
		ptr = ha->ddb_list.next;
		list_del_init(ptr);

		/* Free memory for device entry */
		ddb_entry = list_entry(ptr, ddb_entry_t, list_entry);
		kfree(ddb_entry);
	}

	LEAVE("qla4000_free_ddb_list");
}

/*
 * qla4000_mem_free
 *      Frees adapter allocated memory.
 *
 * Input:
 *      ha = adapter block pointer.
 */
STATIC void
qla4000_mem_free(scsi_qla_host_t *ha)
{
	ENTER("qla4000_mem_free");

	/* Free memory potentially allocated by qla4000_proc_info() */
	if (qla4000_buffer != NULL)
	{
		QL4PRINT(QLP3|QLP7, printk("qla4000_mem_free: free proc_info memory\n"));
		kfree(qla4000_buffer);
		qla4000_buffer = NULL;
		qla4000_buffer_size = 0;
	}

	/* Free consistent memory allocated for request queue */
	QL4PRINT(QLP3|QLP7, printk("qla4000_mem_free: free request queue memory\n"));
	qla4000_free_physical_memory(ha->pdev, &ha->req_queue);

	/* Free consistent memory allocated for response queue */
	QL4PRINT(QLP3|QLP7, printk("qla4000_mem_free: free response queue memory\n"));
	qla4000_free_physical_memory(ha->pdev, &ha->rsp_queue);

	/* Free consistent memory allocated for dma buffer */
	QL4PRINT(QLP3|QLP7, printk("qla4000_mem_free: free dma buffer memory\n"));
	qla4000_free_physical_memory(ha->pdev, &ha->dma_buf);

	/* Free srb pool */
	QL4PRINT(QLP3|QLP7, printk("qla4000_mem_free: free srb pool\n"));
	qla4000_free_srb_pool(ha);

	/* Free ddb list */
	QL4PRINT(QLP3|QLP7, printk("qla4000_mem_free: free ddb list\n"));
	qla4000_free_ddb_list(ha);

	/* Upmap memory to access PCI registers */
	if (ha->memory_mapped_address)
	{
		QL4PRINT(QLP3|QLP7, printk("qla4000_mem_free: unmap mem io region\n"));
		iounmap((void *) ha->memory_mapped_address);
	}

	LEAVE("qla4000_mem_free");
}

/**************************************************************************
*   qla4000_register_resources
*
* Description:
*   Registers the irq with Linux.
*
* Input:
*     ha - pointer to host adapter structure
*     maxchannels - MAX number of channels.
*
* Returns:
*  0 - Sucessfully reserved resources.
*  1 - Failed to reserved a resource.
**************************************************************************/
STATIC uint8_t
qla4000_register_resources(scsi_qla_host_t *ha)
{
	uint8_t status = QLA_SUCCESS;
	struct Scsi_Host *host = ha->host;

	ENTER("qla4000_register_resources");

	/* Register the IRQ with Linux (sharable) */
	if (request_irq(host->irq,
			qla4000_intr_handler,
			SA_INTERRUPT|SA_SHIRQ,
			QL4000BoardTbl[ha->devnum].bdName/*"qla4000"*/,
			ha) != QLA_SUCCESS)
	{
		printk(KERN_WARNING "qla4000 : Failed to reserved interrupt %d.  Already in use\n", host->irq);
		status = QLA_ERROR;
	}
	else
	{
		QL4PRINT(QLP7|QLP10, printk("qla4000_register_resources: IRQ %d\n", host->irq));
	}

	LEAVE("qla4000_register_resources");
	return(status);
}

/**************************************************************************
 *   qla4000_select_queue_depth
 *
 *   Sets the queue depth for each SCSI device hanging off the input
 *   host adapter.  We use a queue depth of 2 for devices that do not
 *   support tagged queueing.
 **************************************************************************/
void
qla4000_select_queue_depth(struct Scsi_Host *host, Scsi_Device *scsi_devs)
{
	Scsi_Device *device;
	scsi_qla_host_t  *ha = (scsi_qla_host_t *) host->hostdata;

	ENTER("qla4000_select_queue_depth");
	for (device = scsi_devs; device != NULL; device = device->next)
	{
		int target = device->id;
		ddb_entry_t *ddb_entry = qla4000_lookup_ddb_by_SCSIID(ha, 0, target);

		if (device->host != host)
			continue;

		if (device->tagged_supported && (ddb_entry != NULL))
		{
			device->tagged_queue = 1;
			device->current_tag = 0;
			device->queue_depth = ddb_entry->exe_throttle;
#if defined(MODULE)
			if (!(ql4xmaxqdepth == 0 || ql4xmaxqdepth > 256))
				device->queue_depth = ql4xmaxqdepth;
#endif

			printk(KERN_INFO "scsi(%d:%d:%d:%d): Enabled tagged queuing, queue depth %d.\n",
				       (int)ha->host_no, device->channel, target,
				       device->lun, device->queue_depth);
		}
		else
		{
			device->queue_depth = 1;
		}
	}
	LEAVE("qla4000_select_queue_depth");
}

/*
 * Set parameters for the driver from the /proc filesystem.
 */
int
qla4000_set_info(char *buffer, int length, struct Scsi_Host *HBAptr)
{
	return(-ENOSYS);  /* Currently this is a no-op */
}

#define	PROC_BUF	(&qla4000_buffer[len])

inline int qla4000_proc_dump_srb_info(int len, srb_t *srb)
{
	(len) += sprintf(PROC_BUF, "srb %p", srb);

	if (srb->cmd)
	{
		Scsi_Cmnd *cmd = srb->cmd;
		//int i;

		len += sprintf(PROC_BUF, ", b%d,t%d,l%d, SS=%d, DS=%d, LS=%d",
			       cmd->channel, cmd->target, cmd->lun,
			       srb->state,
			       atomic_read(&srb->ddb_entry->state),
			       srb->lun_entry->lun_state);

		//len += sprintf(PROC_BUF, ", cdb=",
		//for (i=0; i<cmd->cmd_len; i++)
		//        len += sprintf(PROC_BUF, "%02X ", cmd->cmnd[i]);

	}

	len += sprintf(PROC_BUF, "\n");
	return(len);
}

/*************************************************************************
 * qla4000_proc_info
 *
 * Description:
 *   Return information to handle /proc support for the driver.
 *
 * inout : decides on the direction of the dataflow and the meaning of the
 *         variables
 * buffer: If inout==FALSE data is being written to it else read from it
 *         (ptrs to a page buffer)
 * *start: If inout==FALSE start of the valid data in the buffer
 * offset: If inout==FALSE offset from the beginning of the imaginary file
 *         from which we start writing into the buffer
 * length: If inout==FALSE max number of bytes to be written into the buffer
 *         else number of bytes in the buffer
 *
 * Returns:
 *************************************************************************/
int
qla4000_proc_info (char *buffer, char **start, off_t offset, int length,
		   int hostno, int inout)
{
	scsi_qla_host_t *ha;
	ddb_entry_t     *ddb_entry;
	struct list_head *ptr, *next; /* used for traversing lists */
	qla_boards_t    *bdp;
	int             size = 0;
	int             len = 0;
	#if DISPLAY_SRBS_IN_PROC
	unsigned long   flags;
	#endif

	QL4PRINT(QLP3, printk("Entering qla4000_proc_info: qlabuff=%p, offset=0x%lx, "
			      "length=0x%x\n", qla4000_buffer, offset, length));

	/* Find the host that was specified */
	for (ha=qla4000_hostlist; (ha != NULL) && ha->host->host_no != hostno; ha=ha->next)
		continue;

	/* if host wasn't found then exit */
	if (!ha)
	{
		size = sprintf(buffer,
			       "Can't find adapter for host number %d\n",
			       hostno);

		if (size > length)
			return(size);
		else
			return(0);
	}


	if (inout == TRUE)   /* Has data been written to the file? */
	{
		QL4PRINT(QLP3, printk("qla4000_proc_info: has data been written to the file. \n"));
		return(qla4000_set_info(buffer, length, ha->host));
	}

	/*
	 * if our old buffer is the right size use it otherwise
	 * allocate a new one.
	 */
	size = 4096 * 5;  /* get page(s) */
	if (qla4000_buffer_size != size)
	{
		/* deallocate this buffer and get a new one */
		if (qla4000_buffer != NULL)
		{
			kfree(qla4000_buffer);
			qla4000_buffer_size = 0;
		}
		qla4000_buffer = kmalloc(size, GFP_KERNEL);
	}

	if (qla4000_buffer == NULL)
	{
		size = sprintf(buffer, "qla4000_proc_info - kmalloc error at line %d\n",
			       __LINE__);
		return(size);
	}
	/* save the size of our buffer */
	qla4000_buffer_size = size;

	/* start building the print buffer */
	bdp = &QL4000BoardTbl[ha->devnum];
	len += sprintf(PROC_BUF, "QLogic PCI to iSCSI Adapter for ISP 4000:\n");
	len += sprintf(PROC_BUF, "Driver version %s\n", QLA4XXX_DRIVER_VERSION);
	len += sprintf(PROC_BUF, "Code starts at address = %p\n", code_start);

	len += sprintf(PROC_BUF, "Firmware version %2d.%02d Patch %02d Build %02d\n",
		       ha->firmware_version[0], ha->firmware_version[1],
		       ha->patch_number, ha->build_number);
	len += sprintf(PROC_BUF, "SCSI Host Adapter Information: %s\n", bdp->bdName);
	len += sprintf(PROC_BUF, "Request Queue = 0x%x, Response Queue = 0x%x\n",
		       ha->req_queue.phys_addr, ha->rsp_queue.phys_addr);
	len += sprintf(PROC_BUF, "Request Queue count= 0x%x, Response Queue count= 0x%x\n",
		       REQUEST_QUEUE_DEPTH, RESPONSE_QUEUE_DEPTH);
	len += sprintf(PROC_BUF, "Number of free request entries  = %d of %d\n",
		       ha->req_q_count, REQUEST_QUEUE_DEPTH);
	len += sprintf(PROC_BUF, "Number of free aen entries    = %d of %d\n",
		       ha->aen_q_count, MAX_AEN_ENTRIES);
#if 0
	len += sprintf(PROC_BUF, "Number of adapter errors %d\n", 
		       ha->adapter_error_count);
#endif
	#if TRACK_SPURIOUS_INTRS
	len += sprintf(PROC_BUF, "Number of Spurious Interrupts = %d\n",
		       ha->spurious_int_count);
	#endif
	len += sprintf(PROC_BUF, "Number of Mailbox Timeouts = %d\n",
		       ha->mailbox_timeout_count);
	len += sprintf(PROC_BUF, "Total H/W int = 0x%lx\n", 
			(unsigned long int)ha->isr_count);
	len += sprintf(PROC_BUF, "Total RISC int = 0x%lx\n", 
			(unsigned long int)ha->total_io_count);
#if 0
	len += sprintf(PROC_BUF, "Number of Aborted I/O's = %d\n",
		       ha->aborted_io_count); 
	len += sprintf(PROC_BUF, "Number of Active Command Timeouts = %d\n",
		       ha->io_timeout_count); 
	len += sprintf(PROC_BUF, "Seconds since last Interrupt = %d\n",
		       ha->seconds_since_last_intr);
#endif
	len += sprintf(PROC_BUF,
			"    Device queue depth = 0x%x\n",
		       (ql4xmaxqdepth == 0) ? 16 : ql4xmaxqdepth);
	len += sprintf(PROC_BUF, "Number of commands in pending_srb_q = %d\n",
		       ha->pending_srb_q_count);
	len += sprintf(PROC_BUF, "Number of commands in done_srb_q = %d\n",
		       ha->done_srb_count);

	#if DISPLAY_SRBS_IN_PROC
	if (ha->pending_srb_q_count)
	{
		len += sprintf(PROC_BUF, "\nDump pending_srb_q:\n");
		spin_lock_irqsave(&ha->list_lock, flags);
		list_for_each_safe(ptr, next, &ha->pending_srb_q)
		{
			srb_t *srb = list_entry(ptr, srb_t , list_entry);
			len = qla4000_proc_dump_srb_info(len, srb);
		}
		spin_unlock_irqrestore(&ha->list_lock, flags);
		len += sprintf(PROC_BUF, "\n");
	}
	#endif

	len += sprintf(PROC_BUF, "Number of commands in retry_srb_q = %d\n",
		       ha->retry_srb_q_count);

	#if DISPLAY_SRBS_IN_PROC
	if (ha->retry_srb_q_count)
	{
		len += sprintf(PROC_BUF, "\nDump retry_srb_q:\n");
		spin_lock_irqsave(&ha->list_lock, flags);
		list_for_each_safe(ptr, next, &ha->retry_srb_q)
		{
			srb_t *srb = list_entry(ptr, srb_t , list_entry);
			len = qla4000_proc_dump_srb_info(len, srb);
		}
		spin_unlock_irqrestore(&ha->list_lock, flags);
		len += sprintf(PROC_BUF, "\n");
	}
	#endif

	len += sprintf(PROC_BUF, "Number of active commands = %d\n",
		       ha->active_srb_count);

	#if DISPLAY_SRBS_IN_PROC
	if (ha->active_srb_count)
	{
		int i;

		spin_lock_irqsave(&ha->hardware_lock, flags);
		len += sprintf(PROC_BUF, "\nDump active commands:\n");
		for (i=1; i<QLA4000_MAX_CMDS_TO_RISC; i++)
		{
			srb_t *srb = ha->active_srb_array[i];
			if (srb)
				len = qla4000_proc_dump_srb_info(len, srb);
		}
		len += sprintf(PROC_BUF, "\n");
		spin_unlock_irqrestore(&ha->hardware_lock, flags);
	}
	#endif

	len += sprintf(PROC_BUF, "Total number of IOCBs (used/max) "
			"= (%d/%d)\n", (int)ha->iocb_cnt, (int)ha->iocb_hiwat);
	len += sprintf(PROC_BUF, "Number of free srbs    = %d of %d\n",
		       ha->free_srb_q_count, ha->num_srbs_allocated);
	len += sprintf(PROC_BUF, "\n");
	len += sprintf(PROC_BUF, "SCSI device Information:\n");
	len += sprintf(PROC_BUF, "  H: B: T: L:\n");

	/* scan for all equipment stats */
	list_for_each_safe(ptr, next, &ha->ddb_list)
	{
		int lun;

		/* Get ddb and make sure it's valid */
		ddb_entry = list_entry(ptr, ddb_entry_t, list_entry);

		for (lun = 0; lun < MAX_LUNS; lun++)
		{
			lun_entry_t *lun_entry = qla4000_lookup_lun_handle(ha, ddb_entry, lun);
			if (lun_entry)
			{
				len += sprintf(PROC_BUF, "(%2d:%2d:%2d:%2d): fw_index 0x%x, ",
					       ha->host_no, ddb_entry->bus, ddb_entry->target, lun, ddb_entry->fw_ddb_index);

				/* current number of outstanding requests */
				len += sprintf(PROC_BUF, "Active reqs %2d, ", lun_entry->out_count);
				len += sprintf(PROC_BUF, "DS=%d, ", atomic_read(&ddb_entry->state));
				len += sprintf(PROC_BUF, "LS=%d", lun_entry->lun_state);

				if (atomic_read(&ddb_entry->state) == DEV_STATE_MISSING)
				{
					len += sprintf(PROC_BUF, ", PDTO= %d of %d",
						       atomic_read(&ddb_entry->port_down_timer),
						       ha->port_down_retry_count);
				}

				len += sprintf(PROC_BUF, "\n");

				if (len >= qla4000_buffer_size - 256)
					break;
			}
		}
	}

	if (len >= qla4000_buffer_size)
	{
		printk(KERN_WARNING "qla4000: Overflow buffer at line %d\n", __LINE__);
	}

	if (offset > len - 1)
	{
		kfree(qla4000_buffer);
		qla4000_buffer = NULL;
		qla4000_buffer_size = length = 0;
		*start = NULL;
	}
	else
	{
		*start = buffer;   /* Start of wanted data */
		if (len - offset < length)
			length = len - offset;
		memcpy(buffer, &qla4000_buffer[offset], length);
	}

	QL4PRINT(QLP3, printk("Exiting qla4000_proc_info: qlabuff=%p, offset=0x%lx, "
			      "length=0x%x\n", qla4000_buffer, offset, length));

	return(length);
}

/**************************************************************************
 *   qla4000_release
 *   Free the passed in Scsi_Host memory structures prior to unloading the
 *   module.
 **************************************************************************/
int
qla4000_release(struct Scsi_Host *host)
{
	scsi_qla_host_t *ha = (scsi_qla_host_t *) host->hostdata;

	ENTER("qla4000_release");

	if (ha)
	{
		/* There shouldn't be outstanding commands at this point,
		 * so no need to check here.
		 */

		/* turn-off interrupts on the card */
		qla4000_disable_intrs(ha);

		/* Detach interrupt */
		if (host->irq)
		{
			QL4PRINT(QLP3|QLP7, printk("qla4000_release: IRQ %d\n", host->irq));
			free_irq(host->irq, ha);
		}

		/* Remove watchdog timer, if present */
		if (timer_pending(&ha->watchdog_timer))
		{
			QL4PRINT(QLP3|QLP6|QLP7, printk("qla4000_release: Removing watchdog timer for adapter %d\n", ha->instance));
			del_timer_sync(&ha->watchdog_timer);
			ha->watchdog_timer.function = NULL;
		}

		/* Kill the kernel thread for this host */
		if (ha->dpc_handler != NULL)
		{
			QL4PRINT(QLP7, printk("qla4000_release: kernel thread qla4000_dpc_%d\n", ha->host_no));
			ha->dpc_notify = &qla4000_detect_sem;

			send_sig(SIGHUP, ha->dpc_handler, 1);
			down(&qla4000_detect_sem);
			ha->dpc_notify = NULL;
		}

		apidev_cleanup();

		/* Free proc buffer */
		if (qla4000_buffer != NULL)
		{
			QL4PRINT(QLP7, printk("qla4000_release: free proc buffer\n"));
			kfree(qla4000_buffer);
			qla4000_buffer_size = 0;
		}

		/* Free	memory back to the OS and upmap I/O region */
		qla4000_mem_free(ha);
	}
	else
	{
		QL4PRINT(QLP1|QLP10, printk("qla4000_release: NULL host adapter\n"));
	}

	LEAVE("qla4000_release");
	return(QLA_SUCCESS);
}

/****************************************************************************/
/* Create character driver "HbaApiDev" w dynamically allocated major number */
/* and create "/proc/scsi/<DRIVER_NAME>/HbaApiNode" as the device node associated */
/* with the major number.                                                   */
/****************************************************************************/
#define APIDEV_NODE  "HbaApiNode"
#define APIDEV_NAME  "HbaApiDev"

STATIC int apidev_major = 0;
STATIC struct Scsi_Host *apidev_host = 0;

STATIC int apidev_open(struct inode *inode, struct file *file)
{
	QL4PRINT(QLP3|QLP4, printk("qla4000_apidev: open MAJOR number = %d, MINOR number = %d\n", MAJOR (inode->i_rdev), MINOR (inode->i_rdev)));
	return(0);
}

STATIC int apidev_close(struct inode *inode, struct file *file)
{
	QL4PRINT(QLP3|QLP4, printk("qla4000_apidev: closed\n"));
	return(0);
}

STATIC int apidev_ioctl(struct inode *inode, struct file *fp, unsigned int cmd, unsigned long arg)
{
	static Scsi_Device fake_scsi_device;
	fake_scsi_device.host = apidev_host;
	QL4PRINT(QLP3|QLP4, printk("qla4000_apidev: ioctl (%X)\n", cmd));
	return(qla4000_ioctl(&fake_scsi_device, (int)cmd, (void*)arg));
}

static struct file_operations apidev_fops = {
	ioctl:    apidev_ioctl,
	open:     apidev_open,
	release:  apidev_close
};

STATIC int apidev_init(struct Scsi_Host *host)
{
	if (apidev_host) return(0);

	if (0 > (apidev_major = register_chrdev(0, APIDEV_NAME, &apidev_fops)))
	{
		QL4PRINT(QLP4|QLP7, printk("qla4000_apidev: register_chrdev rc=%d\n",apidev_major));
		return(apidev_major);
	}

	apidev_host = host;
	QL4PRINT(QLP4|QLP7, printk("qla4000: Created /proc/scsi/%s/%s major=%d\n", DRIVER_NAME, APIDEV_NODE,apidev_major));
	proc_mknod(APIDEV_NODE, 0777+S_IFCHR,host->hostt->proc_dir,(kdev_t)MKDEV(apidev_major,0));
	return(0);
}

STATIC int apidev_cleanup(void)
{
	if (!apidev_host) return(0);
	unregister_chrdev(apidev_major,APIDEV_NAME);

	QL4PRINT(QLP4|QLP7, printk("qla4000_apidev: cleanup\n"));
	remove_proc_entry(APIDEV_NODE,apidev_host->hostt->proc_dir);
	apidev_host = 0;
	return(0);
}

/**************************************************************************
 *   qla4000_info
 *     Return a string describing the driver.
 **************************************************************************/
const char *
qla4000_info(struct Scsi_Host *host)
{
	static char qla4000_info_buffer[125];
	char *bp;
	scsi_qla_host_t *ha;
	qla_boards_t   *bdp;

	apidev_init(host);

	bp = &qla4000_info_buffer[0];
	ha = (scsi_qla_host_t *)host->hostdata;
	bdp = &QL4000BoardTbl[ha->devnum];
	memset(bp, 0, sizeof(qla4000_info_buffer));
	sprintf(bp,
		"QLogic %s PCI to iSCSI Host Adapter: bus %d device %d irq %d\n"
		"        Firmware v%2d.%02d.%02d.%02d, Driver v%s",
		(char *)&bdp->bdName[0], ha->pci_bus, 
		(ha->pci_device_fn & 0xf8) >> 3,
		host->irq, ha->firmware_version[0], ha->firmware_version[1],
		ha->patch_number, ha->build_number, QLA4XXX_DRIVER_VERSION);
	return(bp);
}

/**************************************************************************
 * qla4000_biosparam
 *   Return the disk geometry for the given SCSI device.
 **************************************************************************/
int
qla4000_biosparam(Disk *disk, kdev_t dev, int geom[])
{
	int heads, sectors, cylinders;

	heads = 64;
	sectors = 32;
	cylinders = disk->capacity / (heads * sectors);
	if (cylinders > 1024)
	{
		heads = 255;
		sectors = 63;
		cylinders = disk->capacity / (heads * sectors);
		/* if (cylinders > 1023)
		cylinders = 1023; */
	}

	geom[0] = heads;
	geom[1] = sectors;
	geom[2] = cylinders;

		printk(KERN_DEBUG "qla4000: biosparam  heads %d, sectors %d, cylinders %d\n",
		       heads, sectors, cylinders);
	return(0);
}

/*
 * Initialize rings
 *
 * Input:
 *      ha                = adapter block pointer.
 *
 * Returns:
 *      0 = success.
 */
STATIC uint8_t
qla4000_init_rings(scsi_qla_host_t *ha)
{
	uint16_t    i;
	ISP_REG_T   *reg = ha->base_address;
	QUEUE_ENTRY *response_queue;
	unsigned long flags = 0;

	ENTER("qla4000_init_rings");

	/* Initialize request queue. */
	spin_lock_irqsave(&ha->hardware_lock, flags);
	ha->request_out  = (uint16_t) RD_REG_DWORD(&reg->requestQueueOutPointer);
	ha->request_in   = ha->request_out;
	ha->request_ptr  = &((QUEUE_ENTRY *) ha->req_queue.virt_addr)[ha->request_in];
	ha->req_q_count    = REQUEST_QUEUE_DEPTH;

	/* Initialize response queue. */
	ha->response_in  = (uint16_t) RD_REG_DWORD(&reg->responseQueueInPointer);
	ha->response_out = ha->response_in;
	ha->response_ptr = &((QUEUE_ENTRY *) ha->rsp_queue.virt_addr)[ha->response_out];

	/* Initialize response queue entries */
	response_queue = (QUEUE_ENTRY *) ha->rsp_queue.virt_addr;
	for (i=0; i < RESPONSE_QUEUE_DEPTH; i++, response_queue++)
		response_queue->signature = RESPONSE_PROCESSED;

	/* Initialize active array */
	for (i=0; i<QLA4000_MAX_CMDS_TO_RISC; i++)
		ha->active_srb_array[i] = 0;
	ha->active_srb_count = 0;

	/* Initialize aen queue */
	ha->aen_q_count = MAX_AEN_ENTRIES;

	spin_unlock_irqrestore(&ha->hardware_lock, flags);
	LEAVE("qla4000_init_rings");
	return(QLA_SUCCESS);
}

STATIC uint8_t
qla4000_initialize_fw_cb(scsi_qla_host_t *ha)
{
	INIT_FW_CTRL_BLK  *init_fw_cb = (INIT_FW_CTRL_BLK *) ha->dma_buf.virt_addr;
	static uint32_t          mbox_cmd[MBOX_REG_COUNT];
	static uint32_t          mbox_sts[MBOX_REG_COUNT];
	uint8_t           status = QLA_ERROR;

	ENTER("qla4000_initialize_fw_cb");

	memset(init_fw_cb, 0, sizeof(INIT_FW_CTRL_BLK));
	spin_lock(&ha->dma_buf.spinlock);

	/*
	 * Get Initialize Firmware Control Block
	 */
	memset(&mbox_cmd, 0, sizeof(mbox_cmd));
	mbox_cmd[0] = MBOX_CMD_GET_INIT_FW_CTRL_BLOCK;
	mbox_cmd[2] = LSL(ha->dma_buf.phys_addr);
	mbox_cmd[3] = MSL(ha->dma_buf.phys_addr);

	if (qla4000_mailbox_command(ha, 4, 1, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS)
	{
		goto exit_init_fw_cb;
	}

	QL4PRINT(QLP10, printk("Init Fw Ctrl Blk: \n"));
	qla4000_dump_bytes(QLP10, ha->dma_buf.virt_addr, sizeof(INIT_FW_CTRL_BLK));

	/*
	 * Initialize request and response queues
	 */
	qla4000_init_rings(ha);

	/*
	 * Fill in the request and response queue information
	 */
	init_fw_cb->ReqQOutPtr  = ha->request_out;
	init_fw_cb->RspQInPtr   = ha->response_in;
	init_fw_cb->ReqQLen     = REQUEST_QUEUE_DEPTH;
	init_fw_cb->RspQLen     = RESPONSE_QUEUE_DEPTH;
	init_fw_cb->ReqQAddrLo  = LSL(ha->req_queue.phys_addr);
	init_fw_cb->ReqQAddrHi  = MSL(ha->req_queue.phys_addr);
	init_fw_cb->RspQAddrLo  = LSL(ha->rsp_queue.phys_addr);
	init_fw_cb->RspQAddrHi  = MSL(ha->rsp_queue.phys_addr);

	/*
	 * Set up required options
	 */
	init_fw_cb->FwOptions |= (FWOPT_SESSION_MODE | FWOPT_INITIATOR_MODE);
	init_fw_cb->FwOptions &= ~FWOPT_TARGET_MODE;

	/*
	 * Save IPAddress and FirmwareOptions
	 */
	memcpy(ha->ip_address, init_fw_cb->IPAddr, sizeof(ha->ip_address));
	ha->firmware_options = init_fw_cb->FwOptions;

	/* Save Port Down Retry Count */
	ha->port_down_retry_count    = ql4xportdownretrycount;

	/* High-water mark of IOCBs */
	ha->iocb_hiwat = 254;

	/*
	 * Send Initialize Firmware Control Block
	 */
	QL4PRINT(QLP7, printk("qla4000_initialize_fw_cb: init_fw cmd sent\n"));
	mbox_cmd[0] = MBOX_CMD_INITIALIZE_FIRMWARE;
	mbox_cmd[1] = 0;
	mbox_cmd[2] = LSL(ha->dma_buf.phys_addr);
	mbox_cmd[3] = MSL(ha->dma_buf.phys_addr);

	if (qla4000_mailbox_command(ha, 4, 1, &mbox_cmd[0], &mbox_sts[0]) == QLA_SUCCESS)
	{
		QL4PRINT(QLP10, printk("Init Fw Ctrl Blk: \n"));
		qla4000_dump_bytes(QLP10, ha->dma_buf.virt_addr, sizeof(INIT_FW_CTRL_BLK));
		QL4PRINT(QLP7, printk("IP Addr ha%d: %d.%d.%d.%d\n", ha->host_no,
				      ha->ip_address[0], ha->ip_address[1], ha->ip_address[2], ha->ip_address[3]));

		status = QLA_SUCCESS;
	}
	else
	{
		QL4PRINT(QLP1|QLP7, printk("qla4000_initialize_fw_cb: MBOX_CMD_INITIALIZE_FIRMWARE failed w/ status %04X\n", mbox_sts[0]));
	}

	exit_init_fw_cb:
	spin_unlock(&ha->dma_buf.spinlock);
	LEAVE("qla4000_initialize_fw_cb");
	return(status);
}

STATIC uint8_t
qla4000_get_fw_version(scsi_qla_host_t *ha)
{
	static uint32_t mbox_cmd[MBOX_REG_COUNT];
	static uint32_t mbox_sts[MBOX_REG_COUNT];

	/*
	 * Get firmware version
	 */
	mbox_cmd[0] = MBOX_CMD_ABOUT_FW;
	if (qla4000_mailbox_command(ha, 1, 5, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS)
	{
		QL4PRINT(QLP2, printk("qla4000_initialize_fw_cb: MBOX_CMD_ABOUT_FW failed w/ status %04X\n", mbox_sts[0]));
		return(QLA_ERROR);
	}

	/*
	 * Save firmware version information
	 */
	ha->firmware_version[0] = mbox_sts[1];
	ha->firmware_version[1] = mbox_sts[2];
	ha->patch_number        = mbox_sts[3];
	ha->build_number        = mbox_sts[4];

	QL4PRINT(QLP7, printk("FW Version %02d.%02d Patch %02d Build %02d\n",
			      ha->firmware_version[0], ha->firmware_version[1],
			      ha->patch_number, ha->build_number));

	return(QLA_SUCCESS);
}

STATIC uint8_t
qla4000_validate_mac_address(scsi_qla_host_t *ha)
{
	static uint32_t mbox_cmd[MBOX_REG_COUNT];
	static uint32_t mbox_sts[MBOX_REG_COUNT];
	FLASH_SYS_INFO *sys_info;

	ENTER("qla4000_validate_mac_address");
	spin_lock(&ha->dma_buf.spinlock);
	sys_info = (FLASH_SYS_INFO *) ha->dma_buf.virt_addr;

	/* Get flash sys info */
	mbox_cmd[0] = MBOX_CMD_READ_FLASH;
	mbox_cmd[1] = LSL(ha->dma_buf.phys_addr);
	mbox_cmd[2] = MSL(ha->dma_buf.phys_addr);
	mbox_cmd[3] = FLASH_OFFSET_SYS_INFO;
	mbox_cmd[4] = sizeof(*sys_info);

	if (qla4000_mailbox_command(ha, 5, 1, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS)
	{
		QL4PRINT(QLP2, printk("qla4000_validate_mac_address: MBOX_CMD_READ_FLASH failed w/ status %04X\n", mbox_sts[0]));
		spin_unlock(&ha->dma_buf.spinlock);
		return(QLA_ERROR);
	}

	//FIXME: Put M.A.C. validation code here ...

	#ifdef DEBUG
	{
		int i;

		/* Display Debug Print Info */
		QL4PRINT(QLP10, printk("Flash Sys Info: \n"));
		qla4000_dump_bytes(QLP10, sys_info, sizeof(*sys_info));

		for (i=0; i < sys_info->physAddrCount; i++)
		{
			QL4PRINT(QLP7, printk("MAC Addr%d %02x:%02x:%02x:%02x:%02x:%02x\n", i,
					      sys_info->physAddr[i].address[0],
					      sys_info->physAddr[i].address[1],
					      sys_info->physAddr[i].address[2],
					      sys_info->physAddr[i].address[3],
					      sys_info->physAddr[i].address[4],
					      sys_info->physAddr[i].address[5]));
		}
	}
	#endif

	spin_unlock(&ha->dma_buf.spinlock);
	LEAVE("qla4000_validate_mac_address");
	return(QLA_SUCCESS);
}

uint8_t
qla4000_get_firmware_state(scsi_qla_host_t *ha)
{
	static uint32_t mbox_cmd[MBOX_REG_COUNT];
	static uint32_t mbox_sts[MBOX_REG_COUNT];

	ENTER("qla4000_get_firmware_state");

	/* Get firmware version */
	mbox_cmd[0] = MBOX_CMD_GET_FW_STATE;
	if (qla4000_mailbox_command(ha, 1, 4, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS)
	{
		QL4PRINT(QLP1, printk("qla4000_get_firmware_state: MBOX_CMD_GET_FW_STATE failed w/ status %04X\n", mbox_sts[0]));
		return(QLA_ERROR);
	}

	ha->firmware_state = mbox_sts[1];
	LEAVE("qla4000_get_firmware_state");
	return(QLA_SUCCESS);
}

STATIC uint8_t
qla4000_init_firmware(scsi_qla_host_t *ha)
{
	uint8_t  status = QLA_ERROR;
	uint32_t timeout_count;

	ENTER("qla4000_init_firmware");

	qla4000_initialize_fw_cb(ha);

	for (timeout_count = INIT_WAIT_COUNT * 1000; timeout_count > 0; timeout_count-=500)
	{
		/* Get firmware state */
		if (qla4000_get_firmware_state(ha) != QLA_SUCCESS)
		{
			QL4PRINT(QLP1|QLP2, printk("qla4000_init_firmware: unable to get firmware state\n"));
			goto exit_init_firmware;
		}

		switch (ha->firmware_state)
		{
		case FW_STATE_CONFIG_WAIT:
			/*
			 * The firmware has not yet been issued an Initialize
			 * Firmware command, so issue it now.
			 */
			if (qla4000_initialize_fw_cb(ha) == QLA_ERROR)
				goto exit_init_firmware;
			continue; /*go back and test for ready state (without waiting) */

		case FW_STATE_READY:
			/* The firmware is ready to process SCSI commands. */
			qla4000_enable_intrs(ha);
			status = QLA_SUCCESS;
			goto exit_init_firmware;

		case FW_STATE_WAIT_LOGIN:
		case FW_STATE_INITIALIZING:
			QL4PRINT(QLP7, printk("qla4000_init_firmware: waiting for firmware to initialize\n"));
			break;

		case FW_STATE_ERROR:
			QL4PRINT(QLP1, printk("qla4000_init_firmware: an unrecoverable error has occurred\n"));
			goto exit_init_firmware;
		}

		QL4PRINT(QLP11, printk("qla4000_init_firmware: delay 500 msec, time remaining %d\n", timeout_count));
		mdelay(500);
	}

	QL4PRINT(QLP1, printk("qla4000_init_firmware: FW Initialization timed out!\n"));

	exit_init_firmware:
	LEAVE("qla4000_init_firmware");
	return(status);
}

uint8_t
qla4000_login_device(scsi_qla_host_t *ha, uint16_t fw_ddb_index, uint16_t connection_id)
{
	ddb_entry_t *ddb_entry;
	static uint32_t    mbox_cmd[MBOX_REG_COUNT];
	static uint32_t    mbox_sts[MBOX_REG_COUNT];
	uint8_t     status = QLA_ERROR;


	ENTER("qla4000_login_device");

	QL4PRINT(QLP3, printk("qla4000_login_device: Login device %X\n", fw_ddb_index));
	ddb_entry = qla4000_lookup_ddb_by_fw_index(ha, fw_ddb_index);

	if (ddb_entry == NULL)
	{
		QL4PRINT(QLP1|QLP3, printk("qla4000_login_device: Invalid device %X\n", fw_ddb_index));
		goto exit_login;
	}

	if (qla4000_get_ddb_entry(ha, fw_ddb_index, NULL, NULL, NULL, &ddb_entry->device_state) == QLA_ERROR)
	{
		QL4PRINT(QLP1|QLP3, printk("qla4000_login_device: 1st get ddb entry failed\n"));
		goto exit_login;
	}

	if (ddb_entry->device_state == DDB_DS_SESSION_ACTIVE)
	{
		status = QLA_SUCCESS;
		QL4PRINT(QLP1|QLP3, printk("qla4000_login_device: login successful for index %x\n", ddb_entry->fw_ddb_index));
		goto exit_login;
	}

	mbox_cmd[0] = MBOX_CMD_CONN_CLOSE_SESS_LOGOUT;
	mbox_cmd[1] = fw_ddb_index;
	mbox_cmd[2] = connection_id;
	mbox_cmd[3] = LOGOUT_OPTION_RELOGIN;

	if (qla4000_mailbox_command(ha, 4, 2, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS)
	{
		QL4PRINT(QLP1|QLP3, printk("qla4000_login_device: MBOX_CMD_CONN_CLOSE_SESS_LOGOUT failed sts %04X",
					   mbox_sts[0]));
		if (mbox_sts[0] == 0x4005)
		{
			QL4PRINT(QLP1|QLP3, printk(", reason %04X\n", mbox_sts[1]));
		}
		else
		{
			QL4PRINT(QLP1|QLP3, printk("\n"));
		}

		goto exit_login;
	}

	status = QLA_SUCCESS;

	exit_login:
	LEAVE("qla4000_login_device");
	return(status);
}

uint8_t
qla4000_logout_device(scsi_qla_host_t *ha, uint16_t fw_ddb_index, uint16_t connection_id)
{
	uint8_t     status = QLA_ERROR;
	static uint32_t    mbox_cmd[MBOX_REG_COUNT];
	static uint32_t    mbox_sts[MBOX_REG_COUNT];
	ddb_entry_t *ddb_entry;
	uint32_t    old_device_state;

	ENTER("qla4000_logout_device");

	QL4PRINT(QLP3, printk("qla4000_logout_device: Logout device %x\n", fw_ddb_index));
	ddb_entry = qla4000_lookup_ddb_by_fw_index(ha, fw_ddb_index);

	if (ddb_entry == NULL)
	{
		QL4PRINT(QLP1|QLP3, printk("qla4000_logout_device: Invalid device %X\n", fw_ddb_index));
		goto exit_logout;
	}

	if (qla4000_get_ddb_entry(ha, fw_ddb_index, NULL, NULL, NULL, &old_device_state) != QLA_SUCCESS)
	{
		QL4PRINT(QLP3, printk("qla4000_logout_device: get_ddb_entry failed\n"));
		goto exit_logout;
	}

	mbox_cmd[0] = MBOX_CMD_CONN_CLOSE_SESS_LOGOUT;
	mbox_cmd[1] = fw_ddb_index;
	mbox_cmd[2] = connection_id;
	mbox_cmd[3] = LOGOUT_OPTION_CLOSE_SESSION;

	if (qla4000_mailbox_command(ha, 4, 2, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS)
	{
		QL4PRINT(QLP1|QLP3, printk("qla4000_logout_device: MBOX_CMD_CONN_CLOSE_SESS_LOGOUT failed sts %04X",
					   mbox_sts[0]));
		if (mbox_sts[0] == 0x4005)
		{
			QL4PRINT(QLP1|QLP3, printk(", reason %04X\n", mbox_sts[1]));
		}
		else
		{
			QL4PRINT(QLP1|QLP3, printk("\n"));
		}

		goto exit_logout;
	}

	status = QLA_SUCCESS;

	exit_logout:
	LEAVE("qla4000_logout_device");
	return(status);
}

/*
 * qla4000_relogin_device
 *      This routine does a session relogin with the specified device.
 *		The ddb entry must be assigned prior to making this call.
 *
 * Input:
 *      ha = adapter block pointer.
 *
 * Returns:
 *      QLA_SUCCESS = success
 * 		QLA_ERROR   = error
 */
uint8_t
qla4000_relogin_device(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry)
{

	ENTER("qla4000_relogin_device");

	printk("qla4000: Relogin index [0x%x]\n", ddb_entry->fw_ddb_index);

	qla4000_set_ddb_entry(ha, ddb_entry->fw_ddb_index, NULL);

	/* set_bit(DEV_FLAG_RELOGIN, &ddb_entry->flags); */
	if (atomic_read(&ddb_entry->relogin_retry_count) != 0)
		atomic_set(&ddb_entry->relogin_timer, 
			   RELOGIN_TIMEOUT);

	LEAVE("qla4000_relogin_device");
	return(QLA_SUCCESS);
}

uint8_t
qla4000_send_internal_scsi_passthru(scsi_qla_host_t *ha,
				    EXT_SCSI_PASSTHRU_ISCSI *scsi_pass,
				    void *data_ptr,
				    uint32_t data_len)
{
	uint8_t     status = QLA_ERROR;
	srb_t       *srb;
	ddb_entry_t *ddb_entry;
	Scsi_Device *scsi_device;
	Scsi_Cmnd   *cmd;
	int i;
	unsigned long flags = 0;

	ENTER("qla4000_send_internal_scsi_passthru");
	spin_lock_irqsave(&ha->dma_buf.spinlock, flags);
	/* spin_lock(&ha->dma_buf.spinlock); */

	/* ---- Allocate larger DMA buffer, if neccessary ---- */
	if (ha->dma_buf.buf_len < sizeof(*cmd) + sizeof(*scsi_device) &&
	    qla4000_resize_dma_buf(ha, &ha->dma_buf, sizeof(*cmd) + sizeof(*scsi_device)) != QLA_SUCCESS)
	{
		QL4PRINT(QLP2|QLP4,
			 printk("qla4000_send_internal_scsi_passthru: ERROR cannot allocate requested DMA buffer size 0x%x.\n",
				sizeof(*cmd) + sizeof(*scsi_device)));
		goto exit_send_scsi_passthru;
	}

	memset(ha->dma_buf.virt_addr, 0, ha->dma_buf.buf_len);
	cmd = ha->dma_buf.virt_addr;
	scsi_device = ha->dma_buf.virt_addr + sizeof(*cmd);

	/* ---- Make sure device exists ---- */
	ddb_entry = qla4000_lookup_ddb_by_SCSIID(ha, scsi_pass->Addr.Bus, scsi_pass->Addr.Target);
	if (ddb_entry == NULL)
	{
		QL4PRINT(QLP2|QLP4, printk("qla4000_send_internal_scsi_passthru: invalid device (b%d,t%d,l%d) specified.\n",
					   scsi_pass->Addr.Bus, scsi_pass->Addr.Target, scsi_pass->Addr.Lun));
		goto exit_send_scsi_passthru;
	}

	/* ---- Make sure device is in an active state ---- */
	if (ddb_entry->device_state != DDB_DS_SESSION_ACTIVE)
	{
		QL4PRINT(QLP2|QLP4, printk("qla4000_send_internal_scsi_passthru: device (b%d,t%d) not in active state.\n",
					   scsi_pass->Addr.Bus, scsi_pass->Addr.Target));
		goto exit_send_scsi_passthru;
	}

	/* ---- Retrieve srb from pool ---- */
	srb = del_from_free_srb_q_head(ha);
	if (srb == NULL)
	{
		QL4PRINT(QLP5|QLP2|QLP3, printk("qla4000_internal_scsi_passthru: srb not available\n"));
		goto exit_send_scsi_passthru;
	}

	/* ---- Fill in the SCSI command structure ---- */
	cmd->channel         = scsi_pass->Addr.Bus;
	cmd->target          = scsi_pass->Addr.Target;
	cmd->lun             = scsi_pass->Addr.Lun;
	cmd->device          = scsi_device;
	cmd->host            = ha->host;
	cmd->request_buffer  = data_ptr;
	cmd->request_bufflen = data_len;
	cmd->scsi_done       = qla4000_scsi_pass_done;
	CMD_TIMEOUT(cmd)     = INTERNAL_PASSTHRU_TIMEOUT * HZ;
	CMD_SP(cmd) = (char *)srb;
	srb->cmd = cmd;
	srb->ddb_entry = ddb_entry;
	srb->lun_entry = qla4000_lookup_lun_handle(ha, ddb_entry, cmd->lun);


	if (scsi_pass->CdbLength == 6 ||
	    scsi_pass->CdbLength == 10 ||
	    scsi_pass->CdbLength == 12)
	{
		cmd->cmd_len = scsi_pass->CdbLength;
	}
	else
	{
		QL4PRINT(QLP2|QLP4, printk("qla4000_internal_scsi_passthru: Unsupported CDB length 0x%x \n", cmd->cmd_len));
		goto exit_send_scsi_passthru;
	}

	switch (scsi_pass->Direction)
	{
	case EXT_DEF_SCSI_PASSTHRU_DATA_IN:  cmd->sc_data_direction = SCSI_DATA_READ;  break;
	case EXT_DEF_SCSI_PASSTHRU_DATA_OUT: cmd->sc_data_direction = SCSI_DATA_WRITE; break;
	default:                             cmd->sc_data_direction = SCSI_DATA_NONE;  break;
	}

	memcpy(cmd->cmnd,      scsi_pass->Cdb, cmd->cmd_len);
	memcpy(cmd->data_cmnd, scsi_pass->Cdb, cmd->cmd_len);

	QL4PRINT(QLP4, printk("qla4000_internal_scsi_passthru (%d:%d:%d:%d): CDB = ",
			      ha->host_no, cmd->channel, cmd->target, cmd->lun));
	for (i = 0; i < cmd->cmd_len; i++)
		QL4PRINT(QLP4, printk("%02X ", cmd->cmnd[i]));
	QL4PRINT(QLP4, printk("\n"));

	/* ---- prepare for receiving completion ---- */
	ha->ioctl_scsi_pass_in_progress = TRUE;
	ha->ioctl_tov = CMD_TIMEOUT(cmd);

	qla4000_ioctl_sem_init(ha);
	CMD_COMPL_STATUS(cmd)  = (int) INVALID_ENTRY;
	CMD_PASSTHRU_TYPE(cmd) = TRUE;

	/* ---- send command to adapter ---- */
	QL4PRINT(QLP4, printk("qla4000_internal_scsi_passthru (%d:%d:%d:%d): sending command.\n",
			      ha->host_no, cmd->channel, cmd->target, cmd->lun));

	ha->ioctl_cmpl_timer.expires = jiffies + ha->ioctl_tov;
	add_timer(&ha->ioctl_cmpl_timer);

	if (qla4000_send_command_to_isp(ha, srb) != QLA_SUCCESS)
	{
		add_to_free_srb_q(ha, srb);
		QL4PRINT(QLP5|QLP2|QLP3, printk("qla4000_internal_scsi_passthru: error sending cmd to isp\n"));
		del_timer(&ha->ioctl_cmpl_timer);
		goto exit_send_scsi_passthru;
	}

	down(&ha->ioctl_cmpl_sem);

	/*******************************************************
	 *						       *
	 *             Passthru Completion                     *
	 *						       *
	 *******************************************************/
	del_timer(&ha->ioctl_cmpl_timer);

	/* ---- check for timeout --- */
	if (ha->ioctl_scsi_pass_in_progress == 1)
	{
		QL4PRINT(QLP2|QLP4, printk("%s: ERROR = command timeout.\n",
					__func__));

		spin_lock_irq(&io_request_lock);
		spin_unlock_irqrestore(&ha->dma_buf.spinlock, flags);
		qla4000_reset_target(ha, ddb_entry, 0);
		spin_lock_irqsave(&ha->dma_buf.spinlock, flags);
		spin_unlock_irq(&io_request_lock);
		goto exit_send_scsi_passthru;
	}

	/* --- Return info from status entry --- */
	scsi_pass->Reserved[0] = (uint8_t) CMD_SCSI_STATUS(cmd);
	scsi_pass->Reserved[1] = (uint8_t) CMD_COMPL_STATUS(cmd);
	scsi_pass->Reserved[2] = (uint8_t) CMD_ACTUAL_SNSLEN(cmd);
	scsi_pass->Reserved[3] = (uint8_t) CMD_HOST_STATUS(cmd);

	/* ---- Copy SCSI Passthru structure with updated sense buffer to user space ---- */
	if (CMD_ACTUAL_SNSLEN(cmd))
	{
		memcpy(scsi_pass->SenseData, CMD_SNSP(cmd),
		       MIN(CMD_ACTUAL_SNSLEN(cmd), sizeof(scsi_pass->SenseData)));
	}

	/* ---- check for command completion --- */
	if (CMD_COMPL_STATUS(cmd) == INVALID_ENTRY)
	{
		QL4PRINT(QLP2|QLP6, printk("qla4000_internal_scsi_passthru: ERROR = command not completed.\n"));
		goto exit_send_scsi_passthru;
	}
	else if (CMD_HOST_STATUS(cmd) == DID_OK)
	{
		status = QLA_SUCCESS;
	}
	else if (CMD_COMPL_STATUS(cmd) == SCS_DATA_UNDERRUN)
	{
		QL4PRINT(QLP2|QLP6, printk("qla4000_internal_scsi_passthru: Data underrun.  Resid = 0x%x\n",
					   CMD_RESID_LEN(cmd)));
		scsi_pass->Reserved[4] = MSB(CMD_RESID_LEN(cmd));
		scsi_pass->Reserved[5] = LSB(CMD_RESID_LEN(cmd));
	}
	else if (CMD_COMPL_STATUS(cmd) == SCS_DATA_OVERRUN)
	{
		QL4PRINT(QLP2|QLP6, printk("qla4000_internal_scsi_passthru: Data overrun.  Resid = 0x%x\n",
					   CMD_RESID_LEN(cmd)));
		scsi_pass->Reserved[4] = MSB(CMD_RESID_LEN(cmd));
		scsi_pass->Reserved[5] = LSB(CMD_RESID_LEN(cmd));
	}
	else
	{
		QL4PRINT(QLP2|QLP6, printk("qla4000_internal_scsi_passthru: Command completed in ERROR. cs=%04x, ss=%-4x\n",
					   CMD_COMPL_STATUS(cmd), CMD_SCSI_STATUS(cmd)));

		goto exit_send_scsi_passthru;
	}

	exit_send_scsi_passthru:
	spin_unlock_irqrestore(&ha->dma_buf.spinlock, flags);
	/* spin_unlock(&ha->dma_buf.spinlock); */
	LEAVE("qla4000_send_internal_scsi_passthru");
	return(status);
}

uint8_t
qla4000_send_inquiry_cmd(scsi_qla_host_t *ha, uint16_t target, uint16_t lun, inquiry_t *inquiry)
{
	uint8_t     status = QLA_ERROR;
	EXT_SCSI_PASSTHRU_ISCSI *scsi_pass;

	ENTER("qla4000_send_inquiry_cmd");
	scsi_pass = (EXT_SCSI_PASSTHRU_ISCSI *) kmalloc(sizeof(*scsi_pass), GFP_KERNEL);
	if (scsi_pass)
	{
		/* set up SCSI Passthru structure */
		memset(scsi_pass, 0, sizeof(*scsi_pass));
		scsi_pass->Addr.Bus = 0;
		scsi_pass->Addr.Target = target;
		scsi_pass->Addr.Lun = lun;
		scsi_pass->Cdb[0] = INQUIRY;
		scsi_pass->Cdb[1] = lun << 5;
		scsi_pass->Cdb[4] = sizeof(*inquiry);
		scsi_pass->CdbLength = 6;
		scsi_pass->Direction = EXT_DEF_SCSI_PASSTHRU_DATA_IN;
		memset(inquiry, 0, sizeof(*inquiry));

		QL4PRINT(QLP3, printk("qla4000_send_inquiry_cmd: %d:%d:%d:%d:\n",
				      ha->host_no, 0, target, lun));
		status = qla4000_send_internal_scsi_passthru(ha, scsi_pass, inquiry, sizeof(*inquiry));

		kfree(scsi_pass);
	}

	LEAVE("qla4000_send_inquiry_cmd");
	return(status);
}

uint8_t
qla4000_send_report_luns_cmd(scsi_qla_host_t *ha, uint16_t target, report_luns_t *report_luns)
{
	uint8_t status = QLA_ERROR;
	EXT_SCSI_PASSTHRU_ISCSI *scsi_pass;

	ENTER("qla4000_send_report_luns_cmd");
	scsi_pass = (EXT_SCSI_PASSTHRU_ISCSI *) kmalloc(sizeof(*scsi_pass), GFP_KERNEL);
	if (scsi_pass)
	{
		/* set up SCSI Passthru structure */
		memset(scsi_pass, 0, sizeof(*scsi_pass));
		scsi_pass->Addr.Bus = 0;
		scsi_pass->Addr.Target = target;
		scsi_pass->Cdb[0] = REPORT_LUNS;
		scsi_pass->Cdb[8] = MSB(sizeof(*report_luns));
		scsi_pass->Cdb[9] = LSB(sizeof(*report_luns));
		scsi_pass->CdbLength = 12;
		scsi_pass->Direction = EXT_DEF_SCSI_PASSTHRU_DATA_IN;
		memset(report_luns, 0, sizeof(*report_luns));

		QL4PRINT(QLP3, printk("qla4000_send_report_luns_cmd: %d:%d:%d:\n",
				      ha->host_no, 0, target));
		status = qla4000_send_internal_scsi_passthru(ha, scsi_pass, report_luns, sizeof(*report_luns));

		if ((status == QLA_SUCCESS) && (SENSE_KEY(scsi_pass->SenseData) != 0))
		{
			status = QLA_ERROR;
		}

		kfree(scsi_pass);
	}

	LEAVE("qla4000_send_report_luns_cmd");
	return(status);
}

uint8_t
qla4000_send_test_unit_ready_cmd(scsi_qla_host_t *ha, uint16_t target, uint16_t lun, uint8_t *scsi_status)
{
	uint8_t status = QLA_ERROR;
	EXT_SCSI_PASSTHRU_ISCSI *scsi_pass;

	ENTER("qla4000_send_test_unit_ready_cmd");
	scsi_pass = (EXT_SCSI_PASSTHRU_ISCSI *) kmalloc(sizeof(*scsi_pass), GFP_KERNEL);
	if (scsi_pass)
	{
		/* set up SCSI Passthru structure */
		memset(scsi_pass, 0, sizeof(*scsi_pass));
		scsi_pass->Addr.Bus = 0;
		scsi_pass->Addr.Target = target;
		scsi_pass->Addr.Lun = lun;
		scsi_pass->Cdb[0] = TEST_UNIT_READY;
		scsi_pass->Cdb[1] = lun << 5;
		scsi_pass->CdbLength = 6;

		status = qla4000_send_internal_scsi_passthru(ha, scsi_pass, NULL, 0);
		if (scsi_status) *scsi_status = scsi_pass->Reserved[0];

		kfree(scsi_pass);
	}

	LEAVE("qla4000_send_test_unit_ready_cmd");
	return(status);
}


uint8_t
qla4000_get_ddb_entry(
		     scsi_qla_host_t *ha,
		     uint16_t        fw_ddb_index,
		     DEVDB_ENTRY     *fw_ddb_entry,
		     uint32_t        *num_valid_ddb_entries,
		     uint32_t        *next_ddb_index,
		     uint32_t        *device_state)
{
	uint8_t         status = QLA_ERROR;
	static uint32_t        mbox_cmd[MBOX_REG_COUNT];
	static uint32_t        mbox_sts[MBOX_REG_COUNT];
	DEVDB_ENTRY     *tmp_data = 0;
	dma_addr_t      tmp_data_phys_addr = 0;

	ENTER("qla4000_get_ddb_entry");

	/* Make sure the device index is valid */
	if (fw_ddb_index >= MAX_DDB_ENTRIES)
	{
		QL4PRINT(QLP1|QLP3|QLP7, printk("qla4000_get_ddb_entry: device index (%X) out of range.\n", fw_ddb_index));
		goto exit_get_ddb;
	}

	if (fw_ddb_entry)
	{
		tmp_data = pci_alloc_consistent(ha->pdev, sizeof(*tmp_data), &tmp_data_phys_addr);
		if (tmp_data == NULL)
		{
			QL4PRINT(QLP1|QLP3|QLP7, printk("qla4000_get_ddb_entry: device index (%X) out of range.\n", fw_ddb_index));
			goto exit_get_ddb;
		}

		memset(tmp_data, 0, sizeof(*tmp_data));
	}

	mbox_cmd[0] = MBOX_CMD_GET_DATABASE_ENTRY;
	mbox_cmd[1] = (uint32_t) fw_ddb_index;
	mbox_cmd[2] = LSL(tmp_data_phys_addr);
	mbox_cmd[3] = MSL(tmp_data_phys_addr);

	if (qla4000_mailbox_command(ha, 4, 5, &mbox_cmd[0], &mbox_sts[0]) == QLA_ERROR)
	{
		QL4PRINT(QLP1|QLP3|QLP7, printk("qla4000_get_ddb_entry: MBOX_CMD_GET_DATABASE_ENTRY failed with status 0x%04X\n", mbox_sts[0]));
		goto exit_get_ddb;
	}

	QL4PRINT(QLP3|QLP7, printk("qla4000_get_ddb_entry: (ha%d) Device %2x MB0 %04x Tot %04x Next %04x State %04x\n",
				ha->host_no,
				   fw_ddb_index, mbox_sts[0], mbox_sts[2], mbox_sts[3], mbox_sts[4]));

	if (fw_ddb_index != mbox_sts[1])
	{
		QL4PRINT(QLP1|QLP3|QLP7, printk("qla4000_get_ddb_entry: device index mismatch %04X != %04X.\n", fw_ddb_index, mbox_sts[1]));
		goto exit_get_ddb;
	}

	if (fw_ddb_entry)
	{
		memcpy(fw_ddb_entry, tmp_data, sizeof(*tmp_data));

		QL4PRINT(QLP10, printk("fw_ddb_entry = %p\n", fw_ddb_entry));
		qla4000_dump_bytes(QLP10, fw_ddb_entry, sizeof(*fw_ddb_entry));

		QL4PRINT(QLP3|QLP7, printk("qla4000_get_ddb_entry: (ha%d) device %2x %d.%d.%d.%d:%04d \"%s\"\n",
					   ha->host_no, fw_ddb_index,
					   fw_ddb_entry->ipAddr[0], fw_ddb_entry->ipAddr[1],
					   fw_ddb_entry->ipAddr[2], fw_ddb_entry->ipAddr[3],
					   fw_ddb_entry->portNumber, fw_ddb_entry->iscsiName));
	}

	if (num_valid_ddb_entries)
		*num_valid_ddb_entries = mbox_sts[2];

	if (next_ddb_index)
		*next_ddb_index = mbox_sts[3];

	if (device_state)
		*device_state = mbox_sts[4];

	status = QLA_SUCCESS;

	exit_get_ddb:
	if (tmp_data)
		pci_free_consistent(ha->pdev, sizeof(*tmp_data), tmp_data, tmp_data_phys_addr);

	LEAVE("qla4000_get_ddb_entry");
	return(status);
}

uint8_t
qla4000_set_ddb_entry(
		     scsi_qla_host_t *ha,
		     uint16_t        fw_ddb_index,
		     DEVDB_ENTRY     *fw_ddb_entry)
{
	uint8_t         status = QLA_ERROR;
	static uint32_t         mbox_cmd[MBOX_REG_COUNT];
	static uint32_t         mbox_sts[MBOX_REG_COUNT];
	DEVDB_ENTRY     *tmp_data = 0;
	dma_addr_t      tmp_data_phys_addr = 0;

	ENTER("qla4000_set_ddb_entry");

	if (fw_ddb_entry)
	{
		tmp_data = pci_alloc_consistent(ha->pdev, sizeof(*tmp_data), &tmp_data_phys_addr);
		if (tmp_data == NULL)
		{
			QL4PRINT(QLP1|QLP3|QLP7, printk("qla4000_get_ddb_entry: device index (%X) out of range.\n", fw_ddb_index));
			goto exit_set_ddb;
		}

		memcpy(tmp_data, fw_ddb_entry, sizeof(*tmp_data));
	}

	QL4PRINT(QLP3|QLP7, printk("qla4000_set_ddb_entry: Device %2x\n",
				   fw_ddb_index));

	/* Do not wait for completion. The firmware will send us an
	 * ASTS_DATABASE_CHANGED (0x8014) to notify us of the login status.
	 */
	mbox_cmd[0] = MBOX_CMD_SET_DATABASE_ENTRY;
	mbox_cmd[1] = (uint32_t) fw_ddb_index;
	mbox_cmd[2] = LSL(tmp_data_phys_addr);
	mbox_cmd[3] = MSL(tmp_data_phys_addr);

	qla4000_mailbox_command(ha, 4, 0, &mbox_cmd[0], &mbox_sts[0]);
	status = QLA_SUCCESS;

	exit_set_ddb:
	if (tmp_data)
		pci_free_consistent(ha->pdev, sizeof(*tmp_data), tmp_data, tmp_data_phys_addr);

	LEAVE("qla4000_set_ddb_entry");
	return(status);
}

uint8_t qla4000_update_ddb_entry(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry)
{
	DEVDB_ENTRY *fw_ddb_entry;
	uint32_t        fw_ddb_index;
	uint8_t status = QLA_ERROR;

	ENTER("qla4000_update_ddb_entry");
	spin_lock(&ha->dma_buf.spinlock);

	if (ddb_entry == NULL)
	{
		QL4PRINT(QLP2, printk("qla4000_update_ddb_entry: ddb_entry is NULL\n"));
		goto exit_update_ddb;
	}


	/* Make sure the dma buffer is valid */
	if (ha->dma_buf.buf_len < sizeof(*fw_ddb_entry))
		qla4000_resize_dma_buf(ha, &ha->dma_buf, sizeof(*fw_ddb_entry));

	if (!ha->dma_buf.virt_addr)
	{
		QL4PRINT(QLP2, printk("qla4000_update_ddb_entry: dma buffer inaccessible.\n"));
		goto exit_update_ddb;
	}

	fw_ddb_entry = (DEVDB_ENTRY *) ha->dma_buf.virt_addr;
	fw_ddb_index = ddb_entry->fw_ddb_index;
	if (qla4000_get_ddb_entry(ha, fw_ddb_index, fw_ddb_entry, NULL, NULL, &ddb_entry->device_state) == QLA_ERROR)
	{
		QL4PRINT(QLP2, printk("qla4000_update_ddb_entry: failed get_ddb_entry for fw_ddb_index %x\n",
				      fw_ddb_index));
		kfree(fw_ddb_entry);
		goto exit_update_ddb;
	}

	status = QLA_SUCCESS;
	switch (ddb_entry->device_state)
	{
	case DDB_DS_SESSION_ACTIVE:
		ddb_entry->target_session_id = fw_ddb_entry->TSID;
		ddb_entry->task_mgmt_timeout = fw_ddb_entry->taskMngmntTimeout;
		ddb_entry->connection_id     = 0;
		ddb_entry->CmdSn             = 0;
		ddb_entry->exe_throttle      = fw_ddb_entry->exeThrottle;
		ddb_entry->relogin_timeout   = fw_ddb_entry->logoutMaxTime;

		QL4PRINT(QLP7, printk("qla4000_update_ddb_entry: device %2x \"%s\"\n",
				      fw_ddb_index, ddb_state_msg[ddb_entry->device_state]));
		break;

	case DDB_DS_NO_CONNECTION_ACTIVE:
	case DDB_DS_NO_SESSION_ACTIVE:
	case DDB_DS_SESSION_FAILED:
		ddb_entry->target_session_id = 0;
		ddb_entry->task_mgmt_timeout = 0;
		ddb_entry->connection_id     = 0;
		ddb_entry->CmdSn             = 0;
		ddb_entry->exe_throttle      = 0;

		QL4PRINT(QLP7, printk("qla4000_update_ddb_entry: device %2x \"%s\"\n",
				      fw_ddb_index, ddb_state_msg[ddb_entry->device_state]));
		break;

	case DDB_DS_UNASSIGNED:
	case DDB_DS_DISCOVERY:
	case DDB_DS_LOGGING_OUT:
		QL4PRINT(QLP7, printk("qla4000_update_ddb_entry: device %2x \"%s\"\n",
				      fw_ddb_index, ddb_state_msg[ddb_entry->device_state]));
		break;

	default:
		QL4PRINT(QLP7, printk("qla4000_get_ddb_entry: Device %2x State %x. Illegal state\n",
				      fw_ddb_index, ddb_entry->device_state));

	}

	exit_update_ddb:
	spin_unlock(&ha->dma_buf.spinlock);
	LEAVE("qla4000_update_ddb_entry");
	return(status);
}

/*
 * qla4000_alloc_lun
 *	  This function enables the specified lun in the ddb_entry's lun table.
 */
static inline void
qla4000_alloc_lun(ddb_entry_t *ddb_entry, uint16_t lun)
{
	lun_entry_t *lun_entry;

	ENTER("qla4000_alloc_lun");
	if (ddb_entry->lun_table[lun] == 0)
	{
		QL4PRINT(QLP3, printk("qla4000_alloc_lun: target %d, added lun %d\n",
				      ddb_entry->target, lun));

		lun_entry = kmalloc(sizeof(*lun_entry), GFP_KERNEL);
		if (lun_entry == 0)
		{
			QL4PRINT(QLP2, printk("qla4000_alloc_lun: Error allocating memory for target %d, lun %d\n",
					      ddb_entry->target, lun));
			goto exit_create_lun;
		}

		memset(lun_entry, 0, sizeof(*lun_entry));
		ddb_entry->lun_table[lun] = lun_entry;
		lun_entry->lun = lun;
		lun_entry->lun_state = LS_LUN_READY;
		ddb_entry->num_valid_luns++;
		spin_lock_init(&lun_entry->lun_lock);
	}

	exit_create_lun:
	LEAVE("qla4000_alloc_lun");
}

/*
 * qla4000_discover_target_luns
 *	  This function performs a discovery on the given target,
 *    as well as all possible luns, and links the ddb with the
 *	  target_map.
 */
static uint8_t
qla4000_discover_target_luns(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, uint32_t target)
{
	uint8_t status = QLA_ERROR;
	report_luns_t *report_luns;
	inquiry_t *inquiry;
	uint16_t lun = 0;
	uint8_t discovered_via_report_luns = FALSE;
	uint8_t discovered_via_inquiry = FALSE;

	ENTER("qla4000_discover_target_luns");

	/* Allocate structures to do discovery */
	report_luns = (report_luns_t*) kmalloc(sizeof(*report_luns), GFP_KERNEL);
	inquiry = (inquiry_t *) kmalloc(sizeof(*inquiry), GFP_KERNEL);
	if ((!report_luns) || (!inquiry))
	{
		QL4PRINT(QLP2, printk("qla4000_discover_target_luns: unable to allocate memory\n"));
		goto exit_discover_target_and_luns;
	}

	/* Initialize ddb_entry for target/lun discovery */
	ddb_entry->target       = target;
	ha->target_map[target]  = ddb_entry;
	/*atomic_set(&ddb_entry->state, DEV_STATE_ONLINE);*/

	/* First create pseudo Lun 0 to issue SCSI commands */
	qla4000_alloc_lun(ddb_entry, lun);

	/* Discover via Report Luns */
	if (qla4000_send_report_luns_cmd(ha, target, report_luns) == QLA_SUCCESS)
	{
		int index = 0;
		uint32_t num_luns;

		num_luns = __be32_to_cpu(report_luns->lun_list_length) / 8;

		QL4PRINT(QLP7, printk("qla4000_discover_target_luns: target %d has %d lun(s)\n",
				      target, num_luns));

		if (num_luns)
		{
			/* If lun 0 doesn't exist, keep pseudo lun 0,
			 * in addition to all luns from report_luns
			 */
			if (report_luns->lun[0].single_level_lun != 0)
			{
				num_luns++;
				index = 0;
			}
			else
				index = 1;

			if (num_luns > MAX_LUNS) num_luns = MAX_LUNS;

			for (; index < num_luns; index++)
				qla4000_alloc_lun(ddb_entry, (uint16_t) report_luns->lun[index].single_level_lun);

			discovered_via_report_luns = TRUE;
			status = QLA_SUCCESS;
		}
	}

	/* Discover via SCSI Inquiry */
	if ((discovered_via_report_luns == FALSE) &&
	    (qla4000_send_inquiry_cmd(ha, target, lun, inquiry) == QLA_SUCCESS) &&
	    (inquiry->data[0] != 0x7f) && (inquiry->data[0] != 0x3f))
	{

		/* A target with lun 0 exists here.  Add SCSI target/lun information */
		QL4PRINT(QLP7, printk("qla4000_discover_target_luns: target %d, mapped to fw_ddb_index %x\n",
				      target, ddb_entry->fw_ddb_index));

		discovered_via_inquiry = TRUE;
		status = QLA_SUCCESS;

		/* search for consecutive luns via inquiry command */
		for (lun = 1; lun < MAX_LUNS; lun++)
		{
			qla4000_alloc_lun(ddb_entry, lun);
			if ((qla4000_send_inquiry_cmd(ha, target, lun, inquiry) == QLA_SUCCESS) &&
			    ((inquiry->data[0] == 0x7f) || (inquiry->data[0] == 0x3f)))
			{
				/* If a lun was NOT found, skip searching
				 * remaining luns.  Proceed to next target
				 */
				qla4000_free_lun(ddb_entry, lun);
				break;
			}
		}
	}

	if ((discovered_via_report_luns == FALSE) &&
	    (discovered_via_inquiry == FALSE))
	{
		/* This target was not found, unlink it from target map */

		qla4000_free_lun(ddb_entry, lun);
		ddb_entry->target       = 0;
		ha->target_map[target]  = 0;
		atomic_set(&ddb_entry->state, DEV_STATE_DEAD);
	}

	exit_discover_target_and_luns:
	if (report_luns) kfree(report_luns);
	if (inquiry) kfree(inquiry);

	LEAVE("qla4000_discover_target_luns");
	return(status);
}

/*
 * qla4000_map_targets_to_ddbs
 *	  This function iterates through the existing list of ddb_entries
 *    and discovers the corresponding SCSI target and luns.
 */
static uint8_t
qla4000_map_targets_to_ddbs(scsi_qla_host_t *ha)
{
	uint8_t status = QLA_ERROR;
	struct list_head *ptr, *next; /* used for traversing ddb_list */
	uint16_t target = 0;

	ENTER("qla4000_map_targets_to_ddbs");

	list_for_each_safe(ptr, next, &ha->ddb_list)
	{
		ddb_entry_t *ddb_entry;

		/* Get ddb_entry */
		ddb_entry = list_entry(ptr, ddb_entry_t, list_entry);

		/* Perform target/lun discovery */
		if (qla4000_discover_target_luns(ha, ddb_entry, target) == QLA_SUCCESS)
		{
			status = QLA_SUCCESS;
			target++;
		}

		if (target == MAX_TARGETS)
			break;
	}

	LEAVE("qla4000_map_targets_to_ddbs");
	return(status);

}

/*
 * qla4000_alloc_ddb
 *    Given a fw_ddb_index, this function allocates a ddb_entry and
 *    inserts it into the ddb list.
 */
static ddb_entry_t *
qla4000_alloc_ddb(scsi_qla_host_t *ha, uint32_t fw_ddb_index)
{
	ddb_entry_t     *ddb_entry;

	QL4PRINT(QLP3, printk("qla4000_alloc_ddb: fw_ddb_index [%x]\n", fw_ddb_index));
	ddb_entry = (ddb_entry_t *) kmalloc(sizeof(*ddb_entry), GFP_KERNEL);
	if (ddb_entry == NULL)
	{
		QL4PRINT(QLP1|QLP3|QLP7,
			 printk("qla4000_create_device: Unable to allocate memory to add fw_ddb_index [%x]\n",
				fw_ddb_index));
	}
	else
	{
		memset(ddb_entry, 0, sizeof(*ddb_entry));
		ddb_entry->fw_ddb_index = fw_ddb_index;
		atomic_set(&ddb_entry->port_down_timer,
			   ha->port_down_retry_count);
		atomic_set(&ddb_entry->relogin_timer, 0);
		atomic_set(&ddb_entry->state, DEV_STATE_ONLINE);
		list_add_tail(&ddb_entry->list_entry, &ha->ddb_list);
		ha->fw_ddb_index_map[fw_ddb_index] = ddb_entry;
	}

	return(ddb_entry);
}

/*
 * qla4000_build_ddb_list
 *    This routine searches for all valid firmware ddb entries and builds
 *    the ddb_list.
 */
static uint8_t
qla4000_build_ddb_list(scsi_qla_host_t *ha)
{
	uint8_t         status = QLA_ERROR;
	uint32_t        fw_ddb_index = 0;
	uint32_t        next_fw_ddb_index = 0;
	uint32_t        ddb_state;
	ddb_entry_t     *ddb_entry;

	ENTER("qla4000_build_ddb_list");

	for (fw_ddb_index = 0; 
	     fw_ddb_index < MAX_DDB_ENTRIES; 
	     fw_ddb_index = next_fw_ddb_index)
	{
		/* First, let's see if a device exists hee */
		if (qla4000_get_ddb_entry(ha, fw_ddb_index, NULL, NULL, 
					  &next_fw_ddb_index, &ddb_state)
		     == QLA_ERROR)
		{
			QL4PRINT(QLP1|QLP3,
				 printk("qla4000_build_ddb_list: get_ddb_entry, "
					"fw_ddb_index %x failed",
					fw_ddb_index));
			goto exit_build_ddb_list;
		}

		/* If the device is logged in (SESSION_ACTIVE) then
		 * add it to internal our ddb list. */
		if (ddb_state == DDB_DS_SESSION_ACTIVE)
		{
			/* Allocate a device structure */
			ddb_entry = qla4000_alloc_ddb(ha, fw_ddb_index);
			if (ddb_entry == NULL)
			{
				QL4PRINT(QLP1|QLP3,
					 printk("qla4000_build_ddb_list: Unable "
						"to allocate memory for device "
						"at fw_ddb_index %x\n",
						fw_ddb_index));
				goto exit_build_ddb_list;
			}

			/* Fill in the device structure */
			if (qla4000_update_ddb_entry(ha, ddb_entry) == QLA_ERROR)
			{
				qla4000_free_ddb(ha, ddb_entry);
				QL4PRINT(QLP1|QLP3,
					 printk("qla4000_build_ddb_list: "
						"update_ddb_index,  fw_ddb_index "
						"%x failed\n",
						fw_ddb_index));
				goto exit_build_ddb_list;

			}

			/* if fw_ddb with session active state found,
			 * add to ddb_list */
			QL4PRINT(QLP7,
				 printk("qla4000_build_ddb_list: fw_ddb index "
					"0x%02X added to ddb list\n",
					fw_ddb_index));

			status = QLA_SUCCESS;
		}

		/* We know we've reached the last device when 
		 * next_fw_ddb_index is 0 */
		if (next_fw_ddb_index == 0)
			break;
	}

	exit_build_ddb_list:
	LEAVE("qla4000_build_ddb_list");
	return(status);
}

/*
 * qla4000_initialize_devices
 *      This routine obtains device information from the F/W database during driver load time.
 *      The device table is rebuilt from scratch.
 *
 * Input:
 *      ha = adapter block pointer.
 *
 * Returns:
 *      QLA_SUCCESS = success
 * 	    QLA_ERROR   = error
 */
STATIC uint8_t
qla4000_initialize_ddb_list(scsi_qla_host_t *ha)
{
	uint16_t        target, fw_ddb_index;
	uint8_t         status;

	ENTER("qla4000_initialize_ddb_list");


	/* Initialize internal DDB list and mappingss */
	for (target = 0; target < MAX_TARGETS; target++)
		ha->target_map[target] = (ddb_entry_t *) INVALID_ENTRY;

	for (fw_ddb_index = 0; fw_ddb_index < MAX_DDB_ENTRIES; fw_ddb_index++)
		ha->fw_ddb_index_map[fw_ddb_index] = (ddb_entry_t *) INVALID_ENTRY;

	/* First perform device discovery for active fw ddb indexes and build ddb list*/
	if ((status = qla4000_build_ddb_list(ha)) == QLA_ERROR)
	{
		if (((status = qla4000_reset_firmware(ha)) == QLA_SUCCESS) &&
		    ((status = qla4000_init_firmware(ha)) == QLA_SUCCESS))
		{
			int i = 0;
			do
			{
				status = qla4000_build_ddb_list(ha);
				i++;
			} while (i<3 && status==QLA_ERROR);
			if (status == QLA_ERROR)
			{
				printk(KERN_WARNING "qla4000: Failed to build device list. Device discovery failed.\n");
			}
		}
		else
		{
			printk(KERN_WARNING "qla4000: Failed to reset adapter. Device discovery failed.\n");
		}
	}
	/* Here we map a SCSI target to a fw_ddb_index and discover all possible luns.  */
	else if ((status == QLA_SUCCESS) &&
		 (status = qla4000_map_targets_to_ddbs(ha)) == QLA_ERROR)
	{
		printk(KERN_WARNING "qla4000: Failed map SCSI target(s) to ddb(s)\n");
	}

	LEAVE("qla4000_initialize_ddb_list");
	return(status);
}

/*
 * qla4000_reinitialize_devices
 *      This routine obtains device information from the F/W database after 
 *      firmware or adapter resets.	 The device table is preserved.
 *
 * Input:
 *      ha = adapter block pointer.
 *
 * Returns:
 *      QLA_SUCCESS = success
 * 	    QLA_ERROR   = error
 */
STATIC uint8_t
qla4000_reinitialize_ddb_list(scsi_qla_host_t *ha)
{
	uint8_t         status = QLA_SUCCESS;
	struct list_head *ptr, *next;
	ddb_entry_t *ddb_entry;

	ENTER("qla4000_reinitialize_ddb_list");

	/* Update the device information for all devices */
	list_for_each_safe(ptr, next, &ha->ddb_list)
	{
		ddb_entry = list_entry(ptr, ddb_entry_t, list_entry);

		if (ddb_entry)
		{
			qla4000_update_ddb_entry(ha, ddb_entry);

			if (ddb_entry->device_state == DDB_DS_SESSION_ACTIVE)
			{
				atomic_set(&ddb_entry->state, DEV_STATE_ONLINE);

				printk(/*KERN_INFO*/ "%s: %d:%d:%d: index [0x%x] marked ONLINE\n",
                                                 __FUNCTION__, ha->host_no, ddb_entry->bus, 
						     ddb_entry->target, ddb_entry->fw_ddb_index);
			}
			else
			{
				if (atomic_read(&ddb_entry->state) == 
				    DEV_STATE_ONLINE)
				{
					atomic_set(&ddb_entry->relogin_retry_count, MAX_RELOGIN_RETRIES);
					atomic_set(&ddb_entry->relogin_timer, 
						   RELOGIN_TIMEOUT);
					atomic_set(&ddb_entry->state,
						   DEV_STATE_MISSING);
					printk(/*KERN_INFO*/ "%s: %d:%d:%d: index [0x%x] marked MISSING\n",
                                                         __FUNCTION__, ha->host_no, ddb_entry->bus, 
							     ddb_entry->target, ddb_entry->fw_ddb_index);
				}
			}                        
		}
	}

	LEAVE("qla4000_reinitialize_ddb_list");
	return(status);
}

/*
 * qla4000_initialize_adapter
 *      Initialize board.
 *
 * Input:
 *      ha = adapter block pointer.
 *
 * Returns:
 *      QLA_SUCCESS = success
 * 		QLA_ERROR   = error
 */
STATIC uint8_t
qla4000_initialize_adapter(scsi_qla_host_t *ha, uint8_t ddb_list_todo)
{
	uint8_t      status;

	ENTER("qla4000_initialize_adapter");

	if ((status = qla4000_init_firmware(ha)) == QLA_ERROR)
	{
		printk(KERN_WARNING "qla4000: Failed to initialize firmware\n");
	}
	else if ((status = qla4000_get_fw_version(ha)) == QLA_ERROR)
	{
		printk(KERN_WARNING "qla4000: Failed to get firmware version\n");
	}
	else if ((status = qla4000_validate_mac_address(ha)) == QLA_ERROR)
	{
		printk(KERN_WARNING "qla4000: Failed to validate mac address\n");
	}
	else 
	{
		if (ddb_list_todo == 0)
		{
			/* 
			 * We want to preserve lun states (i.e. suspended, etc.)
			 * for recovery initiated by the driver.  So just update
			 * the device states for the existing ddb_list
			 */
			if (qla4000_reinitialize_ddb_list(ha) == QLA_ERROR)
				printk(KERN_WARNING "qla4000: Failed to initialize devices\n");
		}
		if (ddb_list_todo == 1)
		{
			/* 
			 * We want to build the ddb_list from scratch during
			 * driver initialization and recovery initiated by the
			 * INT_HBA_RESET IOCTL.
			 */

			/* reinitialize suspended lun queue */
			INIT_LIST_HEAD(&ha->suspended_lun_q);
			ha->suspended_lun_q_count = 0;

			/* free the ddb list if is not empty */
			if (!list_empty(&ha->ddb_list))
				qla4000_free_ddb_list(ha);

			if (qla4000_initialize_ddb_list(ha) == QLA_ERROR)
				printk(KERN_WARNING "qla4000: Failed to initialize devices\n");
		}
	}

	atomic_set(&ha->adapter_state, AS_READY);
	LEAVE("qla4000_initialize_adapter");
	return(status);
}

/*
 * qla4000_get_req_pkt
 *
 * This function performs the following tasks:
 * - returns the current request_in pointer (if queue not full)
 * - advances the request_in pointer
 * - checks for queue full
 * NOTE: Do Not Use with qla4000_send_cmd_to_isp() for performance reasons.
 *
 */
uint8_t
qla4000_get_req_pkt(scsi_qla_host_t *ha, QUEUE_ENTRY **queue_entry)
{
	ISP_REG_T *reg = ha->base_address;
	uint16_t  request_in;
	uint8_t   status = QLA_SUCCESS;

	ENTER("qla4000_get_req_pkt");

	*queue_entry = ha->request_ptr;

	/* get the latest request_in and request_out index */
	request_in = ha->request_in;
	ha->request_out  = (uint16_t) RD_REG_DWORD(&reg->requestQueueOutPointer);

	/* Advance request queue pointer and check for queue full */
	if (request_in == (REQUEST_QUEUE_DEPTH - 1))
	{
		request_in = 0;
		ha->request_ptr = ha->req_queue.virt_addr;
		QL4PRINT(QLP10, printk("qla4000_get_req_pkt: wraparound -- new request_in = %04x, new request_ptr = %p\n",
				       request_in, ha->request_ptr));
	}
	else
	{
		request_in++;
		ha->request_ptr++;
		QL4PRINT(QLP10, printk("qla4000_get_req_pkt: new request_in = %04x, new request_ptr = %p\n",
				       request_in, ha->request_ptr));
	}

		/* request queue is full, try again later */
	if ((request_in + 1) == ha->request_out)
	{
		QL4PRINT(QLP2|QLP3|QLP5, printk("qla4000_get_req_pkt: request queue is full, try again later\n"));

		/* restore request pointer */
		ha->request_ptr = *queue_entry;
		QL4PRINT(QLP10, printk("qla4000_get_req_pkt: restore request_in = %04x, request_ptr = %p\n",
				       ha->request_in, ha->request_ptr));
		status = QLA_ERROR;
	}
	else
	{
		ha->request_in = request_in;
		memset(*queue_entry, 0, sizeof(**queue_entry));
	}

	LEAVE("qla4000_get_req_pkt");
	return(status);
}

/*
 *  Issue marker command.
 *      Function issues marker IOCB.
 *
 * Input:
 *      ha   = adapter block pointer.
 *      b    = SCSI BUS number
 *      t    = SCSI ID
 *      l    = SCSI LUN
 *      type = marker modifier
 */
STATIC uint8_t
qla4000_send_marker(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, uint32_t lun, uint16_t marker_type)
{
	MARKER_ENTRY *marker_entry;
	ISP_REG_T    *reg = ha->base_address;
	uint8_t      status = QLA_SUCCESS;
	unsigned long flags = 0;

	ENTER("qla4000_send_marker");

	/* Acquire hardware specific lock */
	spin_lock_irqsave(&ha->hardware_lock, flags);

	if (marker_type > MM_TARGET_COLD_RESET)
	{
		QL4PRINT(
			QLP5|QLP2,
			printk("qla4000_send_marker: invalid marker type %x\n",
			       marker_type));

		status = QLA_ERROR;
		goto exit_send_marker;
	}

	/* Get pointer to the queue entry for the marker */
	if (qla4000_get_req_pkt(ha, (QUEUE_ENTRY **) &marker_entry) != QLA_SUCCESS)
	{
		QL4PRINT(
			QLP5|QLP2,
			printk("qla4000_send_marker: request queue full, try again later\n"));

		status = QLA_ERROR;
		goto exit_send_marker;
	}

	/* Put the marker in the request queue */
	memset(marker_entry, 0, sizeof(*marker_entry));
	marker_entry->hdr.entryType  = ET_MARKER;
	marker_entry->hdr.entryCount = 1;
	marker_entry->target         = ddb_entry->fw_ddb_index;
	marker_entry->modifier       = marker_type;
	marker_entry->lun[1]         = lun;   //SAMII compliant lun

	#ifdef DEBUG
	{
		char *marker_type_str[]={
			"TARGET_WARM_RESET",
			"LUN_RESET",
			"TARGET_COLD_RESET"};

		printk("scsi %d:%d:%d: %s Marker sent\n",
		       ha->host_no, ddb_entry->bus,
		       ddb_entry->target, marker_type_str[marker_type]);
	}
	#endif

	/* Tell ISP it's got a new I/O request */
	WRT_REG_DWORD(&reg->requestQueueInPointer, ha->request_in);

	exit_send_marker:
	/* Release hardware specific lock */
	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	LEAVE("qla4000_send_marker");
	return(status);
}

void add_to_active_array(scsi_qla_host_t *ha, uint32_t index, srb_t *srb)
{
	/* put command in active array */
	ha->active_srb_array[index] = srb;
	CMD_HANDLE(srb->cmd) = (unsigned char *) index;

	/* update counters */
	ha->active_srb_count++;
	ha->req_q_count -= srb->entry_count;
	srb->ddb_entry->out_count++;
	srb->lun_entry->out_count++;
	srb->active_array_index = index;
	srb->state = SRB_ACTIVE_STATE;
}

srb_t *del_from_active_array(scsi_qla_host_t *ha, uint32_t index)
{
	srb_t *srb = NULL;

	/* validate handle and remove from active array*/
	if (index < QLA4000_MAX_CMDS_TO_RISC)
	{
		srb = ha->active_srb_array[index];
		ha->active_srb_array[index] = 0;

		if (srb)
		{
			/* update counters */
			ha->req_q_count += srb->entry_count;
			ha->iocb_cnt -= srb->iocb_cnt;
			ha->active_srb_count--;
			srb->ddb_entry->out_count--;
			srb->lun_entry->out_count--;
			srb->active_array_index = INVALID_ENTRY;
			CMD_HANDLE(srb->cmd) = NULL;
			//srb->state = SRB_NO_QUEUE_STATE;
		}
		else
	 		printk(KERN_INFO "%s index=%d already completed.\n",__func__,index);
	}
	else
	 printk(KERN_INFO "%s index=%d exceeded max index of %d\n",__func__,index,QLA4000_MAX_CMDS_TO_RISC);

	return(srb);
}

/*
 * qla4000_send_command_to_isp
 *   This function is called by qla4000_queuecommand to build an ISP
 *	 command and pass it to the ISO for execution.
 *
 * Arguements:
 *   ha  - adapter handle
 *	 srb - pointer to SCSI Request Block to be sent to ISP
 *
 * Return Value:
 *	 QLA_SUCCESS - if command was sent to ISP
 *	 QLA_ERROR   - if no ISP request slot or tag handle available
 */
uint8_t
qla4000_send_command_to_isp(scsi_qla_host_t *ha, srb_t *srb)
{
	uint8_t       status = QLA_SUCCESS;
	Scsi_Cmnd     *cmd = srb->cmd;
	ISP_REG_T     *reg = ha->base_address;
	ddb_entry_t   *ddb_entry = srb->ddb_entry;
	COMMAND_ENTRY *cmd_entry;
	struct scatterlist  *saved_sg, *sg = (struct scatterlist *) NULL;
	uint16_t      saved_request_in;
	QUEUE_ENTRY   *saved_request_ptr;
	uint16_t      seg_cnt;	/* number of data segments (sg entries, if sg request) */
	uint16_t      req_cnt;	/* number of request queue entries (commamd + continue) */
	unsigned long flags;
	uint16_t       i;
	uint32_t      index;
	uint8_t       found = FALSE;
	uint16_t	tot_iocbs;

	ENTER("qla4000_send_command_to_isp");
	/*
	 * Calculate number of entries and segments required
	 *---------------------------------------------------*/
	seg_cnt = 0;
	req_cnt = 1;

	if (cmd->use_sg)
	{
		/*
		 * We must build an SG list in adapter format, as the kernel's SG list
		 * cannot be used directly because of data field size (__alpha__)
		 * differences and the kernel SG list uses virtual addresses where
		 * we need physical addresses.
		 */
		sg = (struct scatterlist *) cmd->request_buffer;

		seg_cnt = pci_map_sg(ha->pdev, sg, cmd->use_sg,
				     scsi_to_pci_dma_dir(cmd->sc_data_direction));

		/*
		 * If the number of sg entries is greater than the number of command
		 * segments available, then we need to allocate continuation entries.
		 * Calculate the number of request queue entries needed.
		 */
		if (seg_cnt > COMMAND_SEG)
		{
			req_cnt += (uint16_t)(seg_cnt - COMMAND_SEG) / CONTINUE_SEG;
			if ((uint16_t)(seg_cnt - COMMAND_SEG) % CONTINUE_SEG)
				req_cnt++;
		}


		QL4PRINT(QLP5, printk("qla4000_send_command_to_isp: S/G Transfer cmd=%p seg_cnt=0x%x, req_cnt=%x\n",
				      cmd, seg_cnt, req_cnt));
	}
	else if (cmd->request_bufflen)	/* If data transfer. */
	{
		QL4PRINT(QLP10, printk("qla4000_send_command_to_isp: No S/G transfer t=%x cmd=%p len=%x\n",
				       cmd->target,cmd,cmd->request_bufflen));
		seg_cnt = 1;
	}
	else
	{
		seg_cnt = 0;
	}


	/*
	 * Get pointer to the next queue entry for the command
	 *-----------------------------------------------------*/

	/* Acquire hardware specific lock */
	spin_lock_irqsave(&ha->hardware_lock, flags);

	/* Save some variables to undo things if an error occurs */
	saved_request_in  = ha->request_in;
	saved_request_ptr = ha->request_ptr;
	saved_sg = sg;

	/* Make sure there are enough request queue entries to handle this request. */
	if (ha->req_q_count < req_cnt)
	{
		QL4PRINT(
			QLP1, printk("qla4000_send_command_to_isp: Not enough request queue entries available (need %d, have %d)\n",
				     req_cnt, ha->req_q_count));

		goto error_exit_send_cmd;
	}

	if ((ha->iocb_cnt + 1) >= ha->iocb_hiwat) 
	{
		/* printk("%s: Not Enough IOCBS for request - "
			"iocb_cnt=%x, total iocbs needed=%x, hiwat=%x.\n",
				__func__,
				ha->iocb_cnt,
				1,
				ha->iocb_hiwat); */
		goto error_exit_send_cmd;
	}
	/* Get request queue entry and adjust ring index. */
	ha->request_out  = (uint16_t) RD_REG_DWORD(&reg->requestQueueOutPointer);

	cmd_entry = (COMMAND_ENTRY *) ha->request_ptr;
	memset(cmd_entry, 0, sizeof(*cmd_entry));
	tot_iocbs = 1;

	if (ha->request_in == (REQUEST_QUEUE_DEPTH - 1))
	{
		ha->request_in = 0;
		ha->request_ptr = ha->req_queue.virt_addr;
	}
	else
	{
		ha->request_in++;
		ha->request_ptr++;
	}

	if ((ha->request_in + 1) == ha->request_out)
	{
		/* request queue is full, try again later */
		ha->request_ptr = (QUEUE_ENTRY *) cmd_entry;
		QL4PRINT(
			QLP2|QLP3|QLP5,
			printk("qla4000_send_command_to_isp: request queue is full, try again later\n"));

		goto error_exit_send_cmd;
	}

	/* Check for room in active srb array */
	index = ha->current_active_index;
	for (i=0; i<QLA4000_MAX_CMDS_TO_RISC; i++)
	{
		index++;
		if (index == QLA4000_MAX_CMDS_TO_RISC)
			index = 1;
		if (ha->active_srb_array[index] == 0)
		{
			found = 1;
			ha->current_active_index = index;
			break;
		}
	}

	if (!found)
	{
		QL4PRINT(
			QLP2|QLP3|QLP5,
			printk("qla4000_send_command_to_isp: no room in active array, try again later\n"));

		goto error_exit_send_cmd;
	}

	/*
	 * Build command entry packet to send to ISP
	 *------------------------------------------*/

	/* If in connection mode, bump sequence number */
	if ((ha->firmware_options & FWOPT_SESSION_MODE) != 0)
		ddb_entry->CmdSn++;

	cmd_entry->hdr.entryType     = ET_COMMAND;
	cmd_entry->hdr.entryCount    = srb->entry_count = (uint8_t) req_cnt;
	cmd_entry->handle            = index;
	cmd_entry->target            = ddb_entry->fw_ddb_index;
	cmd_entry->connection_id     = ddb_entry->connection_id;
	cmd_entry->lun[1]            = LSB(cmd->lun); //SAMII compliant lun
	cmd_entry->lun[2]            = MSB(cmd->lun);
	cmd_entry->cmdSeqNum         = ddb_entry->CmdSn;
	cmd_entry->dataSegCnt        = seg_cnt;
	cmd_entry->ttlByteCnt        = cmd->request_bufflen;
	memcpy(cmd_entry->cdb, cmd->cmnd, cmd->cmd_len);

	/* Set firmware timeout to [target_mgmt_timeout] seconds less than OS timeout.
	 * We want the firmware to time out the command first, as recovery is not
	 * always a pleasant experience when the Linux OS detects the timeout */
#if 0
	cmd_entry->timeout           = CMD_TIMEOUT(cmd)/HZ;
	if (cmd_entry->timeout > ddb_entry->task_mgmt_timeout)
		cmd_entry->timeout  -= (ddb_entry->task_mgmt_timeout);
#else
	/* no timeout */
	cmd_entry->timeout           = 0;
#endif

	QL4PRINT(QLP10,
		 printk("qla4000_send_command_to_isp: timeout set to %d seconds, \n",
			cmd_entry->timeout));
	#ifdef DEBUG
	srb->iocb_timeout = cmd_entry->timeout;
	srb->os_timeout = CMD_TIMEOUT(cmd)/HZ;
	srb->task_mgmt_timeout = ddb_entry->task_mgmt_timeout;
	#endif

	/* Set data transfer direction control flags
	 * NOTE: Look at data_direction bits iff there is data to be transferred,
	 *       as the data direction bit is sometimed filled in when there is
	 *       no data to be transferred */
	if (cmd->request_bufflen)
	{
		switch (cmd->sc_data_direction)
		{
		case SCSI_DATA_READ:  cmd_entry->control_flags = CF_READ;    break;
		case SCSI_DATA_WRITE: cmd_entry->control_flags = CF_WRITE;   break;
		case SCSI_DATA_NONE:  cmd_entry->control_flags = CF_NO_DATA; break;
		case SCSI_DATA_UNKNOWN:
		default:
			QL4PRINT(QLP1|QLP10, printk("qla4000_send_command_to_isp: No data direction specified!\n"));
			cmd_entry->control_flags = CF_NO_DATA;
		}
	}
	else
	{
		cmd_entry->control_flags = CF_NO_DATA;
	}


	/* Set tagged queueing control flags */
	switch (cmd->tag)
	{
	case HEAD_OF_QUEUE_TAG: cmd_entry->control_flags |= CF_HEAD_TAG;    break;
	case ORDERED_QUEUE_TAG: cmd_entry->control_flags |= CF_ORDERED_TAG; break;
	case SIMPLE_QUEUE_TAG:  cmd_entry->control_flags |= CF_SIMPLE_TAG;  break;
	default:
            /*
		 * TEST_UNIT_READY commands from scsi_scan will fail due to "overlapped
		 * commands attempted" unless we setup at least a simple queue (midlayer
		 * will embelish this once it can do an INQUIRY command to the device)
		 */
            cmd_entry->control_flags |= CF_SIMPLE_TAG;  break;
	}

	/* Set sata segments and byte counts */
	if (cmd->use_sg)
	{
		/* Data transfer with Scatter/Gather */
		QL4PRINT(QLP10, printk("qla4000_send_command_to_isp: Building S/G data segments..\n"));
		qla4000_dump_dwords(QLP10, sg, 4*16 );

		/* Load COMMAND entry data segments. */
		for (i = 0; i < COMMAND_SEG && seg_cnt; i++, seg_cnt--)
		{
			cmd_entry->dataseg[i].base.addrLow  = cpu_to_le32(LSL(sg_dma_address(sg)));
			cmd_entry->dataseg[i].base.addrHigh = cpu_to_le32(MSL(sg_dma_address(sg)));
			cmd_entry->dataseg[i].count = cpu_to_le32(sg_dma_len(sg));
			QL4PRINT(QLP10, printk("qla4000_send_command_to_isp: S/G Segment phys_addr=%x %x, len=0x%x\n",
					       cpu_to_le32(MSL(sg_dma_address(sg))),
					       cpu_to_le32(LSL(sg_dma_address(sg))),
					       cpu_to_le32(sg_dma_len(sg))));
			sg++;
		}

		/* Build CONTINUATION packets. */
		QL4PRINT(QLP10, printk("qla4000_send_command_to_isp: S/G Building Continuation...seg_cnt=0x%x remains\n", seg_cnt));

		while (seg_cnt > 0)
		{
			CONTINUE_ENTRY *cont_entry;

			/* Get request queue entry and adjust ring index. */
			cont_entry = (CONTINUE_ENTRY *) ha->request_ptr;
			memset(cont_entry, 0, sizeof(*cont_entry));
			/* dg 11/11 tot_iocbs++; */

			if (ha->request_in == (REQUEST_QUEUE_DEPTH - 1))
			{
				ha->request_in = 0;
				ha->request_ptr = ha->req_queue.virt_addr;
			}
			else
			{
				ha->request_in++;
				ha->request_ptr++;
			}

			if ((ha->request_in + 1) == ha->request_out ||
				(ha->iocb_cnt + tot_iocbs) >= ha->iocb_hiwat) 
			{
				/* request queue is full, try again later */
				ha->request_ptr = (QUEUE_ENTRY *) cont_entry;
				QL4PRINT(
					QLP2|QLP3|QLP5,
					printk("qla4000_send_command_to_isp: Not enough request queue entries available for continuation segs\n"));

				goto error_exit_send_cmd;
			}

			/* Load packet defaults. */
			cont_entry->hdr.entryType = ET_CONTINUE;
			cont_entry->hdr.entryCount = 1;
			cont_entry->hdr.systemDefined = (uint8_t) ha->request_in;

			/* Load continuation entry data segments. */
			for (i = 0; i < CONTINUE_SEG && seg_cnt; i++, seg_cnt--)
			{
				cont_entry->dataseg[i].base.addrLow  = cpu_to_le32(LSL(sg_dma_address(sg)));
				cont_entry->dataseg[i].base.addrHigh = cpu_to_le32(MSL(sg_dma_address(sg)));
				cont_entry->dataseg[i].count = cpu_to_le32(sg_dma_len(sg));

				QL4PRINT(QLP10, printk("S/G Segment Cont. phys_addr=%x %x, len=0x%x\n",
						       cpu_to_le32(MSL(sg_dma_address(sg))),
						       cpu_to_le32(LSL(sg_dma_address(sg))),
						       cpu_to_le32(sg_dma_len(sg))));
				sg++;
			}
			QL4PRINT(QLP10, printk("qla4000_send_command_to_isp: continuation packet data - scsi(b%d,t%d,l%d)\n",
					       cmd->channel, cmd->target, cmd->lun));
			qla4000_dump_dwords(QLP10, cont_entry, sizeof(*cont_entry));
		}
	}
	else if (cmd->request_buffer && cmd->request_bufflen)
	{
		/* Data transfer without Scatter/Gather */
		srb->saved_dma_handle = pci_map_single(ha->pdev,
						       cmd->request_buffer,
						       cmd->request_bufflen,
						       scsi_to_pci_dma_dir(cmd->sc_data_direction));

		if (!srb->saved_dma_handle)
		{
			QL4PRINT(QLP5, printk("qla4000_send_command_to_isp: pci_map_single failed!, try again later\n"));
			goto error_exit_send_cmd;
		}

		cmd_entry->dataseg[0].base.addrLow  = cpu_to_le32(LSL(srb->saved_dma_handle));
		cmd_entry->dataseg[0].base.addrHigh = cpu_to_le32(MSL(srb->saved_dma_handle));
		cmd_entry->dataseg[0].count = cmd->request_bufflen;
		QL4PRINT(QLP10, printk("qla4000_send_command_to_isp: No S/G map_single saved_dma_handle=%x\n",
				       srb->saved_dma_handle));
	}
	else
	{
		/* No data being transferred */
		QL4PRINT(QLP10, printk("qla4000_send_command_to_isp: No data xfer\n"));
	}


	/*
	 * Send command to ISP
	 *-----------------------------------------------*/
	add_to_active_array(ha, index, srb);
	srb->flags |= SRB_DMA_VALID;

	/* Track IOCB used */
	ha->iocb_cnt += tot_iocbs;
	srb->iocb_cnt = tot_iocbs;

	/* printk(" %d\n", index); */
	/* Dump cmd_entry buffer */
	QL4PRINT(QLP10, printk("qla4000_send_command_to_isp: cmd_entry 0x%p\n", cmd_entry));
	qla4000_dump_words(QLP10, cmd_entry, sizeof(*cmd_entry));

	/* Tell ISP that there's a new request */
	QL4PRINT(QLP5, printk("qla4000_send_command_to_isp: RequestQueueIn %x\n", ha->request_in));

	srb->u_start = jiffies; /*Time we send the I/O to firmware*/

	WRT_REG_DWORD(&reg->requestQueueInPointer, ha->request_in);
	goto exit_send_cmd;

	error_exit_send_cmd:
	/* Unmap srb dma buffer */
	pci_unmap_sg(ha->pdev, saved_sg, cmd->use_sg, scsi_to_pci_dma_dir(cmd->sc_data_direction));

	/* restore request queue in pointers */
	ha->request_in = saved_request_in;
	ha->request_ptr = saved_request_ptr;
	status = QLA_ERROR;

	exit_send_cmd:
	/* Release hardware specific lock */
	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	LEAVE("qla4000_send_command_to_isp");
	return(status);
}
/*
 * Process cmds for a specified device, or all devices if ddb_entry is NULL
 */
void qla4000_process_pending_srb_q(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry)
{
	struct list_head *ptr, *next; /* used for traversing pending_srb_q */
	srb_t *srb;
	unsigned long flags;

	QL4PRINT(QLP5, printk("qla4000_process_pending_srb_q:\n"));
	qla4000_set_extended_debug_level();

	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_safe(ptr, next, &ha->pending_srb_q)
	{
		srb = list_entry(ptr, srb_t, list_entry);

		if ((ddb_entry == 0) ||
		    (ddb_entry->bus == srb->cmd->channel && ddb_entry->target == srb->cmd->target))
		{
			lun_entry_t *lun_entry;

			if (!ddb_entry)
				ddb_entry = srb->ddb_entry;

			/*
			 * Only issue commands if lun is ready to accept commands
			 */
			if ((atomic_read(&ddb_entry->state) == DEV_STATE_ONLINE) &&
			    (lun_entry = qla4000_lookup_lun_handle(ha,ddb_entry , srb->cmd->lun)) &&
			    (lun_entry->lun_state == LS_LUN_READY))
			{
				/* Send the command to the ISP.  If the command cannot be sent,
				 * put the srb back at the head of the pending_srb_q.
				 * Also, if the command can't be sent, don't try to
				 * send more commands, because the request queue is probably full.
				 */
				__del_from_pending_srb_q(ha, srb);

				if (qla4000_send_command_to_isp(ha, srb) != QLA_SUCCESS)
				{
					__add_to_pending_srb_q_head(ha, srb);
					break;
				}
			}
		}
	} /* list loop */
	spin_unlock_irqrestore(&ha->list_lock, flags);

	qla4000_rst_extended_debug_level();
}

#if 0
/*
 * Process cmds for a specified device, or all devices if ddb_entry is NULL
 */
void qla4000_process_retry_srb_q(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry)
{
	struct list_head *ptr, *next; /* used for traversing pending_srb_q */
	srb_t *srb;
	unsigned long flags;

	QL4PRINT(QLP5, printk("qla4000_process_retry_srb_q:\n"));
	qla4000_set_extended_debug_level();

	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_safe(ptr, next, &ha->retry_srb_q)
	{
		srb = list_entry(ptr, srb_t, list_entry);

		if ((ddb_entry == 0) ||
		    (ddb_entry->bus == srb->cmd->channel && ddb_entry->target == srb->cmd->target))
		{
			lun_entry_t *lun_entry;

			if (!ddb_entry)
				ddb_entry = srb->ddb_entry;

			/*
			 * Only issue commands if lun is ready to accept commands
			 */
			if ((atomic_read(&ddb_entry->state) == DEV_STATE_ONLINE) &&
			    (lun_entry = qla4000_lookup_lun_handle(ha,ddb_entry , srb->cmd->lun)) &&
			    (lun_entry->lun_state == LS_LUN_READY))
			{
				/* Send the command to the ISP.  If the command cannot be sent,
				 * put the srb back at the head of the pending_srb_q.
				 * Also, if the command can't be sent, don't try to
				 * send more commands, because the request queue is probably full.
				 */
				__del_from_retry_srb_q(ha, srb);

				if (qla4000_send_command_to_isp(ha, srb) != QLA_SUCCESS)
				{
					__add_to_retry_srb_q(ha, srb);
					break;
				}
			}
		}
	} /* list loop */
	spin_unlock_irqrestore(&ha->list_lock, flags);

	qla4000_rst_extended_debug_level();
}
#endif

inline lun_entry_t *qla4000_lookup_lun_handle(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, uint16_t lun)
{
	lun_entry_t *lun_entry = NULL;

	if (lun < MAX_LUNS)
	{
		lun_entry = ddb_entry->lun_table[lun];
	}

	QL4PRINT(QLP5, printk("qla4000_lookup_lun_handle: l%d, lun_entry = %p\n",
			      lun, lun_entry));
	return(lun_entry);
}
/*
 * qla4000_lookup_ddb_by_SCSIID
 *   Return handle of internal ddb_entry.  If device doesn't exist, returns NULL;
 */
inline ddb_entry_t *qla4000_lookup_ddb_by_SCSIID(scsi_qla_host_t *ha, uint32_t bus, uint32_t target)
{
	ddb_entry_t *ddb_entry = NULL;


	if ((target < MAX_DDB_ENTRIES) &&
	    (ha->target_map[target] != (ddb_entry_t *) INVALID_ENTRY))
	{
		ddb_entry = ha->target_map[target];
	}

	QL4PRINT(QLP5, printk("qla4000_lookup_ddb_by_SCSIID: b%d:t%d, ddb_entry = %p\n",
			      bus, target, ddb_entry));
	return(ddb_entry);
}

/*
 * qla4000_lookup_ddb_by_fw_index
 *   Return handle of internal ddb_entry.  If device doesn't exist, returns NULL;
 */
inline ddb_entry_t *qla4000_lookup_ddb_by_fw_index(scsi_qla_host_t *ha, uint32_t fw_ddb_index)
{
	ddb_entry_t *ddb_entry = NULL;


	if ((fw_ddb_index < MAX_DDB_ENTRIES) &&
	    (ha->fw_ddb_index_map[fw_ddb_index] != (ddb_entry_t *) INVALID_ENTRY))
	{
		ddb_entry = ha->fw_ddb_index_map[fw_ddb_index];
	}

	QL4PRINT(QLP5, printk("qla4000_lookup_ddb_by_fw_index: [%x], ddb_entry = %p\n", fw_ddb_index, ddb_entry));
	return(ddb_entry);
}

/* qla4000_complete_request
 *   This routine returns a command to the caller via the done_fn specified
 *   in the cmd structure.
 *
 * NOTE: The srb pointer should be guaranteed to be nonzero before calling
 *       this function.
 *	 The caller should ensure that the list_lock is released before
 *       calling this function.
 */
uint8_t qla4000_complete_request(scsi_qla_host_t *ha, srb_t *srb)
{
	uint8_t status = QLA_ERROR;
	Scsi_Cmnd *cmd;
	unsigned long flags;

	ENTER("qla4000_complete_request");

	/* Make sure the cmd pointer is valid */
	if (srb == NULL)
	{
		QL4PRINT(QLP2, printk("qla4000_complete_request: ERROR: NULL srb \n"));
		BUG();
		goto exit_complete_request;
	}

	if ((srb->flags & SRB_FREE_STATE) == 0)
		qla4000_delete_timer_from_cmd(srb);

	cmd = srb->cmd;
	if (cmd == NULL)
	{
		QL4PRINT(QLP2, printk("qla4000_complete_request: ERROR: NULL cmd pointer in srb=%p\n",
				      srb));
		goto exit_complete_request;
	}

	/* Release memory used for this I/O */
	if ((srb->flags & SRB_DMA_VALID) != 0)
	{
		srb->flags &= ~SRB_DMA_VALID;

		/* Release memory used for this I/O */
		if (cmd->use_sg)
		{
			QL4PRINT(QLP10, printk("qla4000_complete_request: S/G unmap_sg cmd=%p\n",cmd));
			pci_unmap_sg(ha->pdev, cmd->request_buffer,
				     cmd->use_sg,
				     scsi_to_pci_dma_dir(cmd->sc_data_direction));
		}
		else if (cmd->request_bufflen)
		{
			QL4PRINT(QLP10, printk("qla4000_complete_request: No S/G unmap_single cmd=%p saved_dma_handle=%x\n",
					       cmd, (uint32_t) srb->saved_dma_handle));

			pci_unmap_single(ha->pdev,srb->saved_dma_handle,
					 cmd->request_bufflen,
					 scsi_to_pci_dma_dir(cmd->sc_data_direction));
		}
	}

	if (host_byte(CMD_RESULT(cmd)) == DID_OK)
	{
		if (!(srb->flags & SRB_GOT_SENSE))
		{
			lun_entry_t *lun = srb->lun_entry;
			/*
			 * If lun was not ready (suspended or timeout) then change state to "READY".
			 */
			spin_lock_irqsave(&lun->lun_lock, flags);
			if ( /* (lun->retry_count < lun->max_retry_count) && */
			    (lun->lun_state != LS_LUN_READY))
			{
				del_from_suspended_lun_q(ha, lun);
				lun->lun_state = LS_LUN_READY;
				/*printk("%s: lun %d set to READY state \n",__FUNCTION__,lun->lun); */
			}
			spin_unlock_irqrestore(&lun->lun_lock, flags);
		}
	}

	#ifdef DEBUG
	/* debug prints */
	if (host_byte(CMD_RESULT(cmd)) == DID_OK)
	{
		switch (cmd->cmnd[0])
		{
		case TEST_UNIT_READY:
			QL4PRINT(QLP13, printk("qla4000_complete_request: %d:%d:%d:%d: TEST_UNIT_READY status = 0x%x\n",
					       ha->host_no, cmd->channel, cmd->target, cmd->lun, cmd->result & 0xff));

			if (driver_byte(cmd->result) & DRIVER_SENSE)
			{
				QL4PRINT(QLP13, printk("REQUEST_SENSE data:  (MAX 0x20 bytes displayed)\n"));
				qla4000_dump_bytes(QLP13, cmd->sense_buffer, MIN(0x20, CMD_SNSLEN(cmd)));
			}
			break;
		case INQUIRY:
			QL4PRINT(QLP13, printk("qla4000_complete_request: %d:%d:%d:%d: INQUIRY data: (MAX 0x30 bytes displayed)\n",
					       ha->host_no, cmd->channel, cmd->target, cmd->lun));
			qla4000_dump_bytes(QLP13, cmd->request_buffer, MIN(0x30, cmd->request_bufflen));
			if (strncmp(cmd->request_buffer, "\7f\00\00\00\7f\00\00\00", 8) == 0)
			{
				QL4PRINT(QLP1|QLP2|QLP13, printk("scsi %d:%d:%d:%d: Device not present.  Possible connection problem with iSCSI router\n",
								 ha->host_no, cmd->channel, cmd->target, cmd->lun));
			}
			break;
		case REQUEST_SENSE:
			QL4PRINT(QLP13, printk("qla4000_complete_request: %d:%d:%d:%d: REQUEST_SENSE data:  (MAX 0x20 bytes displayed)\n",
					       ha->host_no, cmd->channel, cmd->target, cmd->lun));
			qla4000_dump_bytes(QLP13, cmd->request_buffer, MIN(0x20, cmd->request_bufflen));
			break;
		case REPORT_LUNS:
			QL4PRINT(QLP13, printk("qla4000_complete_request: %d:%d:%d:%d: REPORT_LUNS data: (MAX 0x40 bytes displayed)\n",
					       ha->host_no, cmd->channel, cmd->target, cmd->lun));
			qla4000_dump_bytes(QLP13, cmd->request_buffer, MIN(0x40, cmd->request_bufflen));
			break;
		}

	}
	#endif

	/* 
	 * WORKAROUND
	 * A backdoor device-reset (vis eh_resets) requires different 
	 * error handling.  This code differentiates between normal 
	 * error handling and the backdoor method
	 */
	if (host_byte(CMD_RESULT(cmd)) == DID_RESET)
	{
		#define EH_ACTIVE 1
		if (ha->host->eh_active != EH_ACTIVE)
			CMD_RESULT(srb->cmd) = DID_BUS_BUSY << 16;
	}

	#ifdef DEBUG
	if (CMD_RESULT(cmd) & 0xff)
	{
		QL4PRINT(QLP13, printk("REQUEST_SENSE data:  (MAX 0x20 bytes displayed)\n"));
		qla4000_dump_bytes(QLP13, cmd->sense_buffer, MIN(0x20, CMD_SNSLEN(cmd)));
	}

	if (host_byte(CMD_RESULT(cmd)) == DID_RESET ||
	    host_byte(CMD_RESULT(cmd)) == DID_BUS_BUSY ||
	    host_byte(CMD_RESULT(cmd)) == DID_ABORT ||
	    host_byte(CMD_RESULT(cmd)) == DID_ERROR )
	printk("%s: %d:%d:%d cmd=%p sp=%p didstat=%X comp=%x scsistat=(%x), host_failed=%d\n",
	      __FUNCTION__,ha->host_no,cmd->target,cmd->lun,
			cmd,srb,host_byte(CMD_RESULT(cmd)),srb->cc_stat, 
			CMD_RESULT(cmd) & 0xff,
			cmd->host->host_failed);
	#endif

	/* Call the mid-level driver interrupt handler */
	srb->cmd = NULL;
	add_to_free_srb_q(ha, srb);

	QL4PRINT(QLP5, printk("qla4000_complete_request: request completed w/ result hoststat=%X scsistat=(%x)\n",
			      host_byte(CMD_RESULT(cmd)), CMD_RESULT(cmd) & 0xff));

	spin_lock_irqsave(&io_request_lock, flags);
	CMD_SP(cmd) = NULL;
	(*(cmd)->scsi_done)(cmd);
	spin_unlock_irqrestore(&io_request_lock, flags);

	exit_complete_request:
	LEAVE("qla4000_complete_request");
	return(status);
}

/**************************************************************************
 *   qla4000_queuecommand
 *     Queue a command to the controller.
 *
 * Note:
 * The mid-level driver tries to ensure that queuecommand never gets invoked
 * concurrently with itself or the interrupt handler (although the
 * interrupt handler may call this routine as part of request-completion
 * handling).   Unfortunely, it sometimes calls the scheduler in interrupt
 * context which is a big NO! NO!.
 **************************************************************************/
int
qla4000_queuecommand(Scsi_Cmnd *cmd, void (*done_fn)(Scsi_Cmnd *))
{
	scsi_qla_host_t *ha;
	ddb_entry_t     *ddb_entry;
	lun_entry_t     *lun_entry;
	uint32_t        b, t, l;
	int             return_status = 0;
	srb_t           *srb;

	b = cmd->channel;
	t = cmd->target;
	l = cmd->lun;
	ha = (scsi_qla_host_t *) cmd->host->hostdata;
	spin_unlock(&io_request_lock);

	/*
	 * Retrieve srb from pool.  If no srb available,
	 * Notify the OS to queue commands in the OS.
	 * The OS will not attempt to queue more commands
	 * until a command is returned to the OS.
	 */
	srb = del_from_free_srb_q_head(ha);
	if (srb == NULL)
	{
		QL4PRINT(
			QLP1,
			printk("qla4000_queuecommand: srb not available, retry later\n"));
		return_status = 1;
		goto exit_qc;
	}

	/* Link the srb with cmd */
	CMD_SP(cmd) = (char *)srb;
	cmd->scsi_done = done_fn;
	srb->cmd = cmd;
	srb->r_start = jiffies; /*Time we recieved the I/O*/



	if ((CMD_TIMEOUT(cmd)/HZ) > QLA_CMD_TIMER_DELTA)
		qla4000_add_timer_to_cmd(srb,
					 (CMD_TIMEOUT(cmd)/HZ) - QLA_CMD_TIMER_DELTA);
	else
		qla4000_add_timer_to_cmd(srb, (CMD_TIMEOUT(cmd)/HZ));

	/* Extend the command timeout value because the iscsi enviornment 
	 * command needs more time to complete.  Otherwise, commands start 
	 * timing out frequently causing device resets.
	 */
	qla4xxx_extend_timeout(srb->cmd, (ql4xcmdtimeout-1)*EXTEND_CMD_TIMEOUT);

	/* retrieve device and lun handles */
	ddb_entry = qla4000_lookup_ddb_by_SCSIID(ha, b, t);
	if (ddb_entry == 0)
	{
		QL4PRINT(QLP1, printk("qla4000_queuecommand: invalid target b%d, t%d, l%d\n",
				      b, t, l));
		CMD_RESULT(cmd) = (int) (DID_NO_CONNECT << 16);
		goto exit_qc_call_done;
	}

	lun_entry = qla4000_lookup_lun_handle(ha, ddb_entry, l);

#if IGNORE_LUN_18
	if (l == 18)
	{
		CMD_RESULT(cmd) = (int) (DID_NO_CONNECT << 16);
		goto exit_qc_call_done;
	}
#endif
	if (lun_entry == 0)
	{
		QL4PRINT(QLP1, printk("qla4000_queuecommand: invalid lun b%d, t%d, l%d\n",
				      b, t, l));
		CMD_RESULT(cmd) = (int) (DID_NO_CONNECT << 16);
		goto exit_qc_call_done;
	}

	/* Prepare the srb to be sent to the ISP */
	srb->ddb_entry = ddb_entry;
	srb->lun_entry = lun_entry;

	if (cmd->allowed < ql4xcmdretrycount)
	{
		cmd->allowed = ql4xcmdretrycount;
	}

	/* Print debug information */
	QL4PRINT(QLP5, printk("qla4000_queuecommand: b%d, t%d ,l%d, ", b, t, l));
	QL4PRINT(QLP5, printk("cdb = "));
	QL4PRINT(QLP5, printk("%02X %02X %02X %02X %02X %02X %02X %02X %02X %02X\n",
			      cmd->cmnd[0], cmd->cmnd[1], cmd->cmnd[2], cmd->cmnd[3],
			      cmd->cmnd[4], cmd->cmnd[5], cmd->cmnd[6], cmd->cmnd[7],
			      cmd->cmnd[8], cmd->cmnd[9]));

	QL4PRINT(QLP5, printk("qla4000_queuecommand: device state = 0x%x\n", atomic_read(&ddb_entry->state)));
	add_to_pending_srb_q(ha, srb);
	qla4xxx_start_io(ha);

	exit_qc:
	spin_lock_irq(&io_request_lock);
	return(return_status);

	exit_qc_call_done:
	qla4000_complete_request(ha, srb);
	spin_lock_irq(&io_request_lock);
	return(return_status);
}

/*
 * qla4xxx_extend_timeout
 *      This routine will extend the timeout to the specified value.
 *
 * Input:
 *      cmd = SCSI command structure
 *
 * Returns:
 *      None.
 */
static void
qla4xxx_extend_timeout(Scsi_Cmnd *cmd, int timeout)
{
	srb_t *srb = (srb_t *) CMD_SP(cmd);
	u_long our_jiffies = (timeout * HZ) + jiffies;

	if (cmd->eh_timeout.function)
	{
		mod_timer(&cmd->eh_timeout,our_jiffies);
	}
	if (srb->timer.function != NULL)
	{
		/*
		 * Our internal timer should timeout before the midlayer has a
		 * chance begin the abort process
		 */
		mod_timer(&srb->timer,our_jiffies - (QLA_CMD_TIMER_DELTA * HZ));
	}
}

/**************************************************************************
*   qla4xxx_start_io
*
* Description:
*	Retrieve and process next request from the pending queue.
*
* Input:
*     ha - pointer to HBA structure
*
* Returns:
* None.
**************************************************************************/
void
qla4xxx_start_io(scsi_qla_host_t *ha)
{
	ddb_entry_t     *ddb_entry;
	lun_entry_t     *lun_entry;
	srb_t           *sp;
	unsigned long   flags;

	spin_lock_irqsave(&ha->list_lock, flags);
	while (!list_empty(&ha->pending_srb_q))
	{
		sp = list_entry(ha->pending_srb_q.next, srb_t, list_entry);

		ddb_entry = sp->ddb_entry;

		if (ddb_entry == NULL)
			BUG();

		__del_from_pending_srb_q(ha, sp);

		/* If device is dead then send request back to OS */
		if (atomic_read(&ddb_entry->state) == DEV_STATE_DEAD)
		{
			CMD_RESULT(sp->cmd) = DID_NO_CONNECT << 16;
			add_to_done_srb_q(ha, sp);
			continue;
		}

		/*
		 */
		if (atomic_read(&ddb_entry->state) == DEV_STATE_MISSING ||
		    atomic_read(&ha->adapter_state) == AS_RESET)
		{

			qla4xxx_extend_timeout(sp->cmd, EXTEND_CMD_TIMEOUT);
			__add_to_retry_srb_q(ha, sp);
			continue;
		}

		lun_entry = sp->lun_entry;
		if (lun_entry == NULL)
			BUG();

		/*
		 * if this request's lun is suspended then put the request on
		 * the  scsi_retry queue.
		 */
		if (lun_entry->lun_state == LS_LUN_SUSPENDED)
		{
			__add_to_retry_srb_q(ha, sp);
			continue;
		}


		/* Release target queue lock */
		spin_unlock_irqrestore(&ha->list_lock, flags);
		if (qla4000_send_command_to_isp(ha, sp) != QLA_SUCCESS)
		{
			/* Unable to send command to the ISP at this time.
			 * Notify the OS to queue commands in the OS.
			 * The OS will not attempt to queue more commands
			 * until a command is returned to the OS.
			 */
			QL4PRINT(
				QLP5|QLP3,
				printk("%s: unable to send cmd to ISP, retry later\n",__FUNCTION__));
			spin_lock_irqsave(&ha->list_lock, flags);
			__add_to_pending_srb_q_head(ha, sp);
			break;
		}
		spin_lock_irqsave(&ha->list_lock, flags);

	}
	spin_unlock_irqrestore(&ha->list_lock, flags);

	/* any commands in done queue */
	if (!list_empty(&ha->done_srb_q))
	{
		while ((sp = del_from_done_srb_q_head(ha)) != NULL)
			qla4000_complete_request(ha, sp);
	}
}

/**************************************************************************
*   qla2x00_cmd_timeout
*
* Description:
*       Handles the command if it times out in any state.
*
* Input:
*     srb - pointer to validate
*
* Returns:
* None.
**************************************************************************/
void
qla4000_cmd_timeout(srb_t *srb)
{
	Scsi_Cmnd *cmd;
	scsi_qla_host_t *ha;
	ddb_entry_t *ddb_entry;
	lun_entry_t *lun_entry;
	unsigned long flags , cpu_flags;

	ENTER("qla4000_cmd_timeout");

	if (srb == NULL)
	{
		QL4PRINT(QLP2, printk("qla4000_cmd_timeout: NULL srb\n"));
		goto exit_cmd_timeout;
	}

	cmd = srb->cmd;
	if (cmd == NULL)
	{
		QL4PRINT(QLP2, printk("qla4000_cmd_timeout: NULL cmd in srb %p\n", srb));
		goto exit_cmd_timeout;
	}

	ddb_entry = srb->ddb_entry;
	if (ddb_entry == NULL)
	{
		QL4PRINT(QLP2, printk("qla4000_cmd_timeout: NULL ddb_entry in srb %p\n", srb));
		goto exit_cmd_timeout;
	}

	lun_entry = srb->lun_entry;
	if (lun_entry == NULL)
	{
		QL4PRINT(QLP2, printk("qla4000_cmd_timeout: NULL lun_entry in srb %p\n", srb));
		goto exit_cmd_timeout;
	}

	ha = (scsi_qla_host_t *) cmd->host->hostdata;
	spin_lock_irqsave(&ha->list_lock, flags);

	QL4PRINT(QLP1,
		 printk("qla4000_cmd_timeout: %d:%d:%d:%d: srb=%p srb state=%d, DS=%d, LS=%d\n",
			ha->host_no, ddb_entry->bus, ddb_entry->target, lun_entry->lun,
			srb, srb->state,
			atomic_read(&ddb_entry->state),
			lun_entry->lun_state));

	/*
	 * If IO is found in retry Queue
	 * Return this IO back to host
	 */

	switch (srb->state)
	{
	case SRB_PENDING_STATE:
		__del_from_pending_srb_q(ha, srb);
		break;

	case SRB_ACTIVE_STATE:
		ha->io_timeout_count++;

		spin_unlock_irqrestore(&ha->list_lock, flags);
		spin_lock_irqsave(&ha->hardware_lock, cpu_flags);

		/* The command could have completed between the time
		 * we checked the state above and now, so check to see if
		 * the command already completed.  To do this, check to see if
		 * the cmd pointer in the srb is NULL.
		 */
		if (srb->cmd)
		{
                    if (srb == ha->active_srb_array[(uint32_t) CMD_HANDLE(srb->cmd)])
			{
				srb->state = SRB_ACTIVE_TIMEOUT_STATE;
			}
			else
			{
				printk(/* KERN_INFO */
					   "qla4000_cmd_timeout: State indicates srb %p "
					   "cmd %p is with "
					   "ISP, But not in active array\n",srb, cmd);
			}
		}

		spin_unlock_irqrestore(&ha->hardware_lock, cpu_flags);
		goto exit_cmd_timeout;

	case SRB_RETRY_STATE:
		__del_from_retry_srb_q(ha, srb);
		break;


	case SRB_DONE_STATE:
		/* Do nothing, it is on its way back to the OS */
		spin_unlock_irqrestore(&ha->list_lock, flags);
		goto exit_cmd_timeout;

	default:
		/* EMPTY */
		QL4PRINT(QLP2|QLP3|QLP5,
			 printk("qla4000_cmd_timeout: ha %d, LOST command state = "
				"0x%x, srb=%p\n", ha->host_no, srb->state,srb));

		printk(KERN_INFO
		       "cmd_timeout: LOST command state = 0x%x\n", srb->state);
		break;
	}

	/*
	 * If state is marked as dead return the cmd with
	 * DID_NO_CONNECT status.  Otherwise set the host_byte to
	 * DID_BUS_BUSY to let the OS retry this cmd.
	 */
	if (atomic_read(&ddb_entry->state) == DEV_STATE_DEAD)
		cmd->result = DID_NO_CONNECT << 16;
	else
		cmd->result = DID_BUS_BUSY << 16;

	spin_unlock_irqrestore(&ha->list_lock, flags);
	qla4000_complete_request(ha, srb);
	exit_cmd_timeout:
	LEAVE("qla4000_cmd_timeout");
}

/**************************************************************************
*   qla4000_add_timer_to_cmd
*
* Description:
*       Creates a timer for the specified command. The timeout is usually
*       the command time from kernel minus 2 secs.
*
* Input:
*     srb - pointer to validate
*
* Returns:
*     None.
**************************************************************************/
void
qla4000_add_timer_to_cmd(srb_t *srb, int timeout)
{
	init_timer(&srb->timer);
	srb->timer.expires = jiffies + timeout * HZ;
	srb->timer.data = (unsigned long) srb;
	srb->timer.function = (void (*) (unsigned long))qla4000_cmd_timeout;
	#if DEBUG
	srb->driver_timeout = timeout;
	#endif
	add_timer(&srb->timer);
	//QL4PRINT(QLP3, printk("qla4000_add_timer_to_cmd: srb %p, timeout %d\n", srb, timeout));
}

/**************************************************************************
*   qla4000_delete_timer_from_cmd
*
* Description:
*       Delete the timer for the specified command.
*
* Input:
*     srb - pointer to validate
*
* Returns:
*     None.
**************************************************************************/
void
qla4000_delete_timer_from_cmd(srb_t *srb )
{
	if (srb->timer.function != NULL)
	{
		del_timer(&srb->timer);
		srb->timer.function =  NULL;
		srb->timer.data = (unsigned long) NULL;
		//QL4PRINT(QLP3, printk("qla4000_delete_timer_from_cmd: srb %p\n", srb));
	}
}


/****************************************************************************/
/*                        Interrupt Service Routine.                        */
/****************************************************************************/

STATIC void
qla4000_suspend_lun(scsi_qla_host_t *ha, srb_t *srb, uint16_t time, uint16_t retries)
{
	lun_entry_t *lun;
	unsigned long flags;
	uint8_t status = 0 ;
	struct list_head *list, *temp; /* used for traversing lists */
	srb_t *sp;

	ENTER("qla4000_suspend_lun");

	/* if the lun is already suspended then don't do it again */
	lun = srb->lun_entry;
	spin_lock_irqsave(&lun->lun_lock, flags);
	if (lun->lun_state == LS_LUN_READY ||
	    lun->lun_state == LS_LUN_RETRY)
	{
		if (lun->lun_state == LS_LUN_READY)
		{
			lun->max_retry_count = retries;
			lun->retry_count = 0;
		}

			/* put lun on suspend queue */
			add_to_suspended_lun_q(ha, lun);
		
		/* Set the suspend time */
		atomic_set(&lun->suspend_timer, time);
		QL4PRINT(QLP2, printk("%s lun %d retry count = %d\n",
				      __FUNCTION__, lun->lun, lun->retry_count));

		/* now suspend the lun */
		lun->lun_state = LS_LUN_SUSPENDED;
		status = 1;

	}
	spin_unlock_irqrestore(&lun->lun_lock, flags);

	/*
	 * Remove all pending commands from request queue
	 * and  put them in the retry queue.
	 */
	if (status)
	{
		spin_lock_irqsave(&ha->list_lock, flags);
		list_for_each_safe(list, temp, &ha->pending_srb_q)
		{
			sp = list_entry(list, srb_t, list_entry);
			if (sp->lun_entry != lun)
				continue;

			__del_from_pending_srb_q(ha, sp);

			if (retries > sp->cmd->allowed)
				sp->cmd->allowed = retries;
			__add_to_retry_srb_q(ha,sp);

		} /* list_for_each_safe */
		spin_unlock_irqrestore(&ha->list_lock, flags);
	}

	add_to_retry_srb_q(ha,srb);

	LEAVE("qla4000_suspend_lun");
}

/*
 *  qla4000_status_entry
 *      Processes received ISP status entry.
 *
 * Input:
 *      ha           = adapter block pointer.
 *      sts_entry    = status entry pointer.
 */
STATIC void
qla4000_status_entry(scsi_qla_host_t *ha, STATUS_ENTRY *sts_entry)
{
	srb_t *srb;
	uint8_t  *strp;

	ENTER("qla4000_status_entry");

	srb = del_from_active_array(ha, sts_entry->handle);
	if (srb)
	{
		Scsi_Cmnd *cmd = srb->cmd;
		scsi_qla_host_t *ha = (scsi_qla_host_t *) cmd->host->hostdata;
		#if 0
		ddb_entry_t *ddb_entry;
		#endif
		if (CMD_SP(cmd) == NULL)
			return;
		/*
		 * Translate ISP error to a Linux SCSI error
		 */
		switch (sts_entry->completionStatus)
		{
		case SCS_COMPLETE:
			if ((sts_entry->scsiStatus & SCSI_STATUS_MASK) == SCSISTAT_BUSY)
			{
				CMD_RESULT(cmd)= DID_BUS_BUSY << 16;
			}
			else
			{
				CMD_RESULT(cmd) = DID_OK << 16;

				/*
				 * Special case considertaion On an Inquiry
				 * command (0x12) for Lun 0, device responds
				 * with no devices (0x7F), then Linux will not
				 * scan further Luns. While reporting that some
				 * device exists on Lun 0 Linux will scan all
				 * devices on this target.
				 */
				if (cmd->cmnd[0] == 0x12 && cmd->lun == 0)
				{
					strp = (uint8_t *)cmd->request_buffer;
					if (*strp == 0x7f)
					{
						/* Make lun unassigned and processor type */
						*strp = 0x00;
					}
				}
			}

			/* NOTE: check conditions are handled below */
			break;
		case SCS_INCOMPLETE:
			QL4PRINT(QLP5|QLP1, printk("qla4000_status_entry: %d:%d:%d:%d: Incomplete, state=0x%02x\n",
						   ha->host_no, cmd->channel, cmd->target, cmd->lun, sts_entry->state_flags));
			if (!(sts_entry->state_flags & STATE_FLAG_SENT_COMMAND))
				CMD_RESULT(cmd) = DID_ERROR << 16;
			else if (!(sts_entry->state_flags & STATE_FLAG_TRANSFERRED_DATA))
				CMD_RESULT(cmd) = DID_ERROR << 16;
			else if (!(sts_entry->state_flags & STATE_FLAG_GOT_STATUS))
				CMD_RESULT(cmd) = DID_ERROR << 16;
			break;
		case SCS_RESET_OCCURRED:
			QL4PRINT(QLP1, printk("qla4000_status_entry: %d:%d:%d:%d: Device RESET occurred\n",
					      ha->host_no, cmd->channel, cmd->target, cmd->lun));
			CMD_RESULT(cmd) = DID_RESET << 16;
			break;
		case SCS_ABORTED:
			QL4PRINT(QLP1, printk("qla4000_status_entry: %d:%d:%d:%d: Abort occurred\n",
					      ha->host_no, cmd->channel, cmd->target, cmd->lun));
			CMD_RESULT(cmd) = DID_ABORT << 16;
			ha->aborted_io_count++;
			break;
		case SCS_TIMEOUT:
			printk("qla4000_status_entry: %d:%d:%d:%d: Timeout occurred - resetting HBA\n",
					      ha->host_no, cmd->channel, cmd->target, cmd->lun);
			/* F/W logout the connection when this occurs */
			CMD_RESULT(cmd) = DID_BUS_BUSY << 16;

			break;
		case SCS_DATA_OVERRUN:
			QL4PRINT(QLP1, printk("qla4000_status_entry: %d:%d:%d:%d: Data overrun, residual = 0x%x\n",
					      ha->host_no, cmd->channel, cmd->target, cmd->lun, sts_entry->residualByteCnt));
			QL4PRINT(QLP10, printk("qla4000_status_entry: response packet data\n"));
			qla4000_dump_bytes(QLP10, sts_entry, sizeof(*sts_entry) * sts_entry->hdr.entryCount);

			CMD_RESULT(cmd) = DID_ERROR << 16;
			break;
		case SCS_DATA_UNDERRUN:
			if ((CMD_XFRLEN(cmd) - sts_entry->residualByteCnt) < cmd->underflow)
			{
				QL4PRINT(QLP1,
					printk("qla4000_status_entry: %d:%d:%d:%d: Data underrun, xferlen = 0x%x, residual = 0x%x\n",
					       ha->host_no, cmd->channel, cmd->target, cmd->lun, CMD_XFRLEN(cmd), sts_entry->residualByteCnt));
				CMD_RESULT(cmd) = DID_ERROR << 16;
				CMD_RESID_LEN(cmd) = sts_entry->residualByteCnt;
			}
			else
			{
				CMD_RESULT(cmd) = DID_OK << 16;

				/*
				 * Special case considertaion On an Inquiry
				 * command (0x12) for Lun 0, device responds
				 * with no devices (0x7F), then Linux will not
				 * scan further Luns. While reporting that some
				 * device exists on Lun 0 Linux will scan all
				 * devices on this target.
				 */
				if (((sts_entry->scsiStatus & SCSI_STATUS_MASK) == 0) &&
				    (cmd->cmnd[0] == 0x12 && cmd->lun == 0 ))
				{
					strp = (uint8_t *)cmd->request_buffer;
					if (*strp == 0x7f)
					{
						/* Make lun unassigned and processor type */
						*strp = 0x00;
					}
				}
			}
			break;

		case SCS_DEVICE_LOGGED_OUT:
		case SCS_DEVICE_UNAVAILABLE:
#if 0
			/*
			 * The device has logged out because either a command has timed out
			 * or the device has gone away. Let's wait for the aen to relogin
			 * the device.	
			 */
			ddb_entry = qla4000_lookup_ddb_by_SCSIID(ha, 0, cmd->target);

			qla4000_set_extended_debug_level();

			if (!ddb_entry)
			{
				//BUG();
				QL4PRINT(QLP2, printk("%s: schedule DPC to reset HA\n", __FUNCTION__));	
				set_bit(DPC_RESET_HA, &ha->dpc_flags);
				set_bit(AF_SCHEDULE_DPC, &ha->flags);

				return;
			}

			if (atomic_read(&ddb_entry->state) == DEV_STATE_DEAD)
			{
				QL4PRINT(QLP5|QLP2, printk("qla4000_status_entry: %d:%d:%d: device unavailable - no longer connected.\n",
							   ha->host_no, cmd->channel, cmd->target));
				CMD_RESULT(cmd) = DID_NO_CONNECT << 16;
			}
			else
			{
				QL4PRINT(QLP5|QLP1, printk("qla4000_status_entry: %d:%d:%d: device unavailable - device missing.\n",
							   ha->host_no, cmd->channel, cmd->target));
				if (atomic_read(&ddb_entry->state) == 
				    DEV_STATE_ONLINE)
				{
					atomic_set(&ddb_entry->relogin_retry_count, MAX_RELOGIN_RETRIES);
					atomic_set(&ddb_entry->relogin_timer, 
						   RELOGIN_TIMEOUT);
					printk(/*KERN_INFO*/ "%s: %d:%d:%d: index [0x%x] marked MISSING\n",
						   __FUNCTION__, ha->host_no, ddb_entry->bus, ddb_entry->target, ddb_entry->fw_ddb_index);

				}
				/*
				 * Hold this request in the retry queue. If we may recover
				 * the device before it times out, then we'll resend it.
				 * That way we don't exhaust the mid-lay's retries.
				 * This also hinders I/O failures.
				 */
				atomic_set(&ddb_entry->state, DEV_STATE_MISSING);
				qla4xxx_extend_timeout(cmd, EXTEND_CMD_TIMEOUT);
				add_to_retry_srb_q(ha,srb);
				goto exit_status_entry;
			}
#else
			CMD_RESULT(cmd) = DID_ERROR << 16;
#endif

			break;

		case SCS_QUEUE_FULL:
			/*
			 * SCSI Mid-Layer handles device queue full
			 */				 
			CMD_RESULT(cmd) = DID_OK << 16;
			CMD_RESULT(cmd) |= (sts_entry->scsiStatus & SCSI_STATUS_MASK);
			QL4PRINT(QLP5,
			printk("%s: QUEUE FULL detected: %d:%d:%d compl=%02x, scsi=%02x, state=%02x, iFlags=%02x, iResp=%02x\n",
				       __FUNCTION__,ha->host_no, cmd->target, cmd->lun,
				       sts_entry->completionStatus, sts_entry->scsiStatus,
				       sts_entry->state_flags, sts_entry->iscsiFlags, sts_entry->iscsiResponse));
			break;

		case SCS_DMA_ERROR:
		case SCS_TRANSPORT_ERROR:
		case SCS_DATA_DIRECTION_ERROR:
		case SCS_DEVICE_CONFIG_CHANGED:
		default:
			CMD_RESULT(cmd) = DID_ERROR << 16;
			break;
		}

		/* Check for errors */
		if ( (host_byte(CMD_RESULT(cmd)) != DID_OK) || 
			 sts_entry->scsiStatus )
		{
			QL4PRINT(QLP5,
				printk("qla4000_status_entry: %d:%d:%d:%d: host status (%s), compl=%02x, scsi=%02x, state=%02x, iFlags=%02x, iResp=%02x\n",
				       ha->host_no, cmd->channel, cmd->target, cmd->lun,
				       host_sts_msg[host_byte(CMD_RESULT(cmd))], sts_entry->completionStatus, sts_entry->scsiStatus,
				       sts_entry->state_flags, sts_entry->iscsiFlags, sts_entry->iscsiResponse));

			if ((sts_entry->scsiStatus & SCSI_STATUS_MASK) == SCSISTAT_CHECK_CONDITION)
			{
				CMD_RESULT(cmd) |= (sts_entry->scsiStatus & SCSI_STATUS_MASK);

				/* Save Sense info, if available */
				if (sts_entry->senseDataByteCnt &&
				    sts_entry->scsiSenseData[0] & 0x70 &&
				    sts_entry->scsiSenseData[2] & 0x0f)
				{
					lun_entry_t *lun = srb->lun_entry;

					QL4PRINT(QLP1,
						printk("qla4000_status_entry: sense key = %x, ASC/ASCQ = %02x/%02x\n",
						       sts_entry->scsiSenseData[2] & 0x0f, sts_entry->scsiSenseData[12], sts_entry->scsiSenseData[13]));

					CMD_ACTUAL_SNSLEN(cmd) = sts_entry->senseDataByteCnt;
					memcpy(CMD_SNSP(cmd), sts_entry->scsiSenseData, MIN(CMD_ACTUAL_SNSLEN(cmd), CMD_SNSLEN(cmd)));
					qla4000_dump_bytes(QLP10, CMD_SNSP(cmd), MIN(CMD_ACTUAL_SNSLEN(cmd), CMD_SNSLEN(cmd)));

					srb->flags |= SRB_GOT_SENSE;

					if (SENSE_KEY(sts_entry->scsiSenseData) == RECOVERED_ERROR)
					{
						CMD_RESULT(cmd) = DID_OK << 16;
						cmd->sense_buffer[0] = 0;
					}
					/*
					 * Suspend the lun (except during initialization)
					 */
					else if (test_bit(AF_INIT_DONE, &ha->flags) && 
					    lun->lun_state != LS_LUN_TIMEOUT)
					{
						/*
						 * if target is "in process of being
						 * ready then suspend lun for 6 secs and
						 * retry all the commands.
						 */
						if ((SENSE_KEY(sts_entry->scsiSenseData) == NOT_READY) &&
						    (ASC(sts_entry->scsiSenseData) == 4) &&
						    (ASCQ(sts_entry->scsiSenseData) == 1))
						{
							/* To give the lun more time to become ready,
							 * suspend lun then retry command */
							qla4000_suspend_lun(ha, srb, SUSPEND_SECONDS, SUSPEND_RETRIES);
							return;
						}
						else if ((SENSE_KEY(sts_entry->scsiSenseData) == HARDWARE_ERROR) &&
							 (ASC(sts_entry->scsiSenseData) == 8) &&
							 (ASCQ(sts_entry->scsiSenseData) == 0))
						{
							/* To give the lun more time to become ready,
							 * suspend lun then retry command */
							qla4000_suspend_lun(ha, srb, SUSPEND_SECONDS,(ha->port_down_retry_count /SUSPEND_SECONDS)) ;
							return;
						}
					}
				} /* sense info */
			} /* check condition */
		} /* error occurred */

		/* fill in info for passthru command */
		if (CMD_PASSTHRU_TYPE(cmd) == TRUE)
		{
			CMD_COMPL_STATUS(cmd)   = sts_entry->completionStatus;
			CMD_SCSI_STATUS(cmd)    = sts_entry->scsiStatus;
			CMD_ISCSI_RESPONSE(cmd) = sts_entry->iscsiResponse;
			CMD_STATE_FLAGS(cmd)    = sts_entry->state_flags;
			CMD_HOST_STATUS(cmd)    = host_byte(CMD_RESULT(cmd));
		}

		/* complete the request */
		srb->cc_stat   = sts_entry->completionStatus;
		qla4000_complete_request(ha, srb);
	}
	else
	{
		printk(KERN_WARNING "qla4000: Status Entry invalid handle  - sp=%p.  This cmd may have already been completed.\n", srb);
		QL4PRINT(QLP1, printk("qla4000_status_entry: sts_entry 0x%p\n", sts_entry));
		QL4PRINT(QLP2, printk("%s: schedule DPC to reset HA\n", __FUNCTION__));	
	}

	#if 0
	exit_status_entry:
	#endif
	LEAVE("qla4000_status_entry");
}

/*
 * qla4000_process_response_queue
 *      This routine handles the Response Queue Completion.
 *
 * Input:
 *      ha = adapter block pointer.
 *
 * Output:
 *      none
 *
 * Remarks:
 *      hardware_lock locked upon entry
 *
 * Returns:
 *      QLA_SUCCESS 	if OK
 *	    QLA_FATAL_ERROR if failed
 */
STATIC uint32_t
qla4000_process_response_queue(scsi_qla_host_t *ha)
{
	uint32_t count = 0;
	uint32_t i;
	srb_t    *srb = 0;
	STATUS_ENTRY *sts_entry;
	PERFORMANCE_STATUS_ENTRY *perf_sts_entry;
	ISP_REG_T *reg = ha->base_address;

	ENTER("qla4000_process_response_queue");

	/* Process all responses from response queue */
	while (ha->response_ptr->signature != RESPONSE_PROCESSED)
	{
		/* Make sure we don't get ahead of the adapter's in pointer (causing it to be full) */
		ha->response_in = (uint16_t) RD_REG_DWORD(&reg->responseQueueInPointer);

		if (ha->response_out == ha->response_in)
		{
			QL4PRINT(QLP5, printk("qla4000_process_response_queue: Response count %x out %x in %x, next %p:%x.  Finished!\n",
					      count, ha->response_out, ha->response_in,
					      ha->request_ptr, ha->response_ptr->signature));
			break;
		}

		count++;

		sts_entry = (STATUS_ENTRY *) ha->response_ptr;
		QL4PRINT(QLP10, printk("qla4000_process_response_queue: RespPtr %p, type %x\n",
				       ha->response_ptr, sts_entry->hdr.entryType));

		/* Advance pointers for next entry */
		if (ha->response_out == (RESPONSE_QUEUE_DEPTH - 1))
		{
			ha->response_out = 0;
			ha->response_ptr = ha->rsp_queue.virt_addr;
		}
		else
		{
			ha->response_out++;
			ha->response_ptr++;
		}

		/* process entry */
		switch (sts_entry->hdr.entryType)
		{
		case ET_STATUS:
			/* Common status - Single completion posted in single IOSB */
			qla4000_status_entry(ha, sts_entry);
			break;

		case ET_PERFORMANCE_STATUS:
			/* Performance Status Entry Handling -
			 * Multiple completions posted ina single IOSB.
			 * All completions are good */
			perf_sts_entry= (PERFORMANCE_STATUS_ENTRY *) sts_entry;

			for (i =0; i < perf_sts_entry->entryCount; i++)
			{
				srb = del_from_active_array(ha, perf_sts_entry->handleArray[i]);
				if (srb == NULL)
					goto exit_prq_invalid_handle;

				CMD_RESULT(srb->cmd) =(int)  DID_OK << 16;
				qla4000_complete_request(ha, srb);
			}
			break;

		case ET_STATUS_CONTINUATION:
			/* Just throw away the status continuation entries */
			QL4PRINT(QLP10, printk("qla4000_process_response_queue: Status Continuation entry - ignoring\n"));
			break;

		case ET_COMMAND:
			/* ISP device queue is full. Command not accepted by ISP.
			 * Queue command for later */

			srb = del_from_active_array(ha, sts_entry->handle);
			if (srb == NULL)
				goto exit_prq_invalid_handle;

			QL4PRINT(QLP2, printk("qla4000_process_response_queue: FW device queue full, srb %p, put pending_srb_q\n", srb));
			add_to_pending_srb_q(ha, srb);
			break;

		case ET_CONTINUE:
			/* Just throw away the continuation entries */
			QL4PRINT(QLP2, printk("qla4000_process_response_queue: Continuation entry - ignoring\n"));
			break;

		default:
			/* Invalid entry in response queue, reset RISC firmware */
			QL4PRINT(QLP5|QLP2, printk("qla4000_process_response_queue: Invalid entry %x in response queue \n",
						   sts_entry->hdr.entryType));
			QL4PRINT(QLP10, printk("qla4000_process_response_queue: Dumping Response Entry %p:%x out %x in%x\n",
					       sts_entry, ((QUEUE_ENTRY*)sts_entry)->signature, ha->response_out, ha->response_in));
			qla4000_dump_bytes(QLP10, sts_entry, sizeof(*sts_entry));
			goto exit_prq_error;
		}

		/* Set queue entry processed signature */
		((QUEUE_ENTRY *)sts_entry)->signature = RESPONSE_PROCESSED;
	}

	/* Done with responses, update the ISP */
	WRT_REG_DWORD(&reg->responseQueueOutPointer, ha->response_out);
	LEAVE("qla4000_process_response_queue");
	return(QLA_SUCCESS);

	exit_prq_invalid_handle:
	QL4PRINT(QLP5|QLP2, printk("qla4000_process_response_queue: Invalid handle(srb)=%p type=%x IOCS=%x\n",
				   srb, sts_entry->hdr.entryType, sts_entry->completionStatus));

	exit_prq_error:
	WRT_REG_DWORD(&reg->requestQueueOutPointer, ha->response_out);

	QL4PRINT(QLP5|QLP2, printk("qla4000_process_response_queue: schedule DPC to restart ha %d\n",
				   ha->instance));

	set_bit(DPC_RESET_HA, &ha->dpc_flags);
	set_bit(AF_SCHEDULE_DPC, &ha->flags);

	LEAVE("qla4000_process_response_queue");
	return(QLA_ERROR);
}

/*
 * qla4000_interrupt_service_routine
 *      Service Interrupt.
 *
 * Input:
 *      ha = adapter block pointer.
 *
 * Output:
 *      none
 *
 * Remarks:
 *      hardware_lock locked upon entry
 *
 * Returns:
 *      0 = success, An interrupt was found and processed
 *	    1 = failure, The adapter was not interrupting
 */
STATIC uint8_t
qla4000_interrupt_service_routine(scsi_qla_host_t *ha)
{
	uint8_t   status = QLA_ERROR;
	ISP_REG_T *reg   = ha->base_address;
	uint32_t  intr_status;

	ENTER("qla4000_interrupt_service_routine");
	intr_status = RD_REG_DWORD(&reg->interruptStatusOut);

	QL4PRINT(QLP11, printk("qla4000_interrupt_service_routine: intr_status = %X\n", intr_status));

	if (intr_status == INVALID_REGISTER)
	{
		// *** FIXME ***
		// Funny stuff can happen when you're resetting
		if (atomic_read(&ha->adapter_state) == AS_RESET)
		{
			status = QLA_SUCCESS;
			goto exit_isr;
		}

		QL4PRINT(QLP11, printk("qla4000_interrupt_service_routine: Unrecognized interruptStatusOut %x.\n", intr_status));
		status = QLA_ERROR;
		goto exit_isr;
	}

	ha->seconds_since_last_intr = 0;

	/*
	 * Process fast post interrupt.
	 */
	if (intr_status & ISO_FASTPOST_COMPLETE)
	{
		srb_t *srb;

		srb = del_from_active_array(ha, RD_REG_DWORD(&reg->fastPostHandle));
		if (srb)
		{
			/* complete request */
			qla4000_complete_request(ha, srb);
		}
		else
		{
			/* Invalid FastPostHandle from RISC, reset RISC firmware */
			printk("qla4000(%d): Invalid fast_post handle=0x%x, "
			       "Scheduling Adapter Reset\n",
			       ha->host_no, RD_REG_WORD(&reg->fastPostHandle));
			QL4PRINT(QLP2, printk("%s: schedule DPC to reset HA\n", __FUNCTION__));	
			set_bit(DPC_RESET_HA, &ha->dpc_flags);
			set_bit(AF_SCHEDULE_DPC, &ha->flags);
		}

		/* Clear Fast Post Interrupt */
		WRT_REG_DWORD(&reg->interruptStatusOut, ISO_FASTPOST_COMPLETE);
	}

	/*
	 * Process response queue interrupt.
	 */
	if (intr_status & ISO_RSP_QUEUE_COMPLETE)
	{
		qla4000_process_response_queue(ha);

		/* Clear Response Queue Interrupt */
		WRT_REG_DWORD(&reg->interruptStatusOut, ISO_RSP_QUEUE_COMPLETE);
	}

	/*
	 * Process mailbox/asynch event  interrupt.
	 */
	if (intr_status & ISO_MB0X_CMD_COMPLETE)
	{
		uint32_t mbox_status = RD_REG_DWORD(&reg->mailboxOut0);

		switch (mbox_status)
		{
		case MBOX_STS_BUSY:
		case MBOX_STS_COMMAND_COMPLETE:
		case MBOX_STS_INVALID_COMMAND:
		case MBOX_STS_HOST_INTERFACE_ERROR:
		case MBOX_STS_TEST_FAILED:
		case MBOX_STS_COMMAND_ERROR:
		case MBOX_STS_COMMAND_PARAMETER_ERROR:
		case MBOX_STS_INTERMEDIATE:
			if (test_bit(AF_MBOX_COMMAND, &ha->flags))
			{
				/*
				 * Copy all mailbox registers to temporary location and
				 * set mailbox command done flag
				 */
				uint8_t i;

				for (i=1; i<ha->mbox_status_count; i++)
					ha->mbox_status[i] = RD_REG_DWORD(&reg->mailboxOut[i-1]);
				ha->mbox_status[0] = mbox_status;

				QL4PRINT(QLP11, printk("qla4000_interrupt_service_routine: mailbox cmd done!\n"));
				set_bit(AF_MBOX_COMMAND_DONE, &ha->flags);
				wake_up(&ha->mailbox_wait_queue);
			}
			break;

		case MBOX_ASTS_SYSTEM_ERROR:
		case MBOX_ASTS_REQUEST_TRANSFER_ERROR:
		case MBOX_ASTS_RESPONSE_TRANSFER_ERROR:
		case MBOX_ASTS_PROTOCOL_STATISTIC_ALARM:
		case MBOX_ASTS_SCSI_COMMAND_PDU_REJECTED:
		case MBOX_ASTS_DATABASE_CHANGED:
		case MBOX_ASTS_UNSOLICITED_PDU_RECEIVED:

			/* Save AEN information and process it in the DPC routine */
			spin_lock(&ha->aen_q_spinlock);
			if (ha->aen_q_count != 0)
			{
				int i;

				QL4PRINT(QLP2|QLP5, printk("qla4000_interrupt_service_routine: aen queued!\n"));

				/* advance pointer */
				if (ha->aen_in == (MAX_AEN_ENTRIES - 1))
					ha->aen_in = 0;
				else
					ha->aen_in++;

				/* decrement available counter */
				ha->aen_q_count--;

				for (i=1; i < MBOX_AEN_REG_COUNT; i++)
					ha->aen_q[ha->aen_in].mbox_sts[i] = RD_REG_DWORD(&reg->mailboxOut[i-1]);
				ha->aen_q[ha->aen_in].mbox_sts[0] = mbox_status;
			}
			else
			{
				QL4PRINT(QLP2|QLP5, printk("qla4000_interrupt_service_routine: aen queue overflowed!\n"));
			}
			spin_unlock(&ha->aen_q_spinlock);

			/* The DPC routine will process the aen */
			set_bit(AF_SCHEDULE_DPC, &ha->flags);
			break;

		default:
			QL4PRINT(QLP5|QLP2, printk("qla4000: Unexpected MailBox status %08X\n", mbox_status));
			/*qla4000_dump_registers(QLP10, ha);*/
			QL4PRINT(QLP2, printk("%s: schedule DPC to reset HA\n", __FUNCTION__));	
			set_bit(DPC_RESET_HA, &ha->dpc_flags);
			set_bit(AF_SCHEDULE_DPC, &ha->flags);
			ha->mbox_status[0] = mbox_status;
		}

		/* Clear Mailbox Interrupt by writing MailboxOut0 value back to MailboxOut0 register. */
		WRT_REG_DWORD(&reg->mailboxOut0, mbox_status);

	} /* end mailbox interrupt */

	exit_isr:
	LEAVE("qla4000_interrupt_service_routine");
	return(status);
}

/*
 * Mailbox Command
 *      Issue mailbox command and wait for completion, if outcount > 0.
 *
 * Input:
 *      ha 		= adapter block pointer.
 *      inCount	= number of mailbox registers to load.
 *      outCount    = number of mailbox registers to return.
 *      mbx_cmd 	= data pointer for mailbox in registers.
 *      mbx_sts 	= data pointer for mailbox out registers.
 *
 * Output:
 *      mbx_sts[MAILBOX_REGISTER_COUNT] = returned mailbox out data.
 *
 * Returns:
 *      QLA_SUCCESS = success
 *      QLA_ERROR   = error
 */
uint8_t
qla4000_mailbox_command(
		       scsi_qla_host_t *ha,
		       uint8_t inCount,
		       uint8_t outCount,
		       uint32_t *mbx_cmd,
		       uint32_t *mbx_sts)
{
	ISP_REG_T    *reg   = ha->base_address;
	uint8_t      status = QLA_ERROR;
	uint8_t      i;
	uint32_t     wait_count;  /* in milliseconds */
	uint32_t     intr_status;
	unsigned long flags = 0;
	DECLARE_WAITQUEUE(wait, current);


	ENTER("qla4000_mailbox_command");

	down(&ha->mbox_sem);


	set_bit(AF_MBOX_COMMAND, &ha->flags);


	/* Make sure that pointers are valid */
	if (!mbx_cmd || !mbx_sts)
	{
		QL4PRINT(QLP1|QLP3|QLP5, printk("qla4000_mailbox_command: Invalid mbx_cmd or mbx_sts pointer\n"));
		goto mbox_exit;
	}

	/* To prevent overwriting mailbox registers for a command that has
	 * not yet been serviced, check to see if a previously issued
	 * mailbox command is interrupting.
	 * -----------------------------------------------------------------
	 */
	spin_lock_irqsave(&ha->hardware_lock, flags);
	intr_status = RD_REG_DWORD(&reg->interruptStatusOut);
	if (intr_status & ISO_MB0X_CMD_COMPLETE)
	{
		QL4PRINT(QLP2|QLP3, printk("qla4000_mailbox_command: Trying to execute a mailbox request, while another one is interrupting\n"));
		QL4PRINT(QLP1|QLP3, printk("                         Service existing interrupt first\n"));

		/* Service existing interrupt */
		qla4000_interrupt_service_routine(ha);
	}

	#ifdef DEBUG
	if (RD_REG_DWORD(&reg->interruptStatusIn) & ISI_MBOX_CMD_PENDING)
	{
		/* This situation is actually OK, we just want to know when it's happening */
		QL4PRINT(QLP1|QLP3, printk("qla4000_mailbox_command: Executing a mailbox request, while another one is in the F/W's mailbox in queue\n"));
	}
	#endif


	/* Send the mailbox command to the firmware
	 * ----------------------------------------
	 */
	ha->mbox_status_count = outCount;

	for (i=0; i<inCount; i++)
		QL4PRINT(QLP11, printk("qla4000_mailbox_command: Mailbox In[%d]  0x%08X\n", i, mbx_cmd[i]));

	/* Load all mailbox registers, except mailbox 0.*/
	for (i = 1; i < inCount; i++)
	{
		WRT_REG_DWORD(&reg->mailboxIn[i-1], mbx_cmd[i]);
	}

	/* Write Mailbox 0 to alert the firmware that the mailbox registers
	 * contain a command to be processed.  NOTE: We could be interrupted
	 * here if system interrupts are enabled */
	WRT_REG_DWORD(&reg->mailboxIn0, mbx_cmd[0]);
	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	set_current_state(TASK_UNINTERRUPTIBLE);
	add_wait_queue(&ha->mailbox_wait_queue,&wait);

	/*
	 * If we don't want status, don't wait for the mailbox command to complete.
	 * For example, MBOX_CMD_RESET_FW doesn't return status, you must poll the
	 * inbound Interrupt Mask for completion.
	 */
	if (outCount == 0)
	{
		status = QLA_SUCCESS;
		set_current_state(TASK_RUNNING);
		remove_wait_queue(&ha->mailbox_wait_queue,&wait);
		goto mbox_exit;
	}

	/*
	 * Wait for command to complete
	 * -----------------------------
	 */
	for (wait_count = MBOX_WAIT_COUNT * 1000; wait_count != 0; wait_count-=500)
	{
		spin_lock_irqsave(&ha->hardware_lock, flags);
		intr_status = RD_REG_DWORD(&reg->interruptStatusOut);
		QL4PRINT(QLP11, printk("qla4000_mailbox_command: INTR_STATUS = 0x%X\n", intr_status));
		if (intr_status & INTR_PENDING)
		{
			/*
			 * Service the interrupt.  The ISR will save the mailbox
			 * status registers to a temporary storage location in the
			 * adapter structure.
			 */
			ha->mbox_status_count = outCount;
			qla4000_interrupt_service_routine(ha);
		}
		spin_unlock_irqrestore(&ha->hardware_lock, flags);

		/*
		 * Command is done, clear the mailbox flags
		 */
		if (test_bit(AF_MBOX_COMMAND_DONE, &ha->flags))
			break;	/* exit wait loop */

		/*
		 * Delay for 500 milliseconds
		 * NOTE: interrupt_handler may be called here, if interrupts are enabled
		 */
		QL4PRINT(QLP11, printk("qla4000_mailbox_command: delay 500 msec, wait_count = %d\n",
				       wait_count));
		mdelay(500);
	} /* wait loop */

	set_current_state(TASK_RUNNING);
	remove_wait_queue(&ha->mailbox_wait_queue,&wait);

	/*
	 * Check for mailbox timeout
	 */
	if (wait_count == 0)
	{
		QL4PRINT(QLP5|QLP1, 
			 printk("qla4000: Mailbox Cmd 0x%08X timed out ..., "
				"Scheduling Adapter Reset\n", mbx_cmd[0]));

		ha->mailbox_timeout_count++;

		set_bit(DPC_RESET_HA, &ha->dpc_flags);
		set_bit(AF_SCHEDULE_DPC, &ha->flags);
		goto mbox_exit;
	}

	/*
	 * Copy the mailbox out registers to the caller's mailbox in/out structure.
	 */
	for (i=0; i < outCount; i++)
	{
		mbx_sts[i] = ha->mbox_status[i];
		QL4PRINT(QLP11,
			 printk("qla4000_mailbox_command: Mailbox Status[%d]  0x%08X\n", i, mbx_sts[i]));
	}

	/*
	 * Set return status and error flags (if applicable)
	 */
	switch (ha->mbox_status[0])
	{
	
	case MBOX_STS_COMMAND_COMPLETE:
		status = QLA_SUCCESS;
		break;

	case MBOX_STS_INTERMEDIATE:
		status = QLA_SUCCESS;
		QL4PRINT(QLP5, printk("qla4000_mailbox_command: Cmd = %08X, Intermediate completion\n", mbx_cmd[0]));
		break;

	case MBOX_STS_BUSY:
		QL4PRINT(QLP1, printk("qla4000_mailbox_command: Cmd = %08X, ISP BUSY\n", mbx_cmd[0]));
		QL4PRINT(QLP1|QLP5, printk("qla4000: mailbox command %08X, BUSY TIMEOUT\n", mbx_cmd[0]));
		ha->mailbox_timeout_count++;
		break;

	case MBOX_STS_INVALID_COMMAND:
	case MBOX_STS_HOST_INTERFACE_ERROR:
	case MBOX_STS_TEST_FAILED:
	case MBOX_STS_COMMAND_ERROR:
	case MBOX_STS_COMMAND_PARAMETER_ERROR:
	default:
		QL4PRINT(QLP5|QLP1, printk("qla4000_mailbox_command: **** FAILED, cmd = %08X, sts = %08X ****\n",
					   mbx_cmd[0], mbx_sts[0]));
		/*qla4000_dump_registers(QLP10, ha);*/
		break;
	} /* switch mbox status */

	mbox_exit:
	clear_bit(AF_MBOX_COMMAND, &ha->flags);
	clear_bit(AF_MBOX_COMMAND_DONE, &ha->flags);
	LEAVE("qla4000_mailbox_command");
	up(&ha->mbox_sem);
	return(status);
}

uint8_t qla4000_process_ddb_changed(scsi_qla_host_t *ha, uint32_t fw_ddb_index, uint32_t device_state)
{
	uint8_t     status = QLA_ERROR;
	ddb_entry_t *ddb_entry;
	uint32_t    old_device_state;

	ENTER("qla4000_process_ddb_changed");

	/* check for out of range index */
	if (fw_ddb_index >= MAX_DDB_ENTRIES)
	{
		QL4PRINT(QLP2|QLP3,
			 printk("qla4000_process_ddb_changed: device index 0x%x out of range\n",
				fw_ddb_index));
		goto exit_process_database_changed;
	}

	/* Get the corresponging ddb entry */
	ddb_entry = qla4000_lookup_ddb_by_fw_index(ha, fw_ddb_index);

	/* Check to see if it's a device we already have in our database */
	if (ddb_entry == NULL)
	{
		/* New device. Let's add it to the database */
		QL4PRINT(QLP2|QLP3,
			 printk("qla4000_process_ddb_changed: new device at index 0x%x\n",
				fw_ddb_index));

		/* First allocate a device structure */
		ddb_entry = qla4000_alloc_ddb(ha, fw_ddb_index);
		if (ddb_entry == NULL)
		{
			printk(KERN_DEBUG "qla4000: Unable to allocate memory to add fw_ddb_index %x\n",
			       fw_ddb_index);
		}
		else if (qla4000_update_ddb_entry(ha, ddb_entry) == QLA_SUCCESS)
		{
			uint16_t target;


			for (target = 0; target < MAX_TARGETS; target++)
			{
				/*
				 * First verify that this target id is not already mapped to a ddb entry,
				 * then perform a discovery of the target id and all possible luns.
				 */
				if ((qla4000_lookup_ddb_by_SCSIID(ha, 0, target) == NULL) &&
				    (qla4000_discover_target_luns(ha, ddb_entry, target) == QLA_SUCCESS))
				{
					printk(KERN_DEBUG "qla4000: %d:%d:%d: added new device (index 0x%x)\n",
					       ha->host_no, ddb_entry->bus, target, fw_ddb_index);
					status = QLA_SUCCESS;
					break;
				}
			}

			if (status != QLA_SUCCESS)
			{
				printk(KERN_DEBUG "qla4000: failed to add new device at index 0x%x. "
				       "Unable to locate corresponding SCSI target ID\n", fw_ddb_index);
				qla4000_free_ddb(ha, ddb_entry);
			}
		}
		else
		{
			printk(KERN_DEBUG "qla4000: failed to add new device at index 0x%x\n"
			       "Unable to retrieve fw ddb entry\n", fw_ddb_index);
			qla4000_free_ddb(ha, ddb_entry);
		}
	}
	else  /* device already exists */
	{
		old_device_state = ddb_entry->device_state;
		qla4000_get_ddb_entry(ha, ddb_entry->fw_ddb_index, NULL, NULL, NULL, &ddb_entry->device_state);

		/* device came back ? */
		if (ddb_entry->device_state == DDB_DS_SESSION_ACTIVE)
		{
			unsigned long cpu_flags;
			lun_entry_t *lun_entry;
			int i;


			printk(/*KERN_INFO*/ "qla4000: %d:%d:%d: index [0x%x] login notification received\n",
			       ha->host_no, ddb_entry->bus, ddb_entry->target, fw_ddb_index);

			atomic_set(&ddb_entry->port_down_timer,
				   ha->port_down_retry_count);
			atomic_set(&ddb_entry->state, DEV_STATE_ONLINE);
			clear_bit(DEV_FLAG_RELOGIN, &ddb_entry->flags);

			printk(/*KERN_INFO*/ "%s: %d:%d:%d: index [0x%x] marked ONLINE\n",
				   __FUNCTION__, ha->host_no, ddb_entry->bus, ddb_entry->target, ddb_entry->fw_ddb_index);

			/* 
			 * Change the lun state to READY in case the lun 
			 * TIMEOUT before the device came back.
			 */
			for (i = 0; i < MAX_LUNS; i++)
			{
				lun_entry = ddb_entry->lun_table[i];

				if (lun_entry)
				{
					spin_lock_irqsave(&lun_entry->lun_lock, cpu_flags);
					lun_entry->lun_state = LS_LUN_READY;
					spin_unlock_irqrestore(&lun_entry->lun_lock, cpu_flags);
				}
			}

			/* Send out any commands that we're holding */
			qla4xxx_start_io(ha);

			qla4000_rst_extended_debug_level();
		}
		else /* Device went away, try to relogin */
		{
			if (atomic_read(&ddb_entry->state) ==
			    DEV_STATE_ONLINE)
			{
				atomic_set(&ddb_entry->relogin_retry_count,
					   MAX_RELOGIN_RETRIES);
				atomic_set(&ddb_entry->state, DEV_STATE_MISSING);

				printk(/*KERN_INFO*/ "%s: %d:%d:%d: index [0x%x] marked MISSING\n",
					   __FUNCTION__, ha->host_no, ddb_entry->bus, ddb_entry->target, ddb_entry->fw_ddb_index);
			}

			if (old_device_state != ddb_entry->device_state)
			{
				printk(/*KERN_INFO*/ "qla4000: %d:%d:%d: index [0x%x] relogin sent, wait for login notification\n",
				       ha->host_no, ddb_entry->bus, ddb_entry->target, fw_ddb_index);

				qla4000_relogin_device(ha, ddb_entry);
			}

			//qla4000_rst_extended_debug_level();
		}
		status = QLA_SUCCESS;

	} /* valid ddb_entry */

	exit_process_database_changed:
	LEAVE("qla4000_process_ddb_changed");
	return(status);
}

uint8_t qla4000_process_aen(scsi_qla_host_t *ha)
{
	uint8_t status = QLA_ERROR;
	uint32_t mbox_sts[MBOX_AEN_REG_COUNT];
	aen_t   *aen;
	int     i;

	ENTER("qla4000_process_aen");

	while (ha->aen_out != ha->aen_in)
	{
		spin_lock(&ha->aen_q_spinlock);

		/* Advance pointers for next entry */
		if (ha->aen_out == (MAX_AEN_ENTRIES - 1))
			ha->aen_out = 0;
		else
			ha->aen_out++;

		ha->aen_q_count++;
		aen = &ha->aen_q[ha->aen_out];

		/* copy aen information to local structure */
		for (i=0; i < MBOX_AEN_REG_COUNT; i++)
			mbox_sts[i] = aen->mbox_sts[i];

		spin_unlock(&ha->aen_q_spinlock);
		switch (mbox_sts[0])
		{
		case MBOX_ASTS_SYSTEM_ERROR:
		case MBOX_ASTS_REQUEST_TRANSFER_ERROR:
		case MBOX_ASTS_RESPONSE_TRANSFER_ERROR:
			printk(/*KERN_INFO*/ "qla4000: aen %04x, ERROR Status, Reset HA\n",
			       mbox_sts[0]);

			set_bit(DPC_RESET_HA, &ha->dpc_flags);
			set_bit(AF_SCHEDULE_DPC, &ha->flags);
			status = QLA_SUCCESS;
			break;

		case MBOX_ASTS_PROTOCOL_STATISTIC_ALARM:
		case MBOX_ASTS_SCSI_COMMAND_PDU_REJECTED:  /* Target mode only */
		case MBOX_ASTS_UNSOLICITED_PDU_RECEIVED:   /* connection mode only */
			/* No action */
			printk(/*KERN_INFO*/ "qla4000: aen %04x\n", mbox_sts[0]);
			status = QLA_SUCCESS;
			break;

		case MBOX_ASTS_DATABASE_CHANGED:
			printk(/*KERN_INFO*/ "qla4000: aen %04x, index [0x%x], state=%04x, modifier=%04x\n",
			       mbox_sts[0], mbox_sts[2], mbox_sts[3], mbox_sts[1]);

			if (mbox_sts[1] == 1) /* applies to specific device */
			{
				status = qla4000_process_ddb_changed(ha, mbox_sts[2], mbox_sts[3]);
			}
			else if (mbox_sts[1] == 0) /* applies to all devices */
			{
				/* We should never really get here ...*/
				QL4PRINT(QLP2|QLP3, printk("qla4000_process_aen: global datbase changed aen, mbox_sts[1]=%04x\n",
							   mbox_sts[1]));
			}
			break;
		default:
			printk(KERN_INFO "qla4000: aen %04x UNKNOWN\n", 
			       mbox_sts[0]);
		}
	}

	LEAVE("qla4000_process_aen");
	return(status);
}


uint8_t qla4000_process_aen_with_flush(scsi_qla_host_t *ha)
{
	uint8_t status = QLA_ERROR;
	uint32_t mbox_sts[MBOX_AEN_REG_COUNT];
	aen_t   *aen;
	int     i;

	ENTER("qla4000_process_aen");

	while (ha->aen_out != ha->aen_in)
	{
		spin_lock(&ha->aen_q_spinlock);

		/* Advance pointers for next entry */
		if (ha->aen_out == (MAX_AEN_ENTRIES - 1))
			ha->aen_out = 0;
		else
			ha->aen_out++;

		ha->aen_q_count++;
		aen = &ha->aen_q[ha->aen_out];

		/* copy aen information to local structure */
		for (i=0; i < MBOX_AEN_REG_COUNT; i++)
			mbox_sts[i] = aen->mbox_sts[i];

		spin_unlock(&ha->aen_q_spinlock);
		switch (mbox_sts[0])
		{
		case MBOX_ASTS_SYSTEM_ERROR:
		case MBOX_ASTS_REQUEST_TRANSFER_ERROR:
		case MBOX_ASTS_RESPONSE_TRANSFER_ERROR:
			printk(/*KERN_INFO*/ "qla4000: aen %04x, ERROR Status, Reset HA\n",
			       mbox_sts[0]);

			set_bit(DPC_RESET_HA, &ha->dpc_flags);
			set_bit(AF_SCHEDULE_DPC, &ha->flags);
			status = QLA_SUCCESS;
			break;

		case MBOX_ASTS_PROTOCOL_STATISTIC_ALARM:
		case MBOX_ASTS_SCSI_COMMAND_PDU_REJECTED:  /* Target mode only */
		case MBOX_ASTS_UNSOLICITED_PDU_RECEIVED:   /* connection mode only */
			/* No action */
			printk(/*KERN_INFO*/ "qla4000: aen %04x\n", mbox_sts[0]);
			status = QLA_SUCCESS;
			break;

		case MBOX_ASTS_DATABASE_CHANGED:
			break;
		default:
			printk(KERN_INFO "qla4000: aen %04x UNKNOWN\n", 
			       mbox_sts[0]);
		}
	}

	LEAVE("qla4000_process_aen");
	return(status);
}
/**************************************************************************
 * qla4000_intr_handler
 *   Handles the H/W interrupt
 **************************************************************************/
void qla4000_intr_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	scsi_qla_host_t *ha;
	ISP_REG_T       *reg;
	uint32_t        intr_status;
	unsigned long flags = 0;

	ENTER("qla4000_intr_handler");
	ha = (scsi_qla_host_t *) dev_id;
	if (!ha)
	{
		printk(KERN_INFO "qla4000: Interrupt with NULL host ptr\n");
		return;
	}


	ha->isr_count++;
	reg = ha->base_address;

	/*
	 * Check for pending interrupts
	 */
	spin_lock_irqsave(&ha->hardware_lock, flags);
	intr_status = RD_REG_DWORD(&reg->interruptStatusOut);
	if (intr_status & INTR_PENDING)
	{
		/* Repeatedly service interrupt up to a maximum of
		 * MAX_REQS_SERVICED_PER_INTR
		 */
		uint8_t reqs_count = 0;

		while (intr_status & INTR_PENDING)
		{
			qla4000_interrupt_service_routine(ha);
			ha->total_io_count++;
			if (++reqs_count == MAX_REQS_SERVICED_PER_INTR)
			{
				//QL4PRINT(QLP3|QLP5,
				//		    printk("qla4000_intr_handler: exiting, %d requests serviced\n",
				//		    reqs_count));
				break;
			}

			intr_status = RD_REG_DWORD(&reg->interruptStatusOut);
		}
	}
	#if TRACK_SPURIOUS_INTRS
	else
	{
		QL4PRINT(QLP10, printk("qla4000(%d): Spurious interrupt (0x%08x)\n",
				       (int)ha->host_no, intr_status));
		ha->spurious_int_count++;
	}
	#endif
	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	qla4xxx_start_io(ha);

	QL4PRINT(QLP5, printk("qla4000_intr_handler: ha->flags = 0x%08x\n", ha->flags));
	if (test_bit(AF_SCHEDULE_DPC, &ha->flags)  &&
	    !test_bit(AF_DPC_SCHEDULED, &ha->flags)  &&
	    !ha->dpc_active && ha->dpc_wait)
	{
		QL4PRINT(QLP2|QLP5, printk("qla4000_intr_handler: scheduling dpc routine\n"));
		set_bit(AF_DPC_SCHEDULED, &ha->flags);
		up(ha->dpc_wait);
	}

	LEAVE("qla4000_intr_handler");
}

void
qla4000_watchdog(unsigned long data)
{
	scsi_qla_host_t *ha = (scsi_qla_host_t *) data;
	struct list_head *ptr, *next; /* used for traversing lists */
	ddb_entry_t *ddb_entry;
	unsigned long flags = 0;

#if 0
	if (atomic_read(&ha->adapter_state) != AS_READY)
	{
		QL4PRINT(QLP6, printk("qla4000_watchdog: A%d state=\"%s\"\n",
                                  ha->instance, adapter_state_msg[atomic_read(&ha->adapter_state)]));
		goto exit_watchdog;
	}
#endif

#if ISP_RESET_TEST
	if (ha->isp_reset_timer++ == (60 *3))
	{
		printk("%s going to schedule BIG HAMMER\n",__FUNCTION__); 
		set_bit(DPC_RESET_HA, &ha->dpc_flags);
		set_bit(AF_SCHEDULE_DPC, &ha->flags);
		ha->isp_reset_timer = 0;
	}
#endif

#if 0
	QL4PRINT(QLP6, printk("qla4000_watchdog: A%d=%d/%d <%d,%d> {%d,%d,%d} %d\n",
			      ha->instance, ha->spurious_int_count, ha->isr_count,
			      ha->aborted_io_count, ha->mailbox_timeout_count,
			      MAX_AEN_ENTRIES-ha->aen_q_count,
			      ha->pending_srb_q_count, ha->active_srb_count,
			      ha->seconds_since_last_intr));
#endif

	/* Do we need to process the retry queue? */
	if (!list_empty(&ha->retry_srb_q))
	{
		set_bit(AF_SCHEDULE_DPC, &ha->flags);
	}

	/* LUN suspension */
	if (!list_empty(&ha->suspended_lun_q))
	{
		unsigned long cpu_flags;
		QL4PRINT(QLP2,printk("%s Going to process suspend_lun_q\n",__FUNCTION__));
		spin_lock_irqsave(&ha->adapter_lock, flags);
		list_for_each_safe(ptr, next, &ha->suspended_lun_q)
		{
			lun_entry_t *lun_entry;

			lun_entry = list_entry(ptr, lun_entry_t, list_entry);
			spin_lock_irqsave(&lun_entry->lun_lock, cpu_flags);

			if (lun_entry->lun_state == LS_LUN_SUSPENDED &&
			    atomic_read(&lun_entry->suspend_timer) != 0)
			{
				QL4PRINT(QLP2,
					 printk("%s processing suspend_lun_q lun=%d suspend_timer=%d retry_count=%d\n",
						__FUNCTION__, lun_entry->lun,
						atomic_read(&lun_entry->suspend_timer),
						lun_entry->retry_count));

				if (atomic_dec_and_test(&lun_entry->suspend_timer) != 0)
				{
					lun_entry->retry_count++;
					if (lun_entry->retry_count == lun_entry->max_retry_count)
					{
						QL4PRINT(QLP2|QLP6,
							 printk("qla4000_watchdog: LUN %d TIMEOUT RETRY_CNT:%d\n",
								lun_entry->lun,lun_entry->retry_count));

						lun_entry->lun_state = LS_LUN_TIMEOUT;
						__del_from_suspended_lun_q(ha, lun_entry);

					}
					else
					{
						QL4PRINT(QLP2|QLP6,
							 printk("qla4000_watchdog: LUN %d RETRY\n",
								lun_entry->lun));

						lun_entry->lun_state = LS_LUN_RETRY;
						__del_from_suspended_lun_q(ha, lun_entry);
					}
				}
			}
			spin_unlock_irqrestore(&lun_entry->lun_lock, cpu_flags);
		}
		spin_unlock_irqrestore(&ha->adapter_lock, flags);

	}

	/*
	 * Search for relogin's to time-out.
	 */
	list_for_each_safe(ptr, next, &ha->ddb_list)
	{
		uint16_t dev_state;
		ddb_entry = list_entry(ptr, ddb_entry_t, list_entry);

		dev_state = (uint16_t) atomic_read(&ddb_entry->state);


		/* First check to see if the device has exhausted the
		 * port down retry count */
		if (dev_state == DEV_STATE_MISSING)
		{
/*printk("%s [%x] ddb_entry=%p,flags=0x%x port_down_timer=%d,max=%d\n", __FUNCTION__, ddb_entry->fw_ddb_index, ddb_entry,ddb_entry->flags,atomic_read(&ddb_entry->port_down_timer ),ha->port_down_retry_count); */
			if (atomic_read(&ddb_entry->port_down_timer ) == 0)
				continue;

			if (atomic_dec_and_test(&ddb_entry->port_down_timer ) !=0)
			{
				QL4PRINT(
					QLP2|QLP6,
					printk("qla4000_watchdog: index [%x] port down retry count of (%d) secs exhausted, marking device DEAD.\n",
					       ddb_entry->fw_ddb_index, ha->port_down_retry_count));

				atomic_set(&ddb_entry->state, DEV_STATE_DEAD);
				printk(/*KERN_INFO*/ "%s: %d:%d:%d: index [0x%x] marked DEAD\n",
					   __FUNCTION__, ha->host_no, ddb_entry->bus, ddb_entry->target, ddb_entry->fw_ddb_index);
			}

		}

		if (atomic_read(&ddb_entry->relogin_timer))
/*printk("qla4000_watchdog: dev[%x] relogin timer %d, relogin retry count %d\n",
			       ddb_entry->fw_ddb_index,
			       atomic_read(&ddb_entry->relogin_timer),
			       atomic_read(&ddb_entry->relogin_retry_count)); */

		if (atomic_read(&ddb_entry->relogin_timer)  &&
		    (atomic_dec_and_test(&ddb_entry->relogin_timer) != 0 ))
		{
			/*
			 * If the relogin times out and the device is still
			 * MISSING then try and relogin again. 
			 */
			if (atomic_read(&ddb_entry->state) != DEV_STATE_ONLINE                                 
			    && atomic_read(&ddb_entry->relogin_retry_count)                            
			    &&
			    atomic_dec_and_test(&ddb_entry->relogin_retry_count) == 0)
			{
				QL4PRINT(
					QLP1|QLP6,
					printk("qla4000_watchdog: index[%x] relogin timed out-retrying relogin\n",
					       ddb_entry->fw_ddb_index));
/*printk("qla4000_watchdog: dev[%x] retrying login..\n",
				       ddb_entry->fw_ddb_index); */
				set_bit(DPC_RETRY_RELOGIN, &ha->dpc_flags);
				set_bit(DEV_FLAG_RELOGIN, &ddb_entry->flags);
				set_bit(AF_SCHEDULE_DPC, &ha->flags);
			}
		}
	}
#if defined(EH_WAKEUP_WORKAROUND)
        if (ha->host->in_recovery &&
#if defined(EH_WAKEUP_WORKAROUND_REDHAT)
                (atomic_read(&(ha->host->host_busy)) ==
                      ha->host->host_failed) &&
#else
                (ha->host->host_busy == ha->host->host_failed) &&
#endif
                !ha->host->eh_active) {

                if ((ha->eh_start++) == 60) {
                        if (ha->host->eh_wait)
                                up(ha->host->eh_wait);
                        ha->eh_start=0;
                        printk("qla4000(%ld): !!! Waking up error handler "
                                "for scsi layer\n",
                                ha->host_no);
                }
        }
#endif /* EH_WAKEUP_WORKAROUND */


	/* Wakeup the dpc routine for this adapter, if needed */
	if (test_bit(AF_SCHEDULE_DPC, &ha->flags)  &&
	    !test_bit(AF_DPC_SCHEDULED, &ha->flags) &&
	    !ha->dpc_active && ha->dpc_wait)
	{
		QL4PRINT(QLP2|QLP6, printk("qla4000_watchdog: scheduling dpc routine\n"));
		set_bit(AF_DPC_SCHEDULED, &ha->flags);
		up(ha->dpc_wait);
	}


#if 0
	exit_watchdog:
#endif

	/* Reschedule watchdog timer to call us back at the next timer interval */
	ha->watchdog_timer.expires  = jiffies + (WDG_TIMER_INTERVAL * HZ);
	add_timer(&ha->watchdog_timer);

	ha->seconds_since_last_intr += WDG_TIMER_INTERVAL;
}

/**************************************************************************
 *   qla4000_do_dpc
 *
 * Description:
 * This routine is a task that is schedule by the interrupt handler
 * to perform the background processing for interrupts.  We put it
 * on a task queue that is consumed whenever the scheduler runs; that's
 * so you can do anything (i.e. put the process to sleep etc).  In fact, the
 * mid-level tries to sleep when it reaches the driver threshold
 * "host->can_queue". This can cause a panic if we were in our interrupt
 * code .
 **************************************************************************/
void qla4000_do_dpc(unsigned long p)
{
	DECLARE_MUTEX_LOCKED(sem);
	scsi_qla_host_t *ha = (scsi_qla_host_t *) p;
	static struct list_head *ptr, *next;
	static ddb_entry_t *ddb_entry;

	ENTER("qla4000_do_dpc");

#ifdef MODULE
	siginitsetinv(&current->blocked, SHUTDOWN_SIGS);
#else
	siginitsetinv(&current->blocked, 0);
#endif

	lock_kernel();

	/* Flush resources */
	daemonize();

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9)

	/* As mentioned in kernel/sched.c(RA).....
	 * Reparent the calling kernel thread to the init task.
	 *
	 * If a kernel thread is launched as a result of a system call,
	 * or if it ever exists,it should generally reparent itself to init
	 * so that it is correctly cleaned up on exit.
	 *
	 * The various task state such as scheduling policy and priority
	 * may have been inherited from a user process, so we reset them
	 * to sane values here.
	 *
	 * NOTE that reparent_to_init() gives the caller full capabilities.
	 *
	 */
	reparent_to_init();
#endif

	/*
	 * Set the name of this process.
	 */
	sprintf(current->comm, "qla4000_dpc_%d", (int)ha->host_no);
	ha->dpc_wait = &sem;
	ha->dpc_handler = current;

	unlock_kernel();

	/*
	 * Wake up the thread that created us.
	 */
	QL4PRINT(QLP3|QLP7, printk("qla4000_dpc_%d: Wake up parent %d\n",
				   (int)ha->host_no, ha->dpc_notify->count.counter));
	up(ha->dpc_notify);

	while (1)
	{
		/*
		 * If we get a signal, it means we are supposed to go
		 * away and die.  This typically happens if the user is
		 * trying to unload a module.
		 */

		QL4PRINT(QLP2|QLP3|QLP12, printk("qla4000_dpc_%d: DPC handler sleeping **********************\n",
						 (int)ha->host_no));

		down_interruptible(&sem);
		if (signal_pending(current))
			break;	 /* get out */


		QL4PRINT(QLP2|QLP3|QLP12, printk("qla4000_dpc_%d: DPC handler waking up *********************\n",
						 (int)ha->host_no));

		QL4PRINT(QLP2|QLP3, printk("qla4000_do_dpc: ha->flags = 0x%08x\n", ha->flags));
		QL4PRINT(QLP2|QLP3, printk("qla4000_do_dpc: ha->dpc_flags = 0x%08x\n", ha->dpc_flags));

		if (ha->dpc_active)
			continue;

		ha->dpc_active = 1;
		clear_bit(AF_DPC_SCHEDULED, &ha->flags);
		clear_bit(AF_SCHEDULE_DPC, &ha->flags);

		/*
		 * Determine what action is necessary
		 */
		QL4PRINT(QLP3, printk("qla4000_do_dpc: Do we need to recover the adapter?\n"));
		if (test_bit(DPC_RESET_HA_DESTROY_DDB_LIST, &ha->dpc_flags))
			qla4000_recover_adapter(ha, 1);
		if (test_bit(DPC_RESET_HA, &ha->dpc_flags))
			qla4000_recover_adapter(ha, 0);

		QL4PRINT(QLP3, printk("qla4000_do_dpc: Do we need to process an AEN?\n"));
		if (ha->aen_in != ha->aen_out)
			qla4000_process_aen(ha);

		QL4PRINT(QLP3, printk("qla4000_do_dpc: Do we need to reset a device?\n"));

#if 0		
		if (test_and_clear_bit(DPC_RESET_DEVICE, &ha->dpc_flags))
		{
			list_for_each_safe(ptr, next, &ha->ddb_list)
			{
				ddb_entry = list_entry(ptr, ddb_entry_t, list_entry);

				if (atomic_read(&ddb_entry->state) ==  DEV_STATE_RESET)
					qla4000_reset_target(ha, ddb_entry);
			}
		}
#endif		

		QL4PRINT(QLP3, printk("qla4000_do_dpc: Do we need to retry a device relogin?\n"));
/*printk("qla4000_do_dpc: Login? dpc flags =0x%x\n", ha->dpc_flags); */
		if (test_and_clear_bit(DPC_RETRY_RELOGIN, &ha->dpc_flags))
		{
/*printk("%s Trying to issue a relogin\n",__FUNCTION__); */
			list_for_each_safe(ptr, next, &ha->ddb_list)
			{
				ddb_entry = list_entry(ptr, ddb_entry_t, list_entry);

/*printk("%s issue relogin for ddb_entry=%p ddb_index=%d\n",
				       __FUNCTION__,ddb_entry,ddb_entry->fw_ddb_index); */
				if (test_and_clear_bit(DEV_FLAG_RELOGIN, &ddb_entry->flags))
				{
					qla4000_relogin_device(ha, ddb_entry);
				}
			}
		}

		QL4PRINT(QLP3, printk("qla4000_do_dpc: Do we need to return cmds on the retry_q?\n"));
		if (!list_empty(&ha->retry_srb_q))
		{
			srb_t    *srb;
			unsigned long flags;

			spin_lock_irqsave(&ha->list_lock, flags);
			list_for_each_safe(ptr, next, &ha->retry_srb_q)
			{
				srb = list_entry(ptr, srb_t, list_entry);

				if (srb->lun_entry == NULL)
					BUG();

				if (srb->lun_entry->lun_state == LS_LUN_SUSPENDED)
					continue;

				if ((atomic_read(&srb->ddb_entry->state) == DEV_STATE_DEAD))
				{
					QL4PRINT(QLP5, printk("qla4000_flush_retry_srb_q: found srb %p in retry_srb_q, returning\n", srb));
					__del_from_retry_srb_q(ha, srb);
					CMD_RESULT(srb->cmd) = (int) (DID_NO_CONNECT << 16);
					add_to_done_srb_q(ha,srb);
				}

				/* Send requests to OS when device goes ONLINE */
				if ((atomic_read(&srb->ddb_entry->state) == DEV_STATE_ONLINE))
				{
					QL4PRINT(QLP5, printk("qla4000_flush_retry_srb_q: found srb %p in retry_srb_q, returning\n", srb));
					__del_from_retry_srb_q(ha, srb);
					CMD_RESULT(srb->cmd) = (int) (DID_BUS_BUSY << 16);
					add_to_done_srb_q(ha,srb);
				}
			}
			spin_unlock_irqrestore(&ha->list_lock, flags);

			while ((srb = del_from_done_srb_q_head(ha)) != NULL)
				qla4000_complete_request(ha, srb);
		}

#if 0
		/* Send all requests on pending_srb_q to ISP */
		QL4PRINT(QLP3, printk("qla4000_do_dpc: Is there something in the pending_srb_q?\n"));
		if (ha->pending_srb_q_count)
			qla4000_process_pending_srb_q(ha, NULL);
#endif

		qla4000_rst_extended_debug_level();
		ha->dpc_active = 0;
	}

	QL4PRINT(QLP3|QLP7, printk("qla4000_dpc_%d: Wake up, we're leaving\n",
				   (int)ha->host_no));

	/*
	* Make sure that nobody tries to wake us up again.
	*/
	ha->dpc_wait = NULL;
	ha->dpc_handler = NULL;
	ha->dpc_active = 0;

	/*
	* If anyone is waiting for us to exit (i.e. someone trying to unload
	* a driver), then wake up that process to let them know we are on
	* the way out the door.  This may be overkill - I *think* that we
	* could probably just unload the driver and send the signal, and when
	* the error handling thread wakes up that it would just exit without
	* needing to touch any memory associated with the driver itself.
	*/
	if (ha->dpc_notify != NULL)
		up(ha->dpc_notify);

	LEAVE("qla4000_do_dpc");
}


#if STOP_ON_ERROR
/**************************************************************************
*   ql2100_panic
*
**************************************************************************/
static void qla4000_panic(char *cp, scsi_qla_host_t *ha)
{
	printk("qla4000 - PANIC:  %s\n",cp);
	printk("Current time=0x%lx\n", jiffies);
	printk("Number of pending commands =0x%lx\n", ha->pending_srb_q_count);
	printk("Number of retry commands =0x%lx\n", ha->retry_srb_q_count);
	printk("Number of active commands =0x%lx\n", ha->active_srb_count);
	printk("Number of free entries = (%d)\n",ha->req_q_count);
	printk("Request Queue @ 0x%lx, Response Queue @ 0x%lx\n",
	       ha->req_queue.virt_addr, ha->rsp_queue.virt_addr);
	printk("Request In Ptr %d\n", ha->request_in);
	printk("HA flags =0x%lx\n", ha->flags);
	/*qla4000_dump_registers(QLP10, ha);*/

	cli();
	while (1)
	{
		udelay(2); barrier();
	}
	sti();
}
#endif

/*
 * qla4000_eh_wait_on_command
 *    Waits for the command to be returned by the Firmware for some
 *    max time.
 *
 * Input:
 *    ha = actual ha whose done queue will contain the command
 *	      returned by firmware.
 *    cmd = Scsi Command to wait on.
 *    flag = Abort/Reset(Bus or Device Reset)
 *
 * Return:
 *    Not Found : 0
 *    Found : 1
 */
STATIC int
qla4000_eh_wait_on_command(scsi_qla_host_t *ha, Scsi_Cmnd *cmd)
{
#define ABORT_WAIT_TIME	120 /* seconds */

	int             done = 0;
	srb_t           *rp;
	uint32_t        max_wait_time = ABORT_WAIT_TIME;

	ENTER("qla4000_eh_wait_on_command");


	do
	{
		/*Checking to see if its returned to OS */
		rp = (srb_t *) CMD_SP(cmd);
		if (rp == NULL)
		{
			done++;
			break;
		}

		spin_unlock_irq(&io_request_lock);

		set_current_state(TASK_INTERRUPTIBLE);
		schedule_timeout(2*HZ);

		while ((rp = del_from_done_srb_q_head(ha)) != NULL)
			qla4000_complete_request(ha, rp);

		spin_lock_irq(&io_request_lock);

	} while ((max_wait_time--));

	if (done)
		QL4PRINT(QLP2, 
			 printk("%s: found cmd=%p.\n", 
				__FUNCTION__, cmd));

	LEAVE("qla4000_eh_wait_on_command");

	return(done);
}


int
qla4000_eh_abort(Scsi_Cmnd *cmd)
{
	int return_status = FAILED;
	scsi_qla_host_t *ha;
	srb_t *srb;
	struct list_head *ptr, *next;
	unsigned int flags;
#if 0
	static uint32_t mbox_cmd[MBOX_REG_COUNT];
	static uint32_t mbox_sts[MBOX_REG_COUNT];
	ddb_entry_t *ddb_entry;
	int i;
#endif

	ENTER("qla4000_eh_abort");

	ha = (scsi_qla_host_t *) cmd->host->hostdata;
	srb = (srb_t *) CMD_SP(cmd);

	if (!srb)
	{
		/* Srb invalid, for some reason, possibly already returned to the OS */
		QL4PRINT(QLP1, 
			 printk("%s: NULL srb %p, b%d, t%x\n",
				__FUNCTION__, srb, cmd->channel, cmd->target));
		CMD_RESULT(cmd) = (int) (DID_ABORT << 16);
		return_status = SUCCESS;
		return(return_status);
	}

	ha->aborted_io_count++;

	/* Print statements
	 * ---------------- */
	printk("scsi %d:%d:%d:%d: abort srb=%p, cmd=%p, state=%s, r_start=%ld , u_start=%ld\n",
	       ha->host_no, cmd->channel, cmd->target, cmd->lun,
	       srb, cmd, srb_state_msg[srb->state],srb->r_start,srb->u_start);
	qla4000_dump_dwords(QLP10, srb, sizeof(*srb));
	qla4000_set_extended_debug_level();

	/* If srb found in pending_srb_q, return  the cmd with ABORTED status */
	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_safe(ptr, next,  &ha->pending_srb_q)
	{
		srb = list_entry(ptr, srb_t, list_entry);

		if (srb->cmd != cmd)
			continue;

		QL4PRINT(QLP2|QLP5, 
			 printk("%s: srb %p found on pending queue\n", 
				__FUNCTION__, srb));
		__del_from_pending_srb_q(ha, srb);
		CMD_RESULT(cmd) = DID_ABORT << 16;

		spin_unlock_irqrestore(&ha->list_lock, flags);
		spin_unlock_irq(&io_request_lock);
		qla4000_complete_request(ha, srb);
		spin_lock_irq(&io_request_lock);
		return_status = SUCCESS;
		goto exit_abort;
	}
	spin_unlock_irqrestore(&ha->list_lock, flags);

	/* If srb found in retry_srb_q, return it with ABORTED status */
	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_safe(ptr, next, &ha->retry_srb_q)
	{
		srb = list_entry(ptr, srb_t, list_entry);

		if (srb->cmd != cmd)
			continue;

		QL4PRINT(QLP2|QLP5, 
			 printk("%s: srb %p found on retry queue\n", 
				__FUNCTION__, srb));
		__del_from_retry_srb_q(ha, srb);
		CMD_RESULT(cmd) = DID_ABORT << 16;

		spin_unlock_irqrestore(&ha->list_lock, flags);
		spin_unlock_irq(&io_request_lock);
		qla4000_complete_request(ha, srb);
		spin_lock_irq(&io_request_lock);
		return_status = SUCCESS;
		goto exit_abort;
	}
	spin_unlock_irqrestore(&ha->list_lock, flags);

	/* If srb found in done_q, return the cmd with ABORTED status-??? */
	spin_lock_irqsave(&ha->adapter_lock, flags);
	list_for_each_safe(ptr, next, &ha->done_srb_q)
	{
		srb = list_entry(ptr, srb_t, list_entry);

		if (srb->cmd != cmd)
			continue;

		QL4PRINT(QLP2|QLP5, 
			 printk("%s: srb %p found on done queue\n", 
				__FUNCTION__, srb));
		CMD_RESULT(cmd) = DID_ABORT << 16;
		spin_unlock_irqrestore(&ha->adapter_lock, flags);
		spin_unlock_irq(&io_request_lock);
		while ((srb = del_from_done_srb_q_head(ha)) != NULL)
			qla4000_complete_request(ha, srb);

		spin_lock_irq(&io_request_lock);
		return_status = SUCCESS;
		goto exit_abort;
	}
	spin_unlock_irqrestore(&ha->adapter_lock, flags);

#if 0
	/* If srb found in active_srb array, send abort command to RISC */
	spin_lock_irqsave(&ha->hardware_lock, flags);
	for (i=0; i<QLA4000_MAX_CMDS_TO_RISC; i++)
	{
		srb = ha->active_srb_array[i];
		if (srb == NULL)
			continue;

		if (srb->cmd != cmd)
			continue;

		ddb_entry = qla4000_lookup_ddb_by_SCSIID(ha, 0, cmd->target);
		if (ddb_entry == 0)
		{
			spin_unlock_irqrestore(&ha->hardware_lock, flags);
			return(return_status);
		}

		QL4PRINT(QLP2|QLP5, printk("qla4000_eh_abort: ddb_entry[%d] = %p\n", cmd->target, ddb_entry));
		QL4PRINT(QLP2, printk(", iocb_to=%d, driver_to=%d, os_to=%d, task_mgmt to=%d\n",
				      srb->iocb_timeout, srb->driver_timeout,
				      srb->os_timeout, srb->task_mgmt_timeout));
		qla4000_dump_dwords(QLP10, ddb_entry, sizeof(*ddb_entry));


		mbox_cmd[0] = MBOX_CMD_ABORT_TASK;
		mbox_cmd[1] = ddb_entry->fw_ddb_index;	/* target */
		mbox_cmd[2] = (uint32_t) srb->active_array_index; /* handle */

		spin_unlock_irqrestore(&ha->hardware_lock, flags);
		spin_unlock_irq(&io_request_lock);
		qla4000_mailbox_command(ha, 3, 1, &mbox_cmd[0], &mbox_sts[0]);

		spin_lock_irq(&io_request_lock);
		spin_lock_irqsave(&ha->hardware_lock, flags);
		break;
	}
	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	if (qla4000_eh_wait_on_command(ha, cmd) != 0)
	{
		return_status = SUCCESS;
	}
	else
	{
		/* 
		 * If the command never completes, then we reset the adapter
		 * because the firmware is probably in a weird state.  
		 * 
		 * Also, returning FAILED status will escalate the error handler
		 * to do a device reset, which should recover the device if the
		 * iSCSI router is in a weird state.
		 */
		spin_unlock_irq(&io_request_lock);
		qla4000_recover_adapter(ha, 0);
		ddb_entry = qla4000_lookup_ddb_by_SCSIID(ha, 0, cmd->target);
		if (ddb_entry)
		{
			if (qla4000_reset_target(ha, ddb_entry) == QLA_SUCCESS)
				return_status = SUCCESS;
			else
				return_status = FAILED;
		}

                spin_lock_irq(&io_request_lock);
	}
#else
	/* 
	 * Aborts get translated to "device resets" by the scsi switch 
	 * which will return a RESET status and not ABORT. Since the
	 * mid-level is expecting an ABORT status during an abort(), 
	 * we always elevate to device reset.
	*/
	return_status = FAILED;
#endif


	exit_abort:
	QL4PRINT(QLP2, printk("qla4000_eh_abort: return with status = %x\n", return_status));
	LEAVE("qla4000_eh_abort");

	#if STOP_ON_ABORT
	qla4000_panic("qla4000_eh_abort", ha);
	#endif

	qla4000_rst_extended_debug_level();
	return(return_status);
}

/*
 * Reset the HBA via a write to the HBA Bridge's config register space
 * Hard Reset
 */
#define ECBR_ADDR			0x40
#define ECBR_RESET_INTERNAL_BUS	0x0020
uint8_t
qla4000_reset_adapter(scsi_qla_host_t *ha)
{
	uint16_t ecbr;
	uint8_t status = QLA_ERROR;

	ENTER("qla4000_reset_adapter");
	printk("qla4000: HARD RESET Adapter %d\n", 
	       ha->instance);

	if (ha->pci_brgdev)
	{
		if (pci_read_config_word(ha->pci_brgdev,
					 ECBR_ADDR, &ecbr) == 0)
		{
			qla4000_disable_intrs(ha);

			/* return all active commands to the mid-layer */

			ecbr |= ECBR_RESET_INTERNAL_BUS;
			if (pci_write_config_word(ha->pci_brgdev,
						  ECBR_ADDR, ecbr) == 0)
			{
				printk("qla4000: DELAY for 30 seconds\n");
				set_current_state(TASK_INTERRUPTIBLE);
				schedule_timeout(30*HZ);
				qla4000_flush_active_srbs(ha);

				printk("qla4000: HARD RESET Adapter %d: SUCCESS\n",
                                   ha->instance);
				status = QLA_SUCCESS;
			}
		}
		else
		{
			printk("qla4000: HARD RESET Adapter %d failed!\n", 
			       ha->instance);
		}
	}

	LEAVE("qla4000_reset_adapter");
	#if STOP_ON_RESET_ADAPTER
	qla4000_panic("qla4000_reset_adapter", ha);
	#endif
	return(status);
}

/*
 * Soft Reset
 */
uint8_t
qla4000_reset_firmware(scsi_qla_host_t *ha)
{
	uint8_t   status = QLA_ERROR;
	ISP_REG_T *reg   = ha->base_address;
	static uint32_t  mbox_cmd[MBOX_REG_COUNT];
	static uint32_t  mbox_sts[MBOX_REG_COUNT];
	uint32_t  i;
	uint32_t  intr_mask;
	unsigned long flags = 0;

	ENTER("qla4000_reset_firmware");
	printk("qla4000: RESET firmware on adapter %d\n", ha->instance);

	/* Issue mailbox command, do not wait for completion */
	mbox_cmd[0] = MBOX_CMD_RESET_FW;
	qla4000_mailbox_command(ha, 1, 0, &mbox_cmd[0], &mbox_sts[0]);

	/* Poll IntrMaskIn register for completion */
	for (i = RESET_FIRMWARE_DELAY; i!= 0; i--)
	{
		spin_lock_irqsave(&ha->hardware_lock, flags);
		intr_mask = RD_REG_DWORD(&reg->interruptMaskIn);
		spin_unlock_irqrestore(&ha->hardware_lock, flags);

		if (intr_mask == IMI_FW_RESET_COMPLETE)
		{
			status = QLA_SUCCESS;
			break;
		}

		QL4PRINT(QLP2, printk("qla4000_reset_firmware: Delay 1 sec, Time Remaining = %d\n", i));
		set_current_state(TASK_INTERRUPTIBLE);
		schedule_timeout(1 * HZ);
	}

	LEAVE("qla4000_reset_firmware");

	#if STOP_ON_RESET_FIRMWARE
	qla4000_panic("qla4000_reset_firmware", ha);
	#endif

	return(status);
}

/* qla4000_cmd_wait
*	Stall driver until all outstanding commands are returned.
*
* Input:
*	ha = adapter state pointer.
*
* Return;
*  QLA_SUCCESS
*  QLA_ERROR
*
* Context:
*	Kernel context. 
*
* Prerequisit:
*       Caller must release the Hardware Lock.
*/
STATIC uint8_t
qla4000_cmd_wait(scsi_qla_host_t *ha) 
{
	uint32_t index = 0;
	uint8_t stat = QLA_SUCCESS;
	int wait_cnt = 30; /* Initialized for 30 seconds as we expect all 
			      commands to retuned ASAP.*/
	unsigned long flags;

	ENTER("qla4000_cmd_wait: started\n");

	while (wait_cnt)
	{
		spin_lock_irqsave(&ha->hardware_lock, flags);
		/* Find a command that hasn't completed. */
		for (index = 1; index < QLA4000_MAX_CMDS_TO_RISC; index++)
		{
			if (ha->active_srb_array[index] != NULL)
				break;
		}
		spin_unlock_irqrestore(&ha->hardware_lock, flags);

		/* if No Commands are pending, wait is complete */                
		if (index == QLA4000_MAX_CMDS_TO_RISC)
		{
			break;
		}

		/* if we timed out on waiting for commands to come back
		 *  go ahead and do a Hardware reset.
		 */
		wait_cnt--;
		if (wait_cnt == 0)
			stat =  QLA_ERROR;
		else
		{
			/* sleep a second */
			set_current_state(TASK_INTERRUPTIBLE);
			schedule_timeout(1 * HZ);
		}
	} /* End of While (wait_cnt) */

	QL4PRINT(QLP2,printk("qla4000_cmd_wait(%d): "
			     "Done waiting on commands - index=%d\n",
			     (int)ha->host_no,index));

	LEAVE("qla4000_cmd_wait");

	return( stat );
}

/*
 * Recovers adapter from fatal state
 */
uint8_t
qla4000_recover_adapter(scsi_qla_host_t *ha, uint8_t ddb_list_todo)
{
	uint8_t status = QLA_SUCCESS;

	ENTER("qla4000_recover_adapter");

	printk("scsi %d:%d: recover adapter (begin)\n", 
	       (int)ha->host_no, ha->instance);
	qla4000_set_extended_debug_level();

	atomic_set(&ha->adapter_state , AS_RESET);
	QL4PRINT(QLP2,printk("%s calling qla4000_cmd_wait\n",__func__));

	/* Wait for outstanding commands to complete.
	 * Stalls the driver,max for 30 secs
	 */     
	status = qla4000_cmd_wait(ha); 

	qla4000_disable_intrs(ha);

	if (ddb_list_todo == 1)
	{
		qla4000_process_aen_with_flush(ha);
	}
	
	/* Reset the firmware.  If successful, function
	 * returns with ISP interrupts enabled.
	 */
	if (status == QLA_SUCCESS)
	{
	printk(KERN_INFO "scsi %d: %s - Performing soft reset..\n", 
	       (int)ha->host_no,__func__);
		status = qla4000_reset_firmware(ha);
	}

	/* If firmware (SOFT) reset failed, then
	 * do a HARD reset
	 */
	if (status == QLA_ERROR)
	{
	printk(KERN_INFO "scsi %d: %s - Performing hard reset..\n", 
	       (int)ha->host_no,__func__);
		status = qla4000_reset_adapter(ha);
	}

	if (ddb_list_todo == 1)
	{
		qla4000_process_aen_with_flush(ha);
	}
	
	/* Re-initialize firmware. If successful, function returns 
	 * with ISP interrupts enabled */
	if (status == QLA_SUCCESS)
	{
	printk(KERN_INFO "scsi %d: %s - Initializing adapter..\n", 
	       (int)ha->host_no,__func__);
		status = qla4000_initialize_adapter(ha, ddb_list_todo);
	} else 
		printk("qla4000: Failed to RESET firmware on adapter %d\n", ha->instance);

	atomic_set(&ha->adapter_state , AS_READY);

	ha->adapter_error_count++;
	clear_bit(DPC_RESET_HA, &ha->dpc_flags);
	clear_bit(DPC_RESET_HA_DESTROY_DDB_LIST, &ha->dpc_flags);
	qla4000_rst_extended_debug_level();
	printk("scsi %d:%d: recover adapter (end)\n", (int)ha->host_no, ha->instance);
	LEAVE("qla4000_recover_adapter");
	return(status);
}

uint8_t
qla4000_cold_reset_target(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry)
{
	static uint32_t mbox_cmd[MBOX_REG_COUNT];
	static uint32_t mbox_sts[MBOX_REG_COUNT];
	uint8_t  status = QLA_SUCCESS;

	ENTER("qla4000_cold_reset_target");
	QL4PRINT(QLP5|QLP2, printk("qla4000_warm_reset_target: device index [0x%x]\n", ddb_entry->fw_ddb_index));

	/*
	 * Send target warm reset command to ISP, so that the ISP
	 * will return all outstanding requests with RESET status
	 */
	mbox_cmd[0] = MBOX_CMD_TARGET_COLD_RESET;
	mbox_cmd[1] = ddb_entry->fw_ddb_index;
	mbox_cmd[2] = RESET_TARGET_DELAY;

	if (qla4000_mailbox_command(ha, 3, 1, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS)
	{
		QL4PRINT(QLP5|QLP2, printk("qla4000_warm_reset_target: MBOX_CMD_TARGET_WARM_RESET failed w/ status %04x\n", mbox_sts[0]));
		status = QLA_ERROR;
	}
	else
	{
		QL4PRINT(QLP5, printk("qla4000_warm_reset_target: MBOX_CMD_TARGET_WARM_RESET completed successfully\n"));

		/* Send marker to unlock queues */
		status = qla4000_send_marker(ha, ddb_entry, 0, MM_TARGET_WARM_RESET);
	}

	LEAVE("qla4000_cold_reset_target");
	return(status);
}

uint8_t
qla4000_warm_reset_target(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry)
{
	static uint32_t mbox_cmd[MBOX_REG_COUNT];
	static uint32_t mbox_sts[MBOX_REG_COUNT];
	uint8_t  status = QLA_SUCCESS;

	ENTER("qla4000_warm_reset_target");
	QL4PRINT(QLP5|QLP2, printk("qla4000_warm_reset_target: index [0x%x]\n", ddb_entry->fw_ddb_index));

	/*
	 * Send target warm reset command to ISP, so that the ISP
	 * will return all outstanding requests with RESET status
	 */
	mbox_cmd[0] = MBOX_CMD_TARGET_WARM_RESET;
	mbox_cmd[1] = ddb_entry->fw_ddb_index;
	mbox_cmd[2] = RESET_TARGET_DELAY;

	if (qla4000_mailbox_command(ha, 3, 1, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS)
	{
		QL4PRINT(QLP5|QLP2, printk("qla4000_warm_reset_target: MBOX_CMD_TARGET_WARM_RESET failed w/ status %04x\n", mbox_sts[0]));
		status = QLA_ERROR;
	}
	QL4PRINT(QLP5, printk("qla4000_warm_reset_target: MBOX_CMD_TARGET_WARM_RESET completed successfully\n"));

	/* Send marker to unlock queues */
	 qla4000_send_marker(ha, ddb_entry, 0, MM_TARGET_WARM_RESET);

	LEAVE("qla4000_warm_reset_target");
	return(status);
}

int
qla4000_eh_wait_for_pending_target_commands(scsi_qla_host_t *ha, int t)
{
        int     cnt;
        int     status;
        unsigned long   flags;
        srb_t           *sp;
        Scsi_Cmnd       *cmd;

        status = 0;
	ENTER("__func__");
        /*
         * Waiting for all commands for the designated target in the active
         * array
         */
        for (cnt = 1; cnt < QLA4000_MAX_CMDS_TO_RISC; cnt++) {
                spin_lock_irqsave(&ha->hardware_lock, flags);
                sp = ha->active_srb_array[cnt];
                if (sp) {
                        cmd = sp->cmd;
                        spin_unlock_irqrestore(&ha->hardware_lock, flags);
                        if (cmd->target == t) {
					
                                if(!qla4000_eh_wait_on_command(ha, cmd)){
						status++;
						break;
				}
                        }
		}
                else {
                        spin_unlock_irqrestore(&ha->hardware_lock, flags);
                }
        }
	printk("%s return_status=%d\n",__func__,status);
	LEAVE("__func__");
        return (status);
}

/*
 * qla4000_eh_device_reset
 *   This function is called by the Linux OS to reset a specific
 *   target/lun.
 */
int
qla4000_eh_device_reset(Scsi_Cmnd *cmd)
{
	int ret_status = SUCCESS;
	scsi_qla_host_t *ha;
	ddb_entry_t     *ddb_entry;
	srb_t *srb;

	ENTER("qla4000_eh_device_reset");

	ha = (scsi_qla_host_t *) cmd->host->hostdata;

	srb = (srb_t *) CMD_SP(cmd);

	printk(KERN_INFO "scsi %d:%d:%d:%d: device reset\n",
		       ha->host_no, cmd->channel, cmd->target, cmd->lun);

	qla4000_set_extended_debug_level();

	ddb_entry = qla4000_lookup_ddb_by_SCSIID(ha, cmd->channel, cmd->target);
	if (ddb_entry)
	{
		if(qla4000_reset_target(ha, ddb_entry, 0) != QLA_SUCCESS)
			ret_status = FAILED;
	}
	/* restore debug level */
	if (ret_status == SUCCESS)
		qla4000_rst_extended_debug_level();

	QL4PRINT(QLP2, printk("qla4000_eh_device_reset: return with status = %x\n", ret_status));
	LEAVE("qla4000_eh_device_reset");

	#if STOP_ON_RESET_DEVICE
	qla4000_panic("qla4000_eh_device_reset", ha);
	#endif
	return(ret_status);
}

int
qla4000_eh_bus_reset(Scsi_Cmnd *cmd)
{
	uint8_t status = QLA_SUCCESS;
	int     ret_status;
	struct list_head *ptr, *next; /* used for traversing ddb_list */
	scsi_qla_host_t *ha;
	ddb_entry_t *ddb_entry;
	srb_t *srb;

	ENTER("qla4000_eh_bus_reset");


	ha = (scsi_qla_host_t *) cmd->host->hostdata;
	printk(KERN_INFO "scsi %d:%d: bus reset\n", ha->host_no, cmd->channel);

	srb = (srb_t *) CMD_SP(cmd);

#if 0
	spin_unlock_irq(&io_request_lock);
#endif
	qla4000_set_extended_debug_level();

	/* Attempt to reset all valid targets with outstanding commands */
	list_for_each_safe(ptr, next, &ha->ddb_list)
	{
		ddb_entry = list_entry(ptr, ddb_entry_t, list_entry);

		if (ddb_entry)
		{
			QL4PRINT(QLP5, printk("qla4000_eh_bus_reset: reset target b%d, t%x, index %x\n",
					      ddb_entry->bus, ddb_entry->target, ddb_entry->fw_ddb_index));

			/* Issue a warm reset */
			status |= qla4000_reset_target(ha, ddb_entry, 0);
		}
	}

	/*
	 * Status is QLA_SUCCESS if target resets for ALL devices completed successfully.
	 * Otherwise the status is QLA_ERROR.
	 */
	if (status == QLA_SUCCESS)
	{
		printk(KERN_INFO "scsi %d:%d: bus reset SUCCEEDED\n", ha->host_no, cmd->channel);

		/* restore debug level */
		qla4000_rst_extended_debug_level();
		ret_status = SUCCESS;
	}
	else
	{
		printk(KERN_INFO "scsi %d:%d: bus reset FAILED\n", ha->host_no, cmd->channel);
		ret_status = FAILED;
	}

	QL4PRINT(QLP2, printk("qla4000_eh_bus_reset: return with status = %x\n", ret_status));
	LEAVE("qla4000_eh_bus_reset");

	#if STOP_ON_RESET_BUS
	qla4000_panic("qla4000_eh_bus_reset", ha);
	#endif

#if 0
	spin_lock_irq(&io_request_lock);
#endif

	return(ret_status);
}

uint8_t qla4000_reset_target(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, int flag)
{
	uint8_t status = QLA_SUCCESS;
	uint8_t scsi_status;
	uint8_t target = ddb_entry->target;

	spin_unlock_irq(&io_request_lock);

	/* Reset the device */
	if (flag)
		status = qla4000_cold_reset_target(ha, ddb_entry);
	else
		status = qla4000_warm_reset_target(ha, ddb_entry);

	if ( status == QLA_SUCCESS)
	{
		printk(KERN_INFO "scsi %d:%d: device reset SUCCEEDED\n",
			       ha->host_no, target);
		printk("scsi %d:%d: device reset SUCCEEDED\n",
			       ha->host_no, target);
	}
	else
	{
		printk(KERN_INFO "scsi %d:%d: device reset FAILED\n",
			       ha->host_no, target);
	 	status = QLA_ERROR;
	}
	spin_lock_irq(&io_request_lock);

	/* Waiting for all commands to complete for the device */
        if (qla4000_eh_wait_for_pending_target_commands(ha,target))
	 	status = QLA_ERROR;

	/*Wait for the device to become ready */
	spin_unlock_irq(&io_request_lock);
	set_current_state(TASK_INTERRUPTIBLE);
	schedule_timeout(5*HZ);

	/* Send a test unit ready to device */
	printk(KERN_INFO "scsi %d:%d: Sending TUR to device after target reset.\n",
	       ha->host_no, target);

	if (test_bit(AF_INIT_DONE, &ha->flags) && status == QLA_SUCCESS)
            qla4000_send_test_unit_ready_cmd(ha, target, 0, &scsi_status);

	spin_lock_irq(&io_request_lock);
	/* printk("scsi %d:%d: TUR scsi return status = 0x%x\n",
	       ha->host_no, target, scsi_status); */
	return( status );
}


/*
 * Note: Caller should make sure that 
 *	 Hardware lock,io_request_lock,adapter_lock,lun_lock,
 *       are released before this calling routine.
 */

STATIC void
qla4000_flush_active_srbs(scsi_qla_host_t *ha)
{
	srb_t    *srb;
	int      i;
	unsigned long flags;

	ENTER("qla4000_flush_active_srbs");

	spin_lock_irqsave(&ha->hardware_lock, flags);
	for (i = 1; i < QLA4000_MAX_CMDS_TO_RISC; i++)
	{
		if ((srb = ha->active_srb_array[i]) != NULL)
		{
			QL4PRINT(QLP5, printk("qla4000_flush_active_srbs: found srb %p in active array, returning\n", srb));
			del_from_active_array(ha, i);
			CMD_RESULT(srb->cmd) = (int) (DID_RESET << 16);
			add_to_done_srb_q(ha,srb);
		}
	}
	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	if (!list_empty(&ha->done_srb_q))
	{
/*printk("%s: %d srbs in done_q to be completed\n",
		       __FUNCTION__, ha->done_srb_count); */
		while ((srb = del_from_done_srb_q_head(ha)) != NULL)
			qla4000_complete_request(ha, srb);
	}

	LEAVE("qla4000_flush_active_srbs");
}


int
qla4000_eh_host_reset(Scsi_Cmnd *cmd)
{
	int ret_status;
	scsi_qla_host_t *ha;
	ENTER("qla4000_eh_host_reset");


	ha = (scsi_qla_host_t *) cmd->host->hostdata;
	printk("scsi: host reset %d\n", (int)ha->host_no);

	spin_unlock_irq(&io_request_lock);
	if (qla4000_recover_adapter(ha, 0) == QLA_SUCCESS)
		ret_status = SUCCESS;
	else
		ret_status = FAILED;

	QL4PRINT(QLP2, printk("qla4000_eh_host_reset: return with status = %x\n", ret_status));
	LEAVE("qla4000_eh_host_reset");

	spin_lock_irq(&io_request_lock);

	return(ret_status);
}


/****************************************************************************/
/*                QLogic QLA4000 Hardware Support Functions.                */
/****************************************************************************/

/*
* qla4000_enable_intrs
*      Clearing bit enables the PCI interrupt request.
*
* qla2100_disable_intrs
*	   Setting bit disables the PCI interrupt request.
*
* Input:
*      ha = adapter block pointer.
*
* Returns:
*      None
*/
static inline void qla4000_enable_intrs(scsi_qla_host_t *ha)
{
	ISP_REG_T *reg = ha->base_address;
	unsigned long flags = 0;

	ENTER("qla4000_enable_intrs");
	spin_lock_irqsave(&ha->hardware_lock, flags);
	set_bit(AF_INTERRUPTS_ON, &ha->flags);
	WRT_REG_DWORD(&reg->interruptMaskOut, ~((uint32_t)INTR_PENDING));
	QL4PRINT(QLP7, printk("qla4000_enable_intrs: adapter %d, interruptMaskOut = %08x\n",
			      ha->devnum, RD_REG_DWORD(&reg->interruptMaskOut)));
	spin_unlock_irqrestore(&ha->hardware_lock, flags);
	LEAVE("qla4000_enable_intrs");
}

static inline void qla4000_disable_intrs(scsi_qla_host_t *ha)
{
	ISP_REG_T *reg = ha->base_address;
	unsigned long flags = 0;

	ENTER("qla4000_disable_intrs");
	spin_lock_irqsave(&ha->hardware_lock, flags);
	clear_bit(AF_INTERRUPTS_ON, &ha->flags);
	WRT_REG_DWORD(&reg->interruptMaskOut, (uint32_t) INTR_PENDING);
	QL4PRINT(QLP7, 
		 printk("%s: adapter %d, intrMaskOut = %08x\n",
			__FUNCTION__, ha->devnum, 
			RD_REG_DWORD(&reg->interruptMaskOut)));
	spin_unlock_irqrestore(&ha->hardware_lock, flags);
	LEAVE("qla4000_disable_intrs");
}



scsi_qla_host_t *
qla4000_get_adapter_handle(uint16_t instance)
{
	scsi_qla_host_t *ha;

	for (ha = qla4000_hostlist; ha != NULL; ha = ha->next)
	{
		if (ha->instance == instance)
		{
			QL4PRINT(QLP10, printk("qla4000_get_adapter_handle: handle (%p) for instance %d\n", ha, instance));
			return(ha);
		}
	}

	QL4PRINT(QLP1, printk("qla4000_get_adapter_handle: not found\n"));
	return(ha);
}

inline uint32_t qla4000_get_hba_count(void)
{
	return(qla4000_hba_count);
}

/*
 * Scsi_Host_template (see drivers/scsi/hosts.h)
 * Device driver Interfaces to mid-level SCSI driver.
 *----------------------------------------------------------------------------*/

#define QLA4000_LINUX_TEMPLATE {		                	\
	next: 			NULL,					\
	module: 		NULL,					\
	proc_dir: 		NULL,					\
	proc_info: 		qla4000_proc_info,			\
	name: 			"QLogic iSCSI 4000",        		\
	detect: 		qla4000_detect,				\
	release: 		qla4000_release,			\
	info: 			qla4000_info,				\
	ioctl: 			qla4000_ioctl,				\
	command: 		NULL,					\
	queuecommand: 		qla4000_queuecommand,			\
	eh_strategy_handler: 	NULL,					\
	eh_abort_handler: 	qla4000_eh_abort,	       			\
	eh_device_reset_handler: qla4000_eh_device_reset,		\
	eh_bus_reset_handler: 	 qla4000_eh_bus_reset, 			\
	eh_host_reset_handler: 	qla4000_eh_host_reset,			\
	abort: 			NULL,    				\
	reset: 			NULL,     				\
	slave_attach: 		NULL,					\
	bios_param: 		qla4000_biosparam,			\
	can_queue: 		REQUEST_QUEUE_DEPTH/4,\
	this_id: 		-1,	/* scsi id of host adapter    */\
	sg_tablesize: 		32, /* max scatter-gather cmds    */\
	max_sectors:		0,					\
	cmd_per_lun: 		MAX_LINKED_CMDS_PER_LUN, /* linked cmds per lun */\
	present: 		0,	/* number of 4000's present   */\
	unchecked_isa_dma: 	0,	/* no memory DMA restrictions */\
	use_clustering: 	ENABLE_CLUSTERING,			\
	use_new_eh_code: 	1,					\
	emulated: 		0,					\
	proc_name: 		NULL				        \
};

/*
 * We initialize the driver via the linux kernel loadable module interface
 * by defining and initializing the variable 'driver_template' and including
 * the file scsi_module.c. All routines get called automatically.
 */
Scsi_Host_Template driver_template = QLA4000_LINUX_TEMPLATE;
#include <../drivers/scsi/scsi_module.c>

/*
 * Overrides for Emacs so that we almost follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 2
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -2
 * c-argdecl-indent: 2
 * c-label-offset: -2
 * c-continued-statement-offset: 2
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */



